
// *****************************************************************************
//
// source:    AC_evnt.C
//
// type:      source code
//
// created:   16 Sep 2000
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// symbol:    AC_event()
//
// called by: acwrite()
//
// hints:
//
// changes:
// 03 Nov 1999 (AG) add RICH
// 19 Nov 1999 (AG) cna -> ac (alca)
// 22 Nov 1999 (AG) includes in alca
// 12 Jan 2000 (AG) invoke gageEvnt();
// 27 Feb 2000 (AG) introduce alca/AC.h
// 21 Mar 2000 (AG) use modified AC_flag()
// 25 Apr 2000 (AG) use ARTE_mark()
// 18 May 2000 (AG) introduce AC_ECAL_evnt()
// 16 Sep 2000 (AG) added AC_OTR
//  6 Dec 2000 (AG) added timer
//  8 Dec 2000 (AG) introduced AC_ECAL_PRE 
// 19 Dec 2000 (AG) added AC_LUMI
//
// *****************************************************************************

// standard includes
#include <iostream.h>

// ONLINE includes

// ARTE includes
#include "arte/ArteProto.hh"
#include "arte/Chrono.hh"
#include "arte/ChronoManager.hh"
#include "farm/farm.hh"
#include "alca/AC.h"
#include "alca/AC_SVD.hh"
#include "alca/AC_OTR.hh"
#include "alca/AC_RICH.hh"
#include "alca/AC_ECAL.h"
#include "alca/AC_ECAL_PRE.hh"
#include "alca/AC_LUMI.hh"

// externals
extern int FADEB;  // debug flag (runargs=-debug)
extern int FATIME; // timing

// prototypes
extern "C" void cnakuip_();
int gageEvnt();

//
// initialize kuip commands
//
static class ACIniter
{
public:
  ACIniter() { atregkuip( cnakuip_, 1 ); } 
} _cnaIniter;

//
// program code
//
int
AC_evnt()
{
  ARTE_mark( "AC_evnt 0", 0 );

  // start timing
  static Chrono *cron_AC_evnt = 0;
  if( !cron_AC_evnt ){
    ChronoManager CM;
    cron_AC_evnt = CM.createChrono( "cron_AC_evnt" );
  }
  cron_AC_evnt->start();

  // exec
  int rc = 0;

  // SVD
  if( AC_flag( COMP_SVD ) ){
    ARTE_mark( "AC_SVD_evnt", 0 );
    AC_SVD_evnt();
  }

  // OTR
  if( AC_flag( COMP_OTR ) ){
    ARTE_mark( "AC_OTR_evnt", 0 );
    AC_OTR_evnt();
  }

  // RICH
  if( AC_flag( COMP_RICH ) ){
    ARTE_mark( "AC_RICH_evnt", 0 );
    AC_RICH_evnt();
  }

  // ECAL
  if( AC_flag( COMP_ECAL ) ){
    ARTE_mark( "AC_ECAL_evnt", 0 );
    AC_ECAL_evnt();
  }

  // GLOBAL
  if( AC_flag( COMP_GLOB ) ){
    ARTE_mark( "gageEvnt", 0 );
    rc = gageEvnt();
  }

  // ECAL PRE-trigger
  if( AC_flag( COMP_FLT ) ){
    ARTE_mark( "AC_ECAL_PRE_evnt", 0 );
    AC_ECAL_PRE_evnt();
  }

  // LUMI
  if( AC_flag( COMP_LUMI ) ){
    ARTE_mark( "AC_LUMI_evnt", 0 );
    AC_LUMI_evnt();
  }

  // stop timing
  cron_AC_evnt->stop();
  float cputime = cron_AC_evnt->lastTime();
  if( FATIME ) printf( "\033[1m TIME: AC=%4.2f \033[0m \n", cputime );
  
  // return
  ARTE_mark( "AC_evnt 1", 0 );
  return 0;
}
