// ********************************************************************
//
//  Routine:	alca/actaxxSelectITR.C
//  Author:		Torsten Zeuner
//
//  Description:	ITR-specific TAXX selection function,
//				called from actaxxevnt.C
//
//  Modifications:
//   -- none yet					06/12/00 DG
//   ITR Track selection                                18/12/00 Torsten Zeuner
//
// ********************************************************************

#include "alca/actaxxComm.hh"
#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include <vector.h>
#include "alca/actaxxComm.hh"
#include "alca/actaxxFLG.hh"

int gageTraceTRACK(float , float *, float *);

void actaxxSelectITR() {

  int   nhits_mc    = taCommITR.nMC;
  int   nhits_pc    = taCommITR.nPC;
  //	int   nhits_tc    = taCommITR.nTC;	//DG: unused warning
  float zfpoint     = taCommITR.zcut;
  float pmin        = taCommITR.pcut;
  float dxmin       = taCommITR.xcut;
  float dxexc       = taCommITR.xexc;

  int mc_hits, pc_hits;
  int accept;
  float pin[6], pout[6];
  float trackx, tracky;

  vector< ArtePointer<HITB> > hit;

  for ( ArteTable<RSEG>::iterator rsegIt = ArteTable<RSEG>::begin();
	rsegIt != ArteTable<RSEG>::end(); ++rsegIt ) {

    ArtePointer<RSEG> rseg(*rsegIt);

    if ( (*rsegIt)->cmp & Rsegc::bitsign
	 || !( (*rsegIt)->cmp & Rsegc::msg  )
	 || !( (*rsegIt)->cmp & Rsegc::patt )
	 || (*rsegIt)->fit )	continue;

    float pinv = (rseg->txe * rseg->ze - rseg->xe) / 280.;

    if ((rseg->zf < zfpoint) && (fabs(pinv) < 1./pmin)) {
     
      mc_hits = 0;
      pc_hits = 0;
   
      for( ArteRelationIterator<HITB> hitbit = rseg->hitb.begin();
           hitbit != rseg->hitb.end(); hitbit++ ) {

        ArtePointer<HITB> hitb(*hitbit);
        ArtePointer<GEDE> gede = hitb->gede;
        //	ArtePointer<GESL> gesl = gede->gesl;	// DG: unused warning
       
        if ( gede->cmp == 3 ) {

          accept = 1;

          float gasgap = (gede->zmax-gede->zmin)/2;
          float zpos   = gede->zmin + gasgap; 
 
          if ( rseg->zf < 650 ) {

            pin[0] = rseg->zf;
            pin[1] = rseg->xf;
            pin[2] = rseg->yf;
            pin[3] = rseg->txf;
            pin[4] = rseg->tyf;
            pin[5] = rseg->pf;
	    if (rseg->pf == 0) 
              pin[5] = (rseg->txe*rseg->ze-rseg->xe) / 280.;
       
        
            // trace track through magnetic field

            //	int rc = 			/* DG: unused warning */
	    gageTraceTRACK(hitb->z , pin, pout);
          
            trackx = pout[1];
            tracky = pout[2];
          }
          else {

            // first point of rseg outside magnetic field -> linear extrapolation
 
            trackx = rseg->xf + rseg->txf * ( zpos - rseg->zf );
            tracky = rseg->yf + rseg->tyf * ( zpos - rseg->zf );

          }

          // rotation of external point

          float uhitx = trackx * gede->rot[0] - tracky * gede->rot[1];
        
          float dx = uhitx - hitb->x;

          if (fabs(dx)>fabs(dxmin)) accept=0;

          // clean cut

          for (ArteTableIterator<HITB> hitb1it; hitb1it.valid(); hitb1it.next()) {
            ArtePointer<HITB> hitba(*hitb1it);
            ArtePointer<GEDE> geda = hitba->gede;

            if (geda!=gede) continue;
            if (hitb==hitba) continue;

            if (fabs(hitb->x - hitba->x) < fabs(dxexc) ) {
              accept=0;
            }
          }

          if (accept) {
            if (hitb->z < 650.) mc_hits++;
            if (hitb->z >= 650.) pc_hits++;
            hit.push_back(hitb);
          }
        }
      }

      if ( mc_hits >= nhits_mc && pc_hits >= nhits_pc ) {
  
        // track accepted
        // now fill hits to TAXX

        for ( vector<ArtePointer<HITB> >::iterator hitit=hit.begin(); 
             hitit!=hit.end(); hitit++) {

          ArtePointer<HITB> hitb(*hitit);

          int tmp = actaxxAdd(hitb, rseg, TAFLG::itrsel);

	  if ( tmp ==-1	) return;	// Problems with memory allocation to the list
	  if ( tmp == 0	) continue;	// Zero HITB pointer

	  if ( tmp != (int) hitb.index() ) // Something too strange
	  {
            cout << " actaxxSelectITR : Unbelievable, taAdd( " 
                 << hitb.index() << " ) = " << tmp << endl;
	    return;
	  }
	}
      }
    }
  }

  return;
}
