//                              -*- Mode: C++ -*-
//: ECALBoard.C
//
// Author: Mauro Villa   
// ECAL group
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 7/3/98
//
// 20.02.01 internal mapping added (pt full scale)
//          which is taken if the board coverage file name is "default"
//
//



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "care/dgcalinit.hh"
#include "gean/hgdat.h"

int subcal[3][7] = {
  {3, 1, 1, 21, 17, 34, 24}, // inner
  {2, 4, 2,  8,  9, 47, 32}, // middle
  {1, 8, 4,  0,  0, 55, 43}};// outer

//
char innercoverage[]="\
2222BBBBBB2222\
22BBBBBBBBBB22\
BBBBBAAAABBBBB\
BBBBAAHHAABBBB\
BBBBAAHHAABBBB\
BBBBBAAAABBBBB\
22BBBBBBBBBB22\
2222BBBBBB2222";

char middlecoverage[]="\
1111CC1111\
11CCCCCC11\
11CCCCCC11\
1CCCCCCCC1\
CCCCCCCCCC\
CCCC33CCCC\
CCCC33CCCC\
CCCCCCCCCC\
1CCCCCCCC1\
11CCCCCC11\
11CCCCCC11\
1111CC1111";

char outercoverage[]="\
DDDDDDD\
DDDDDDD\
DDDDDDD\
DDD2DDD\
DD222DD\
D22322D\
DD222DD\
DD222DD\
DDDDDDD\
DDDDDDD\
DDDDDDD";



// ------------------------------------------------------------------------
FILE *BCAL_inp, *utfilp(const char*, const char*);  
fpos_t fbcalpos;
// ------------------------------------------------------------------------  
void inp_error(char *routine, char *tag);
// ------------------------------------------------------------------------  

int ECALBoard(char* name, vector<BoardData> & boards,
	      vector<EcalDet> & calos) {
  int nitem, tagid=0;
  char  buffer[256], tag[80], tags[60], C78[120] ;
  char *card;

  if( strcmp(name,"default")==0 ){
    // default board coverage
    // board definitions
    boards.push_back(BoardData('A',50.,1.5, 5.,1,0.01, 0.17)); // ham-inner
    boards.push_back(BoardData('B',50.,2.0,-6.,1,0.01, 0.17)); // feu-inner
    boards.push_back(BoardData('C',50.,3.0,-6.,1,0.02, 0.12)); // feu-middle
    boards.push_back(BoardData('D',50.,3.0,-6.,1,0.015,0.11)); // feu-outer
    //
    char* pc[3];
    pc[0] = innercoverage;
    pc[1] = middlecoverage;
    pc[2] = outercoverage;
    EcalDet cald;
    for(unsigned int i=0; i<3; i++){
      cald.ecaltype = subcal[i][0];
      cald.mstepx = subcal[i][1];
      cald.mstepy = subcal[i][2];
      cald.xmin   = subcal[i][3];
      cald.ymin   = subcal[i][4];
      cald.xmax   = subcal[i][5];
      cald.ymax   = subcal[i][6];
      cald.data[0] = 0;
      strcpy(cald.data,pc[i]);
      calos.push_back(cald);
    }
  } else {
    //  vector<BoardData> boards;
    //  vector<EcalDet> calos;
    
    BCAL_inp =utfilp(name,"");
    if(!BCAL_inp) {
      strcpy(C78,"Can not open ");
      int len=strlen(name);
      for (int ii=len; ii<64; ii++) name[ii]=' ';
      name[64]='\0';
      strncat(C78,name,len);
      printf(" %s \n", C78);      
    }
    
    for(;;) {
      fgetpos(BCAL_inp, &fbcalpos);
      if(fgets(buffer,256,BCAL_inp) == NULL) {
	if(feof(BCAL_inp)) {
	  // Build the tables
	  
	  fclose(BCAL_inp);
	  fprintf(stdout," === CLOSE FILE  %s\n",name);
	  return tagid;                    
	}
      }
      
      card=&buffer[0];
      nitem=sscanf(card,"%s",tag);
      if (nitem==-1) continue;   
      //    ToLower(tag);
      
      if (!strncmp(tag,"board",5)) { 
	BoardData board;
	card+=itemlength(card,nitem)+2;
	//      sscanf(card,"%s", tags);
	//fillstr(tags,60); 
	nitem=sscanf(card,"%c %f %f %f %d %f %f",
		     &board.type,
		     &board.pedChan, &board.sigmaped,
		     &board.mipChan, &board.trigflag,
		     &board.erconst, &board.ersampl);
      // cout<<" Card: "<<card<<endl;
      /*      cout<<"Board type: "<<board.type
	      <<" PED: "<<board.pedChan<<" Sigma: "<<board.sigmaped
	      <<" MIP: "<<board.mipChan<<" Trig: "<<board.trigflag
	      <<" Error: "<<board.ersampl<<"/sqrt(E)+"<<board.erconst<<" %"<<endl;
      */
	boards.push_back(board); 
	if(nitem!=7) inp_error("board.dat",tag);  }
      
      else if( !strncmp(tag,"inner",5) || !strncmp(tag,"middle",6) ||
	       !strncmp(tag,"outer",5) ){
	EcalDet cald;
	if( !strncmp(tag,"inner",5)  ) cald.ecaltype = 3;
	if( !strncmp(tag,"middle",6) ) cald.ecaltype = 2;
	if( !strncmp(tag,"outer",5)  ) cald.ecaltype = 1;
	card+=itemlength(card,nitem);
	nitem=sscanf(card,"%d %d %d %d %d %d", 
		     &cald.mstepx,  &cald.mstepy,
		     &cald.xmin, &cald.ymin,
		     &cald.xmax, &cald.ymax);
	cald.data[0] = 0;
	calos.push_back(cald);
	if(nitem!=6) inp_error("board.dat",tag);  }
      
      else if (!strncmp(tag,"cast",4)) {
	EcalDet* calr = calos.end()-1;
	if( calr->data[0]==0){
	  int sx = calr->xmax-calr->xmin+1;
	  int sy = calr->ymax-calr->ymin+1;
	  if( sx<=0 || sy<=0 || calr->mstepx<=0 || calr->mstepy<=0)
	    inp_error("board.dat",tag);
	  int nx = sx/calr->mstepx;
	  int ny = sy/calr->mstepy;
	  // create the data buffer
	  // unsigned int ndati = nx*ny;
	  // calr->data = new char[ndati];
	  calr->data[0] = 0;
	  card+=itemlength(card,nitem);            
	  nitem=sscanf(card,"%s",tags); 
	  
	  int j = 1;
	  fillstr(tags,nx);      
	  strncpy(calr->data,tags,nx);
	  calr->data[nx] = 0;
	  while (j <= ny) {
	    if(fgets(buffer,256,BCAL_inp) == NULL) inp_error("caldat",tags);
	    else  {
	      card=&buffer[0];
	      nitem=sscanf(card,"%s",tags);
	      fillstr(tags,nx);
	      strcat(calr->data,tags);
	      j++; }  } 
	} else {
	  inp_error("board.dat",tag); 
	}
      }       
      else  {
	if(strncmp(tag,"#",1) &&  strncmp(tag,"*",1))
	  printf("tag not found:   %s\n",tag);    }
    } 
  }
  return tagid;
}









