//                              -*- Mode: C++ -*-
//:ECDbNoise.C
//
//
// Author: Mauro Villa
// ECAL group
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//

#include "care/ECDbNoise.hh"
#include <algorithm>


char * ECDbNoise::comment = "Noise correction data";
char * ECDbNoise::fieldDesc = 
  "F:chancorr F:brdsigma I:nbdcor I:nfd F:fdcorr";
const int ECDbNoise::fieldType[] = {TABLEfloat, TABLEfloat, 
				    TABLEint, TABLEint, TABLEfloat};

void ECDbNoise::init(){
  if( chancorr.size()!=ECALCHAN){
    chancorr.erase(chancorr.begin(), chancorr.end());
    for(int i=0; i<ECALCHAN; i++){
      chancorr.push_back(0.0);
    }
  }
  if( boardsigma.size()!=ECALBOARDS){
    vector<ECFedCor> empty(0);
    boardsigma.erase(boardsigma.begin(), boardsigma.end());
    fedcorrs.erase  (fedcorrs.begin(),   fedcorrs.end());
    for(int i=0; i<ECALBOARDS; i++){
      boardsigma.push_back(0.0);
      fedcorrs.push_back(empty);
    }
  }
}

// Database access functions
int ECDbNoise::openDB(const char * dbname, 
			    const char *tablename, int access){

  if( size()!=ECALBOARDS ) init();

  int rc = 0;
  x=OpenTables((char*)dbname, access);
  if(x!=0){
    tab = BookTable(x, (char*)tablename, ECDbNoise::fieldDesc, 
		    ECDbNoise::comment);
    if( tab==0) rc = 2;
  } else {
    rc =1 ;
  }
  return rc;
}

int ECDbNoise::writeToDB(int maj, int min, int access){
  
  major = maj;
  minor = min;
  TableStamp(x,maj,min);
  SkipTableField(x,1);
  // chan correl 
  PutTableField(x,ECDbNoise::fieldType[0],
		ECALCHAN, (char *) &chancorr[0]);
  PutTableField(x,ECDbNoise::fieldType[1],
		ECALBOARDS, (char *) &boardsigma[0]);

  int sizes[ECALBOARDS];
  int ncorrs = 0;
  for(int i=0; i<ECALBOARDS; ++i){
    sizes[i] = fedcorrs[i].size();
    ncorrs+= fedcorrs[i].size();
  }
  int* fds = new int[ncorrs];
  float * fdcr = new float[ncorrs];
  
  unsigned int k=0;
  for(unsigned int i=0; i<ECALBOARDS; ++i){
    for(unsigned int j=0; j<fedcorrs[i].size(); ++j){
      fds[k] = fedcorrs[i][j].fed;
      fdcr[k] = fedcorrs[i][j].corr;
      k++;
    }
  }
      
  PutTableField(x,ECDbNoise::fieldType[2],
		ECALBOARDS, (char*) sizes);

  PutTableField(x,ECDbNoise::fieldType[3],
		ncorrs, (char*) fds);
  PutTableField(x,ECDbNoise::fieldType[4],
		ncorrs,	(char*) fdcr);

  delete [] fds;
  delete [] fdcr;

  if(!StoreTable(x,tab,access)) return -1;
  major=TableMajor(x);
  minor=TableMinor(x);

  return 0;
}

int ECDbNoise::readFromDB(int maj, int min, int access){
  int rc = 0;
  TableStamp(x,maj,min);
  if(FetchTable(x, tab, access)){
    if( size()!=ECALBOARDS ) init();
    
    SkipTableField(x,1);
    int sizes[ECALBOARDS];
    // chan correl 
    GetTableField(x,ECDbNoise::fieldType[0],
		  ECALCHAN,  (char *) &chancorr[0]);
    GetTableField(x,ECDbNoise::fieldType[1],
		  ECALBOARDS, (char *) &boardsigma[0]);
    GetTableField(x,ECDbNoise::fieldType[2],
		  ECALBOARDS, (char *) sizes);

    int ncorrs = 0;
    for(int i=0; i<ECALBOARDS; ++i){
      ncorrs+= sizes[i];
    }

    int* fds = new int[ncorrs];
    float * fdcr = new float[ncorrs];
  
    GetTableField(x,ECDbNoise::fieldType[3],
		  ncorrs, (char *) fds);
    GetTableField(x,ECDbNoise::fieldType[4],
		  ncorrs, (char *) fdcr);

    int k=0;
    for(int i=0; i<ECALBOARDS; ++i){
      fedcorrs[i].erase(fedcorrs[i].begin(), fedcorrs[i].end());
      for(int j=0; j<sizes[i]; ++j){
	fedcorrs[i].push_back(ECFedCor(fds[k], fdcr[k]));
	k++;
      }
    }
      
    delete [] fds;
    delete [] fdcr;
    
    major = TableMajor(x);
    minor = TableMinor(x);
    rc = ECALBOARDS;
  } else {
    rc = -1;
  }
  return rc;
}

int ECDbNoise::closeDB(){
  int ret1=UnbookTable(x, tab);
  tab=0;
  int ret2=CloseTables(x);
  x=NULL;
  return ((ret1==1) || (ret2==1))? 0: 1;
}




