//                              -*- Mode: C++ -*-
//:ECdbmc.hh
//
//
// Author: Mauro Villa
// ECAL group
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//

#ifndef ecdbmc_hh
#define ecdbmc_hh

extern "C" {
  #include "database/table.h"
}

#include <vector>
#include <stdio.h>
#include <string>
#include <fstream>
#include <ctime>

struct ECEffBoardEntry {
  int stat;
  float eff1;
  float eff2;

  // db related information
  static const char * fieldDesc;
  static const int fieldType[];
  static const int fieldDim[];
  static const char * comment;


  inline ECEffBoardEntry(): stat(0), eff1(1.0), eff2(1.0){};
  inline ECEffBoardEntry(int astat, float aeff1, float aeff2) :
    stat(astat), eff1(aeff1), eff2(aeff2) {};

  inline ECEffBoardEntry(const ECEffBoardEntry & aBoard) :
    stat(aBoard.stat), eff1(aBoard.eff1), eff2(aBoard.eff2) {};
  inline ~ECEffBoardEntry(){};
  inline const ECEffBoardEntry &  operator=(const ECEffBoardEntry & aBoard){
    stat = aBoard.stat;  eff1 = aBoard.eff1; eff2 = aBoard.eff2;
    return *this;
  }

  // read/write ascii data
  //  int readLine(std::istream & inf);
  //  int writeLine(std::ostream & of, int i) const;
};

class ECEffBoardArray {
private:
  //
  // The main vector of efficiencies
  //
  vector<ECEffBoardEntry> eftable;
  // DB specific entries
  int major;
  int minor;
  int tab;
  Tables x;

public:
  enum {ECALBOARDS=226, FEDMIN=16}; // number of ECAL RO boards

  ECEffBoardArray() : eftable(ECALBOARDS), major(0), minor(0), tab(0), x(0){};
  ~ECEffBoardArray(){}

  // Database access functions
  // Return codes: rc=0 OK! rc>0 error
  int openDB(const char * dbname, const char *tablename, int access=0);
  int writeToDB(int major, int minor, int access);
  int readFromDB(int major, int minor, int entries);

  // Return codes: rc=0 OK! rc>0 error
  int closeDB();

  // access functions
  Tables db() const { return x;}
  int    table() const {return tab;}
  int    majorTag() const { return major;}
  int    minorTag() const { return minor;}

  //
  // Iterators
  //
  const ECEffBoardEntry* begin() const {
    return eftable.begin();
  };
  ECEffBoardEntry* begin() { return eftable.begin();}
  const ECEffBoardEntry* end() const { return eftable.end();}
  ECEffBoardEntry* end() { return eftable.end();}
  //
  // Access/set
  //
  inline int store(int fed, const ECEffBoardEntry & entry){
    int rc=0;
    if( fed>=FEDMIN && fed<ECALBOARDS+FEDMIN){ 
      eftable[fed-FEDMIN] = entry; rc =1;
    }
    return rc;
  }
  
  inline ECEffBoardEntry & entry(int fed){
    if( fed>=FEDMIN && fed<ECALBOARDS+FEDMIN ) 
      return eftable[fed-FEDMIN];
    return eftable[0];
  }

  //
  // Size of the array ( should be 226 or 0)
  //
  inline unsigned int size() const { return eftable.size();}

};

// *********************************************


#endif
