//                              -*- Mode: C++ -*-
//:ELCellGrid.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 9/4/97
// Last Update ........
//
#ifndef ELCELLGRID_HH_
#define ELCELLGRID_HH_


#include "care/ELLUTsManager.hh"

/**
class  ELCellGrid.
  In this LUT of dimension nModX*nModY it is stored for each module of ecal
  the index of the first cell contained in the module. Access to the data
  are possible on an X,Y basis and on an IX,IY basis.
WARNING 
  This LUT is initialized only by the LUTsManager

*/

class ELCellGrid  {
public:
  //--
  ///  Returned error codes.
  enum ErrorTypes {OK = 0, HOLE, OUTECAL, AGAIN=100};

  //--
  ///  Detail of constructions left to LUTsManager
  ELCellGrid();

  //--
  ///  Get the cell ID given a point on ECAL
  ECCellID cellIDen(float x, float y, ErrorTypes & ierr) const;
  //--
  ///  Get the module number in which a point on ECAL is
  ArtePointer<GCAL> module(float x, float y, ErrorTypes & ierr) const;
  //--
  ///  Get the cell index in which a point on ECAL is
  CellIdx_t cellIdx(float x, float y, ErrorTypes & ierr) const;

  //--
  ///  Get the index of the first cell in a module
  CellIdx_t  lookUp(int ix, int iy);
  //--
  ///  Get the index of the first cell in a module
  CellIdx_t lookUp(int ix, int iy) const;

  //--
  ///  Set the index of the first cell in a module
  void set(int ix, int iy, CellIdx_t ncell);
  //--
  ///  Declare friend clas LUTsManager that is in charge of initialize this LUT
  friend class ELLUTsManager;

  //--
  ///  Find IX and IY and return an error flag
  CellIdx_t findIXIY(float x,float y,int & ix, int & iy, 
		     ErrorTypes & ierr) const;

private:

  //--
  ///  reserve space for entries
  void reserve(int size);

  //--
  ///  Get the cell index in which a point on ECAL is
  CellIdx_t cellIdxBase(float x, float y, int & ix, int & iy,
				ErrorTypes & ierr) const;

};


#endif // ELCELLGRID_HH_
