using namespace std;

#include <arte/arte.hh>
#include <vector>
#include <map>
#include <algorithm>
#include <fstream.h>
#include "cats/OtrSPoint.hh"
#include "cats/hbook.h"

extern "C"{
  float prob_(const float & ccchi2, const int & ndeg);
}

static const float tan_max  = 1.0;
static const float y_extra  = 5.0;
static const float dy_max   = 10.0;
static const float chi2_cut = 10.0;
static const float qp_max   = 1.0;

static int propagate_to_m = 0;

static const double B     =   0.65945;
static const double zmagn = 450.0;

static float geslz1 = 0.0, geslz2 = 0.0;

bool OtrTripletCompareDist(const TRIPLET &a, const TRIPLET &b){
  if ((a.dist < 1000.0)||(b.dist < 1000.0))
    return (a.dist < b.dist);

  return (fabs(a.x - (a.tx*geslz1)) < fabs(b.x - (b.tx*geslz1)) );
}

bool OtrStereoCompareY(const YSTEREO &a, const YSTEREO &b){
  return (a.y < b.y);
}



void OtrSPoints(vector<pCLUSTER> vzpcluster, vector<pCLUSTER> vspcluster, 
		vector<SPACEPOINT>& vspoint,
		const float symn, const float symx){

  if ((distance(vzpcluster.begin(), vzpcluster.end())==0)||
      (distance(vspcluster.begin(), vspcluster.end())==0)) return;

  propagate_to_m = 1;

  float dist_max = (*(vspcluster.begin()))->gede->pit;

  TRIPLET         triplet;
  vector<TRIPLET> vtriplet;
  vector<YSTEREO> vystereo;

  ArtePointer<GESL> gesl = (*(vspcluster.begin()))->gede->gesl;
  geslz1 = gesl->z1;
  geslz2 = gesl->z2;

  for (vector<pCLUSTER>::iterator pclu1It = vzpcluster.begin();
       pclu1It != vzpcluster.end(); ++pclu1It){
    float x1 = (*pclu1It)->x;
    float z1 = (*pclu1It)->z;

    vector<pCLUSTER>::iterator pclu2_begin = pclu1It; pclu2_begin++;

    for (vector<pCLUSTER>::iterator pclu2It = pclu2_begin;
	 pclu2It != vzpcluster.end(); ++pclu2It){
      if ((*pclu2It)->lay == (*pclu1It)->lay) continue; // the same layer
      float x2 = (*pclu2It)->x;
      float z2 = (*pclu2It)->z;

      if (fabs((x2-x1)/(z2-z1)) > tan_max) break;
      //if (fabs(x1 - ((x2-x1)/(z2-z1)*z1))*0.003474276 > qp_max) break;

      vector<pCLUSTER>::iterator pclu3_begin = pclu2It; pclu3_begin++;
      
      for (vector<pCLUSTER>::iterator pclu3It = pclu3_begin;
	   pclu3It != vzpcluster.end(); ++pclu3It){
	if (((*pclu3It)->lay == (*pclu1It)->lay)||
	    ((*pclu3It)->lay == (*pclu2It)->lay)  ) continue; // the same layer
	float x3 = (*pclu3It)->x;
	float z3 = (*pclu3It)->z;

	if (fabs((x3-x2)/(z3-z2)) > tan_max) break;
	//if (fabs(x2 - ((x3-x2)/(z3-z2)*z2))*0.003474276 > qp_max) break;
	
	CLUSTER *pclul = *pclu1It;
	CLUSTER *pclum = *pclu2It;
	CLUSTER *pclur = *pclu3It;
	CLUSTER *pclu;
	if (pclum->z < pclul->z) {pclu = pclul; pclul = pclum; pclum = pclu;}
	if (pclur->z < pclul->z) {pclu = pclul; pclul = pclur; pclur = pclu;}
	if (pclur->z < pclum->z) {pclu = pclum; pclum = pclur; pclur = pclu;}

	float tx   = (pclur->x - pclul->x)/(pclur->z - pclul->z);
	float dist = fabs(pclum->x - (pclul->x + tx*(pclum->z - pclul->z)) );
	
	if (dist < dist_max){ // store triplet

	  float dzml, dzrm, dzrl;
	  dzml = pclum->z - pclul->z; dzrm = pclur->z - pclum->z; dzrl = pclur->z - pclul->z;
	  triplet.tx = ((pclum->x - pclul->x)*dzml + (pclur->x - pclum->x)*dzrm +
			 (pclur->x - pclul->x)*dzrl)/(dzml*dzml + dzrm*dzrm + dzrl*dzrl);
	  triplet.x  = (pclul->x + pclum->x + pclur->x - 
			triplet.tx*(pclul->z + pclum->z + pclur->z - 3.0*geslz1))/3.0;
	  triplet.dist   = dist;
	  triplet.length = 3;
	  triplet.pclul = pclul; triplet.pclum = pclum; triplet.pclur = pclur;
	  vtriplet.push_back(triplet);

	  pclul->used = 1; pclum->used = 1; pclur->used = 1;
	}
      }
    }
  }

  for (vector<pCLUSTER>::iterator pclu1It = vzpcluster.begin();
       pclu1It != vzpcluster.end(); ++pclu1It){

    if ((*pclu1It)->used){
      (*pclu1It)->used = 0;
      continue;
    }
    float x1 = (*pclu1It)->x;
    float z1 = (*pclu1It)->z;

    vector<pCLUSTER>::iterator pclu2_begin = pclu1It; pclu2_begin++;

    for (vector<pCLUSTER>::iterator pclu2It = pclu2_begin;
	 pclu2It != vzpcluster.end(); ++pclu2It){

      if ((*pclu2It)->used) continue;

      if ((*pclu2It)->lay == (*pclu1It)->lay) continue; // the same layer
      float x2 = (*pclu2It)->x;
      float z2 = (*pclu2It)->z;

      if (fabs((x2-x1)/(z2-z1)) > tan_max) break;

      if (fabs(x1 - ((x2-x1)/(z2-z1)*z1))*0.003474276 > qp_max) break;

      CLUSTER *pclul = *pclu1It;
      CLUSTER *pclur = *pclu2It;
      CLUSTER *pclu;
      if (pclur->z < pclul->z) {pclu = pclul; pclul = pclur; pclur = pclu;}

      triplet.tx = (pclur->x - pclul->x)/(pclur->z - pclul->z);
      triplet.x  = (x1-triplet.tx*z1) + triplet.tx*geslz1;

      triplet.dist   = 100.0; // to separate
      triplet.length = 2;
      triplet.pclul = pclul; triplet.pclur = pclur;
      vtriplet.push_back(triplet);
      
      pclul->used = 1; pclur->used = 1;
    }
    (*pclu1It)->used = 0;
  }
  //

  sort(vtriplet.begin(), vtriplet.end(), OtrTripletCompareDist);

  SPACEPOINT sp;
  YSTEREO    ystereo;
  map<int, int> stereomap;

  map<int, int> tripletmap;
  int ntr = 0;

  for (vector<TRIPLET>::iterator trIt = vtriplet.begin();
       trIt != vtriplet.end(); ++trIt){ // triplet loop

    ntr++;

    vystereo.clear();
    for (vector<pCLUSTER>::iterator pcluIt = vspcluster.begin();
	   pcluIt != vspcluster.end(); ++pcluIt){
      if ((*pcluIt)->used) continue;
      float sina = (*pcluIt)->sina; float cosa = (*pcluIt)->cosa;
      float z = (*pcluIt)->z; float u = (*pcluIt)->x;
      float x = trIt->x + trIt->tx*(z-geslz1);
      float y = (x*cosa - u)/sina;
      float ty = y/z;
      if ((y<symx+y_extra) && (y>symn-y_extra)){
	ystereo.y  = y + ty*(geslz1-z);  // left border of gesl !!!
	ystereo.ty = ty;
	ystereo.z  = z;
	ystereo.psclu = *pcluIt;
	ystereo.used  = 0;
	vystereo.push_back(ystereo);
      }
    }
    if (distance(vystereo.begin(),vystereo.end()) == 0) continue;

    sort(vystereo.begin(),vystereo.end(),OtrStereoCompareY);

    int nstereotriplets = 0;
    //create stereo triplets
    for (vector<YSTEREO>::iterator pclu1It = vystereo.begin();
	 pclu1It != vystereo.end(); ++pclu1It){
      pCLUSTER pclu1 = pclu1It->psclu;
      float x1 = pclu1->x;
      float z1 = pclu1->z;

      vector<YSTEREO>::iterator pclu2_begin = pclu1It; pclu2_begin++;
      
      for (vector<YSTEREO>::iterator pclu2It = pclu2_begin;
	   pclu2It != vystereo.end(); ++pclu2It){
	pCLUSTER pclu2 = pclu2It->psclu;
	if (pclu2->lay == pclu1->lay) continue; // the same layer

	//if (fabs(pclu2It->y - pclu1It->y) > dy_max) continue;
	// around a pointer to the target
        if (fabs(pclu2It->y - pclu1It->y)/fabs(pclu2It->z - pclu1It->z)>tan_max) continue;
	//if (fabs(pclu1It->y - ((pclu2It->y-pclu1It->y)/(pclu2It->z-pclu1It->z)*pclu1It->z))*0.003474276 > qp_max) break;


	float z2 = pclu2->z; 
	float u  = pclu2->x;
	float x  = trIt->x + trIt->tx*(z2-geslz1);
	float y  = (x*pclu2->cosa - u)/pclu2->sina;
	float x2 = x*pclu1->cosa - y*pclu1->sina; // recalculate to stereo layer of the first cluster 
	
       	if (fabs((x2-x1)/(z2-z1)) > tan_max) break;
	
	vector<YSTEREO>::iterator pclu3_begin = pclu2It; pclu3_begin++;
	
	for (vector<YSTEREO>::iterator pclu3It = pclu3_begin;
	     pclu3It != vystereo.end(); ++pclu3It){
	  pCLUSTER pclu3 = pclu3It->psclu;
	  if ((pclu3->lay == pclu1->lay)||
	      (pclu3->lay == pclu2->lay)  ) continue; // the same layer

	  //if (fabs(pclu3It->y - pclu2It->y) > dy_max) continue;
	  if (fabs(pclu3It->y - pclu2It->y)/fabs(pclu3It->z - pclu2It->z)>tan_max) continue;


	  float z3 = pclu3->z; 
	  float u  = pclu3->x;
	  float x  = trIt->x + trIt->tx*(z3-geslz1);
	  float y  = (x*pclu3->cosa - u)/pclu3->sina;
	  float x3 = x*pclu1->cosa - y*pclu1->sina; // recalculate to stereo layer of the first cluster 

	  
	  if (fabs((x3-x2)/(z3-z2)) > tan_max) break;
	  
	  CLUSTER *pclul = pclu1;
	  CLUSTER *pclum = pclu2;
	  CLUSTER *pclur = pclu3;
	  CLUSTER *pclu;
	  if (pclum->z < pclul->z) {pclu = pclul; pclul = pclum; pclum = pclu;}
	  if (pclur->z < pclul->z) {pclu = pclul; pclul = pclur; pclur = pclu;}
	  if (pclur->z < pclum->z) {pclu = pclum; pclum = pclur; pclur = pclu;}

	  float zl = pclul->z; 
	  float xl = pclul->x;

	  float zm = pclum->z; 
	  u        = pclum->x;
	  x        = trIt->x + trIt->tx*(zm-geslz1);
	  float ym = (x*pclum->cosa - u)/pclum->sina;
	  float xm = x*pclul->cosa - ym*pclul->sina; // recalculate to stereo layer of the first cluster 

	  float zr = pclur->z; 
	  u        = pclur->x;
	  x        = trIt->x + trIt->tx*(zr-geslz1);
	  float yr = (x*pclur->cosa - u)/pclur->sina;
	  float xr = x*pclul->cosa - yr*pclul->sina; // recalculate to stereo layer of the first cluster 
	  
	  float tx   = trIt->tx;
	  float distm = fabs(xm - (xl + tx*(zm - zl)) );
	  float distr = fabs(xr - (xl + tx*(zr - zl)) );
	  
	  if ((distm < dist_max)&&(distr < dist_max)){ // store space point

	    sp.xf  = trIt->x;    sp.txf = trIt->tx;
	    sp.yf  = pclu1It->y/* + pclu1It->ty*(geslz1-pclu1It->z)*/;  sp.tyf = pclu1It->ty;
	    sp.zf  = geslz1;

	    sp.xe  = sp.xf + sp.txf*(geslz2-sp.zf);  sp.txe = sp.txf;
	    sp.ye  = sp.yf + sp.tyf*(geslz2-sp.zf);  sp.tye = sp.tyf; 
	    sp.ze  = geslz2;
	    sp.used = 0; sp.level = 0; sp.neighborindex = -1;
	    sp.vpclu.clear();//vphit
	    sp.vpclu.push_back(trIt->pclul);
	    if (trIt->length == 3) sp.vpclu.push_back(trIt->pclum);
	    sp.vpclu.push_back(trIt->pclur);
	    sp.vpclu.push_back(pclul);
	    sp.vpclu.push_back(pclum);
	    sp.vpclu.push_back(pclur);

	    OtrFitSPoint(sp);
	    if (sp.prob > 0.05){

	      float pinv=1.0/(B*zmagn*
			      sqrt(1.0+sp.txf*sp.txf+sp.tyf*sp.tyf)/
			      ((sp.xf-sp.zf*sp.txf)*
			       sqrt(1.0+sp.txf*sp.txf)));
	      pinv = fabs(pinv)+0.0000001;
	      if (pinv > 0.5) pinv = 2.0*pinv*pinv;
	      sp.pf=sp.pe=pinv;

	      vspoint.push_back(sp);
	      pclu1It->used = 1; pclu2It->used = 1; pclu3It->used = 1;
	      nstereotriplets++;
	    }
	  }
	}
      }
    }
    
    // combine rest ystereo clusters into doublets

    if (nstereotriplets == 0)
    for (vector<YSTEREO>::iterator pclu1It = vystereo.begin();
	 pclu1It != vystereo.end(); ++pclu1It){// pclu1It
      if (pclu1It->used) continue;

      pCLUSTER pclu1 = pclu1It->psclu;
      float x1 = pclu1->x;
      float z1 = pclu1->z;
      //float x  = trIt->x + trIt->tx*(z1-geslz1);
      //float y1  = (x*pclu1->cosa - x1)/pclu1->sina;
      vector<YSTEREO>::iterator pclu2_begin = pclu1It;
      if (++pclu2_begin == vystereo.end()) continue;
      
      for (vector<YSTEREO>::iterator pclu2It = pclu2_begin;
	   pclu2It != vystereo.end(); ++pclu2It){// pclu2It
	if (pclu2It->used) continue;
	  pCLUSTER pclu2 = pclu2It->psclu;
	  if (pclu2->lay == pclu1->lay) continue; // the same layer

	  //if (fabs(pclu2It->y - pclu1It->y) > dy_max) continue;
	  if (fabs(pclu2It->y - pclu1It->y)/fabs(pclu2It->z - pclu1It->z)>10.0*tan_max) continue;

	  float z2 = pclu2->z; 
	  float u  = pclu2->x;
	  float x  = trIt->x + trIt->tx*(z2-geslz1);
	  float y2  = (x*pclu2->cosa - u)/pclu2->sina;
	  float x2 = x*pclu1->cosa - y2*pclu1->sina; // recalculate to stereo layer of the first cluster 

	  
	  if (fabs((x2-x1)/(z2-z1)) > 10.0*tan_max) break;
	  

	  float tx   = trIt->tx;
	  float dist = fabs(x2 - (x1 + tx*(z2 - z1)) );
	  
	  if (dist < dist_max){ // store space point

	    sp.xf  = trIt->x;    sp.txf = trIt->tx;
	    sp.yf  = pclu1It->y/* + pclu1It->ty*(geslz1-pclu1It->z)*/;  sp.tyf = pclu1It->ty;
	    sp.zf  = geslz1;

	    sp.xe  = sp.xf + sp.txf*(geslz2-sp.zf);  sp.txe = sp.txf;
	    sp.ye  = sp.yf + sp.tyf*(geslz2-sp.zf);  sp.tye = sp.tyf; 
	    sp.ze  = geslz2;
	    sp.used = 0; sp.level = 0; sp.neighborindex = -1;
	    sp.vpclu.clear();
	    sp.vpclu.push_back(trIt->pclul);
	    if (trIt->length == 3) sp.vpclu.push_back(trIt->pclum);
	    sp.vpclu.push_back(trIt->pclur);
	    sp.vpclu.push_back(pclu1);
	    sp.vpclu.push_back(pclu2);
	    
	    OtrFitSPoint(sp);
	    if (sp.prob > 0.05){

	      float pinv=1.0/(B*zmagn*
			      sqrt(1.0+sp.txf*sp.txf+sp.tyf*sp.tyf)/
			      ((sp.xf-sp.zf*sp.txf)*
			       sqrt(1.0+sp.txf*sp.txf)));
	      pinv = fabs(pinv)+0.0000001;
	      if (pinv > 0.5) pinv = 2.0*pinv*pinv;
	      sp.pf=sp.pe=pinv;

	      vspoint.push_back(sp);
	    }
	  }	  
      }
    }

    // rest doublets
  }

  propagate_to_m = 0;

  return;
}

//========================================================================


void OtrFitSPoint(SPACEPOINT& sp){
  float Rk[4],Kk[4],Bk[4],Gk[4][4];
  float Z,Z0,sin2,cos2,Dk,Dk2,zeta,chi2=0.0,z00=0.0,sigma00=0.1,sigma2;

  float zf = sp.zf;
  float ze = sp.ze;
  
  Z0 = sp.zf; sp.zf = sp.ze; sp.ze = Z0;
  for (vector<pCLUSTER>::iterator cluIt = sp.vpclu.begin();
       cluIt != sp.vpclu.end(); ++cluIt){
    if ((*cluIt)->z < sp.zf) sp.zf = (*cluIt)->z;
    if ((*cluIt)->z > sp.ze) sp.ze = (*cluIt)->z;
  }
  sp.xf = sp.xf + sp.txf*(sp.zf - Z0);
  sp.yf = sp.yf + sp.tyf*(sp.zf - Z0);
  
  Z0 = sp.zf;
  for (int i=0; i<4; i++) 
    for (int j=0; j<=i; j++) 
      Gk[j][i] = 0.0;
  Gk[0][0] = 1.0; Gk[1][1] = 10.0; Gk[2][2] = 1.0; Gk[3][3] = 1.0;
  Rk[0] = sp.xf; Rk[1] = sp.yf; Rk[2] = sp.txf; Rk[3] = sp.tyf;  

  for (vector<pCLUSTER>::iterator cluIt = sp.vpclu.begin();
       cluIt != sp.vpclu.end(); ++cluIt){
    pCLUSTER pclu = (*cluIt);
    Dk = pclu->z-Z0; Dk2 = Dk*Dk;
    //    sigma2 = (pclu->gede->pit*pclu->gede->pit)/(4.0*9.0*pclu->length);
    sigma2 = pclu->sigma*pclu->sigma;
    sin2 = pclu->sina*pclu->sina; cos2 = pclu->cosa*pclu->cosa;
    Z = (cos2*(Gk[0][0] + Dk2*Gk[2][2] + 2.0*Dk*Gk[0][2]) +
	 sin2*(Gk[1][1] + Dk2*Gk[3][3] + 2.0*Dk*Gk[1][3]) -
	 2.0*pclu->cosa*pclu->sina*(Gk[0][1] + Dk*(Gk[0][3] + Gk[1][2]) + Dk2*Gk[2][3]) + sigma2);
    Z = 1.0/Z;
    Bk[0] = pclu->cosa*(Gk[0][0] + Dk*Gk[0][2]) - pclu->sina*(Gk[0][1] + Dk*Gk[0][3]);
    Bk[1] = pclu->cosa*(Gk[0][1] + Dk*Gk[1][2]) - pclu->sina*(Gk[1][1] + Dk*Gk[1][3]);
    Bk[2] = pclu->cosa*(Gk[0][2] + Dk*Gk[2][2]) - pclu->sina*(Gk[1][2] + Dk*Gk[2][3]);
    Bk[3] = pclu->cosa*(Gk[0][3] + Dk*Gk[2][3]) - pclu->sina*(Gk[1][3] + Dk*Gk[3][3]);
    zeta = pclu->x - Rk[0]*pclu->cosa + Rk[1]*pclu->sina - Rk[2]*Dk*pclu->cosa + Rk[3]*Dk*pclu->sina;
    for (int i=0; i<4; i++){
      Kk[i] = Z*Bk[i];
      Rk[i] += Kk[i]*zeta;
      for (int j=0; j<=i; j++) 
	Gk[j][i] -= Kk[j]*Bk[i];
    }
  }

  {
    Dk=z00-Z0;Dk2=Dk*Dk;
    zeta=-Rk[1]-Rk[3]*Dk;
    sigma2=sigma00*sigma00;
    Z=Gk[1][1]+2.0*Dk*Gk[1][3]+Dk2*Gk[3][3]+sigma2;
    Z=1.0/Z;
    Bk[0]=Gk[0][1]+Dk*Gk[0][3]; Bk[1]=Gk[1][1]+Dk*Gk[1][3];
    Bk[2]=Gk[1][2]+Dk*Gk[2][3]; Bk[3]=Gk[1][3]+Dk*Gk[3][3];
    for (int i=0; i<4; i++){
      Kk[i]  = Z*Bk[i];
      Rk[i] += Kk[i]*zeta;
      for (int j=0; j<=i; j++) 
	Gk[j][i] -= Kk[j]*Bk[i];
    }
  }
  

  int nclu = 0;
  for (vector<pCLUSTER>::iterator cluIt = sp.vpclu.begin();
       cluIt != sp.vpclu.end(); ++cluIt){
    pCLUSTER pclu = (*cluIt);
    Dk = pclu->z-Z0; Dk2 = Dk*Dk;
    //    sigma2 = (pclu->gede->pit*pclu->gede->pit)/(4.0*9.0*pclu->length);
    sigma2 = pclu->sigma*pclu->sigma;
    sin2 = pclu->sina*pclu->sina; cos2 = pclu->cosa*pclu->cosa;
    Z = (cos2*(Gk[0][0] + Dk2*Gk[2][2] + 2.0*Dk*Gk[0][2])+
	 sin2*(Gk[1][1] + Dk2*Gk[3][3] + 2.0*Dk*Gk[1][3])-
	 2.0*pclu->cosa*pclu->sina*(Gk[0][1] + Dk*(Gk[0][3] + Gk[1][2]) + Dk2*Gk[2][3]) + sigma2);
    zeta = pclu->x - Rk[0]*pclu->cosa + Rk[1]*pclu->sina - Rk[2]*Dk*pclu->cosa + Rk[3]*Dk*pclu->sina;
    chi2 += zeta*zeta/Z;
    nclu++;
  }

  {
    Dk=z00-Z0;Dk2=Dk*Dk;
    zeta=-Rk[1]-Rk[3]*Dk;
    sigma2=sigma00*sigma00;
    Z=Gk[1][1]+2.0*Dk*Gk[1][3]+Dk2*Gk[3][3]+sigma2;
    chi2+=zeta*zeta/Z;
  }

  sp.xf  = Rk[0]; sp.yf  = Rk[1];
  sp.txf = Rk[2]; sp.tyf = Rk[3];
  for (int i=0;i <4; i++) 
    for (int j=0; j<=i; j++){
      sp.cvf[j][i] = Gk[j][i];
      Gk[j][i] = 0.0; // clear for next steps
    }


  Z0 = sp.ze;
  Gk[0][0] = 1.0; Gk[1][1] = 10.0; Gk[2][2] = 1.0; Gk[3][3] = 1.0;
  Rk[0] = sp.xf+sp.txf*(sp.ze-sp.zf); Rk[1] = sp.yf+sp.tyf*(sp.ze-sp.zf); Rk[2] = sp.txf; Rk[3] = sp.tyf;  

  for (vector<pCLUSTER>::iterator cluIt = sp.vpclu.begin();
       cluIt != sp.vpclu.end(); ++cluIt){
    pCLUSTER pclu = (*cluIt);
    Dk = pclu->z-Z0; Dk2 = Dk*Dk;
    //    sigma2 = (pclu->gede->pit*pclu->gede->pit)/(4.0*9.0*pclu->length);
    sigma2 = pclu->sigma*pclu->sigma;
    sin2 = pclu->sina*pclu->sina; cos2 = pclu->cosa*pclu->cosa;
    Z = (cos2*(Gk[0][0] + Dk2*Gk[2][2] + 2.0*Dk*Gk[0][2]) +
	 sin2*(Gk[1][1] + Dk2*Gk[3][3] + 2.0*Dk*Gk[1][3]) -
	 2.0*pclu->cosa*pclu->sina*(Gk[0][1] + Dk*(Gk[0][3] + Gk[1][2]) + Dk2*Gk[2][3]) + sigma2);
    Z = 1.0/Z;
    Bk[0] = pclu->cosa*(Gk[0][0] + Dk*Gk[0][2]) - pclu->sina*(Gk[0][1] + Dk*Gk[0][3]);
    Bk[1] = pclu->cosa*(Gk[0][1] + Dk*Gk[1][2]) - pclu->sina*(Gk[1][1] + Dk*Gk[1][3]);
    Bk[2] = pclu->cosa*(Gk[0][2] + Dk*Gk[2][2]) - pclu->sina*(Gk[1][2] + Dk*Gk[2][3]);
    Bk[3] = pclu->cosa*(Gk[0][3] + Dk*Gk[2][3]) - pclu->sina*(Gk[1][3] + Dk*Gk[3][3]);
    zeta = pclu->x - Rk[0]*pclu->cosa + Rk[1]*pclu->sina - Rk[2]*Dk*pclu->cosa + Rk[3]*Dk*pclu->sina;
    for (int i=0; i<4; i++){
      Kk[i]  = Z*Bk[i];
      Rk[i] += Kk[i]*zeta;
      for (int j=0; j<=i; j++) 
	Gk[j][i] -= Kk[j]*Bk[i];
    }
  }

  {
    Dk=z00-Z0;Dk2=Dk*Dk;
    zeta=-Rk[1]-Rk[3]*Dk;
    sigma2=sigma00*sigma00;
    Z=Gk[1][1]+2.0*Dk*Gk[1][3]+Dk2*Gk[3][3]+sigma2;
    Z=1.0/Z;
    Bk[0]=Gk[0][1]+Dk*Gk[0][3]; Bk[1]=Gk[1][1]+Dk*Gk[1][3];
    Bk[2]=Gk[1][2]+Dk*Gk[2][3]; Bk[3]=Gk[1][3]+Dk*Gk[3][3];
    for (int i=0; i<4; i++){
      Kk[i]  = Z*Bk[i];
      Rk[i] += Kk[i]*zeta;
      for (int j=0; j<=i; j++) 
	Gk[j][i] -= Kk[j]*Bk[i];
    }
  }
  

  sp.xe  = Rk[0]; sp.ye  = Rk[1];
  sp.txe = Rk[2]; sp.tye = Rk[3];
  for (int i=0;i <4; i++) 
    for (int j=0; j<=i; j++){
      sp.cve[j][i] = Gk[j][i];
    }

  sp.nclu = nclu;


  //  float pit = (*(sp.vpclu.begin()))->gede->pit;
  
  if ((sp.cvf[0][0]< 0.0)||(sp.cvf[1][1]< 0.0)||
      (sp.cvf[2][2]< 0.0)||(sp.cvf[3][3]< 0.0)){
#ifdef XXX    
    cout<<"------------------>"<<endl;
    cout<<"------------------>"<<endl;
    cout<<"------------------> OtrCats bad cvf matrix ("<<nclu<<" hits) "
	<<sp.cvf[0][0]<<" "<<sp.cvf[1][1]<<" "<<sp.cvf[2][2]<<" "<<sp.cvf[3][3]<<endl;
    cout<<"------------------>"<<endl;
    cout<<"------------------>"<<endl;
#endif    
    sp.chi2 = 1000.0;
    sp.prob = 0.0; // will not be stored
  }
  else{
    if ((sp.cve[0][0]< 0.0)||(sp.cve[1][1]< 0.0)||
	(sp.cve[2][2]< 0.0)||(sp.cve[3][3]< 0.0)){
#ifdef XXX      
      cout<<"------------------>"<<endl;
      cout<<"------------------>"<<endl;
      cout<<"------------------> OtrCats bad cve matrix ("<<nclu<<" hits) "
	  <<sp.cve[0][0]<<" "<<sp.cve[1][1]<<" "<<sp.cve[2][2]<<" "<<sp.cve[3][3]<<endl;
      cout<<"------------------>"<<endl;
      cout<<"------------------>"<<endl;
#endif      
      sp.chi2 = 1000.0;
      sp.prob = 0.0; // will not be stored
    }
    else{
      if (chi2<0.0) chi2 = 0.0;
      if (nclu != 4){
	nclu += 1; //ndof++;
	sp.chi2 = chi2;// /(nclu - 4.0);
	sp.prob = prob_(fabs(chi2),(nclu - 4));
      }
      else{
	sp.chi2 = 0.0;
	sp.prob = 0.5;
      }
    }
  }
  
  
  // propagate to the middle of GESL

  if (!propagate_to_m) return;

  float dz;
  dz = 0.5*(zf+ze) - sp.zf;
  sp.xm = sp.xf + sp.txf*dz; sp.txm = sp.txf;
  sp.ym = sp.yf + sp.tyf*dz; sp.tym = sp.tyf;
  sp.zm = 0.5*(zf+ze);
  
  sp.cvm[0][0] = sp.cvf[0][0] + dz * (dz*sp.cvf[2][2] + sp.cvf[0][2] + sp.cvf[0][2]);
  sp.cvm[0][1] = sp.cvf[0][1] + dz * (dz*sp.cvf[2][3] + sp.cvf[0][3] + sp.cvf[1][2]); //sp.cvm[1][0] = sp.cvm[0][1];
  sp.cvm[1][1] = sp.cvf[1][1] + dz * (dz*sp.cvf[3][3] + sp.cvf[1][3] + sp.cvf[1][3]);
  sp.cvm[0][2] = sp.cvf[0][2] + dz * sp.cvf[2][2];                                    //sp.cvm[2][0] = sp.cvm[0][2];  
  sp.cvm[1][2] = sp.cvf[1][2] + dz * sp.cvf[2][3];                                    //sp.cvm[2][1] = sp.cvm[1][2]; 
  sp.cvm[0][3] = sp.cvf[0][3] + dz * sp.cvf[2][3];                                    //sp.cvm[3][0] = sp.cvm[0][3];
  sp.cvm[1][3] = sp.cvf[1][3] + dz * sp.cvf[3][3];                                    //sp.cvm[3][1] = sp.cvm[1][3];
  sp.cvm[2][2] = sp.cvf[2][2];
  sp.cvm[3][3] = sp.cvf[3][3];
  sp.cvm[2][3] = sp.cvf[2][3];                                                        //sp.cvm[3][2] = sp.cvm[2][3];

  if ((sp.cvm[0][0]< 0.0)||(sp.cvm[1][1]< 0.0)||
      (sp.cvm[2][2]< 0.0)||(sp.cvm[3][3]< 0.0)){
#ifdef XXX    
    cout<<"------------------>"<<endl;
    cout<<"------------------>"<<endl;
    cout<<"------------------> OtrCats bad cvm matrix ("<<nclu<<" hits) "
	<<sp.cvm[0][0]<<" "<<sp.cvm[1][1]<<" "<<sp.cvm[2][2]<<" "<<sp.cvm[3][3]<<endl;
    cout<<"------------------>"<<endl;
    cout<<"------------------>"<<endl;
#endif    
    sp.chi2 = 1000.0;
    sp.prob = 0.0; // will not be stored
  }
  
  return;
 }
