// *****************************************************************************
//
// source:    MuDUTool.C
//
// type:      source code
//
// created:   14 Mar 2001
//
// author:    Jing-Ge Shiu, Lars Soezueer 
//            email: soezueer@mail.desy.de
//
// purpose:   auxiliary functions for MUON, MPIX and MPAD
//            called by getFED_IDs, getFED and in MuMC2Digi.C
//
// changes:   
//
// *****************************************************************************
#include <iostream.h>
#include <iomanip.h>
//#include <string.h>
#include <stdlib.h>
#include <netinet/in.h>
#include "etb.h"

// include ARTE head files for new C++ interface    
#include "arte/arte.hh"        

#include "daqif/MuDUTool.h"

/******************************************************
******************************************************/
int jsGetEndian(void) {

  int Endian = 1;
  unsigned int cc = 0x01020304;

  if(cc == htonl(cc)) Endian=0;

  return Endian;
}

/******************************************************
******************************************************/
int jsGetDMUFED(unsigned short mutpx, unsigned short *fedid, unsigned int maxIDs) {

  int* len   = 0;
  if(mutpx == 1) len=(int*)(ArtePointer<DMUT>(0)->bits);
  if(mutpx == 2) len=(int*)(ArtePointer<DMUP>(0)->bits);
  if(mutpx == 3) len=(int*)(ArtePointer<DMUX>(0)->bits);
  if(*len <= 0) return 0;

  unsigned int ncrate = (unsigned int)(*len)/34;
  ncrate=(ncrate>maxIDs)?maxIDs:ncrate;

  for (unsigned int ncr=0; ncr<ncrate; ncr++) {
    unsigned int first=2+34*ncr;
    unsigned int *lbuf1;
    unsigned int word1;
    if(mutpx == 1) lbuf1=(unsigned int*)(ArtePointer<DMUT>(first)->bits);
    if(mutpx == 2) lbuf1=(unsigned int*)(ArtePointer<DMUP>(first)->bits);
    if(mutpx == 3) lbuf1=(unsigned int*)(ArtePointer<DMUX>(first)->bits);  

    if (jsGetEndian() == 0) word1 = *lbuf1;
    else word1 = ESP_SWAP_INT(*lbuf1);

    *fedid = word1&0xFFFF; 
    fedid++;   
  }

  return ncrate;
}

/******************************************************
******************************************************/
int jsCheckFEDTPX(unsigned short mutpx, unsigned short crate) {

  unsigned int fed421 = 0, fed451 = 0, goodtopick=0;

  int* len   = 0;
  if(mutpx == 1) len=(int*)(ArtePointer<DMUT>(0)->bits);
  if(mutpx == 2) len=(int*)(ArtePointer<DMUP>(0)->bits);
  if(mutpx == 3) len=(int*)(ArtePointer<DMUX>(0)->bits);
  if(*len <= 0) return 0;
  unsigned int ncrate = (unsigned int)(*len)/34;
  for (unsigned int ncr=0; ncr<ncrate; ncr++) {
    unsigned int first=2+34*ncr;
    unsigned int *lbuf1, *lbuf2;
    unsigned int *word0, word1, word2;
    if(mutpx == 1) lbuf1=(unsigned int*)(ArtePointer<DMUT>(first)->bits);
    if(mutpx == 2) lbuf1=(unsigned int*)(ArtePointer<DMUP>(first)->bits);
    if(mutpx == 3) lbuf1=(unsigned int*)(ArtePointer<DMUX>(first)->bits);  
    if(mutpx == 1) lbuf2=(unsigned int*)(ArtePointer<DMUT>(first+1)->bits);
    if(mutpx == 2) lbuf2=(unsigned int*)(ArtePointer<DMUP>(first+1)->bits);
    if(mutpx == 3) lbuf2=(unsigned int*)(ArtePointer<DMUX>(first+1)->bits);   

    if (jsGetEndian() == 0) {
      word1 = *lbuf1;
      word2 = *lbuf2;
    } else {
      word1 = ESP_SWAP_INT(*lbuf1);
      word2 = ESP_SWAP_INT(*lbuf2);
    }   
 
  }

  return 0;
}

/******************************************************
******************************************************/
int jsDumpDMU(unsigned short mutpx, unsigned short crate) {

  int* len  = 0;
  unsigned int *lbuf1, word1;

  if(mutpx == 1) len=(int*)(ArtePointer<DMUT>(0)->bits);
  if(mutpx == 2) len=(int*)(ArtePointer<DMUP>(0)->bits);
  if(mutpx == 3) len=(int*)(ArtePointer<DMUX>(0)->bits);
  if(*len <= 0) return 0;
  unsigned int ncrate = (unsigned int)(*len)/34;
  if (ncrate>crate) ncrate=crate;  

  if(mutpx == 1) lbuf1=(unsigned int*)(ArtePointer<DMUT>(1)->bits);
  if(mutpx == 2) lbuf1=(unsigned int*)(ArtePointer<DMUP>(1)->bits);
  if(mutpx == 3) lbuf1=(unsigned int*)(ArtePointer<DMUX>(1)->bits);  
  if (jsGetEndian() == 0) word1 = *lbuf1;
  else word1 = ESP_SWAP_INT(*lbuf1);
  cout << setiosflags(ios::uppercase) << hex << setw(8) << word1 << endl;

  for (unsigned int ncr=0; ncr<ncrate; ncr++) {
    unsigned int first=2+34*ncr;
    unsigned int crline = 6;
    //head for each FED block 
    for (int dii=0; dii<34; dii++) {
       int di = dii;
       if (dii>1) di = 8*((dii-2)/8)+8-(dii-2)%8+1;

       if(mutpx == 1) lbuf1=(unsigned int*)(ArtePointer<DMUT>(first+di)->bits);
       if(mutpx == 2) lbuf1=(unsigned int*)(ArtePointer<DMUP>(first+di)->bits);
       if(mutpx == 3) lbuf1=(unsigned int*)(ArtePointer<DMUX>(first+di)->bits);  
       if (jsGetEndian() == 0) word1 = *lbuf1;
       else word1 = ESP_SWAP_INT(*lbuf1);
        
       if (++crline>7) { cout << endl; crline=0; }
    }

  }
  cout << dec ;
  return 0;
}

/*##################################################  
  check the FED recordid, BX, and FEDid
 ##################################################*/
int jsCheckFED(void) {

  int check = 1;
  ArteTable<EVHD>::iterator evhd;
  ArteTable<DIHD>::iterator dihd;
  
  if ( evhd!=ArteTable<EVHD>::end() ) {
    cout << "evnt/bx = " <<(*evhd)->evt <<"," <<(*dihd)->bx << endl; 
    cout <<"Tube chambers  : " << endl;
    check = jsCheckFEDTPX(1,0);    
    cout <<"Pad chambers  : " << endl;
    jsCheckFEDTPX(2,0);    
    cout <<"Pixel chambers  : " << endl;
    jsCheckFEDTPX(3,0);    

  }  else {
    cout << "*********** empty EVHD : no run number ***********";
  };
  
  return check; 
}

/*#################################################
   find the component id for a FED 
 #################################################*/
short jsMuFindCmp0(unsigned short mutpx, unsigned short fedID, unsigned int &dmupos  ) {

   Word fedid[MAXMUFED], fedno=jsGetDMUFED(mutpx,fedid,MAXMUFED);

   for (unsigned int i=0;i<fedno;i++) 
     if (fedid[i]==fedID) { 
        dmupos = i;
        return 1;
     } 

   return 0;
  
};
short  jsMuFindCmp(unsigned short fedID, unsigned int &dmupos) {

  short fedcmp=0;

  if ( jsMuFindCmp0(1,fedID,dmupos) != 0 )
     fedcmp = COMP_ID_MUON;
  else if ( jsMuFindCmp0(2,fedID,dmupos) != 0 )
          fedcmp = COMP_ID_MPAD;
       else if ( jsMuFindCmp0(3,fedID,dmupos) != 0 )
               fedcmp = COMP_ID_MPIX;
            else return 0 ;

  dmupos=34*(dmupos)+2;
  
  return fedcmp;
}
