// *****************************************************************************
//
// source:    get_RICH_FED.C
//
// type:      source code
//
// created:   13 Mar 2001
//
// author:    Lars Soezueer
//            email: soezueer@mail.desy.de
//
// purpose:   get_RICH_FED: provide an FED from RICH
//            get_Num_RICH_FED: provide the numbers of FEDs in RICH
//            called by daq_rich_if.C
//
// changes:   
//
// *****************************************************************************
#include "rich/RICHDAQ.h"
#include "arte/ArteMessage.hh"

extern "C" {
#include "etb.h"
#include "esp.h"
#include "event.h"
}

char* richpath(void) {
  char* hbroot=getenv("HBROOT");
  if (! hbroot) {
    hbroot="/afs/desy.de/group/hera-b";
    arteMsg << "get_RICH_FED.C: $HBROOT not set, assume " << hbroot;
    arteMsg.warn();
  }
  char* datapath=new char[256];
  datapath[0]=0;
  strcpy(datapath,hbroot);
  strcat(datapath,"/GEO/RICH/");
  return datapath;
}

int get_RICH_FED(short fedid, FED_HDR  *rich_fed_hdr) { 

  ///// replace with a return call
  if (!RICH_DAQ) RICH_DAQ=new RICHDAQ(richpath());

  // -- check if fedid out of range
  if (fedid<0 || fedid>RICH_DAQ->NumFED()-1) {
    fprintf(stderr,"get_RICH_FED: FED id=%d out of range, valid range=0 .. %d \n",fedid,RICH_DAQ->NumFED()-1); 
    return 1;
  }

  // -- advance event
  RICH_DAQ->NextEvent();

  int num_byte_fed=RICH_DAQ->NumChannelPerFED()/8;

  // -- FED_HDR
  rich_fed_hdr->length=sizeof(FED_HDR)+num_byte_fed+sizeof(FED_TAIL); 
  rich_fed_hdr->recid  = 0x1000;
  rich_fed_hdr->fltevt = 0x1000;
  rich_fed_hdr->bxid   = 0x1000;
  rich_fed_hdr->compid = COMP_ID_RICH;
  rich_fed_hdr->fedid  = fedid;
  
  // -- data
  unsigned char *data=(unsigned char*)(rich_fed_hdr+1);
  memcpy(data, RICH_DAQ->daq_etb_fed_data(fedid), num_byte_fed);

  // -- FED_TAIL
  FED_TAIL *rich_fed_tail=(FED_TAIL*)(data+num_byte_fed);
  rich_fed_tail->fltevt=0x1000;

  ESP_put_fed_hdr(rich_fed_hdr, rich_fed_hdr);
  ESP_put_fed_tail(rich_fed_tail, rich_fed_tail);

  return 0;
}

int get_Num_RICH_FED() {
  ///// replace with a return call

  if (!RICH_DAQ) RICH_DAQ=new RICHDAQ(richpath());

  return RICH_DAQ->NumFED();
}
