
#include <stdio.h>
#include <strings.h>
#include <ctype.h>
#include <stdlib.h>
#include "arte/Arte.h"
#include "auto/MEMO.h"
#include "auto/MEMA.h"
#include "auto/MEMS.h"
#include "auto/MEIO.h"
#include "dbarte/DbArteBridge.h"
#include "arte/ArteFlag.h"
#include "arte/mecCount.h"


void chhtot_(char *, char *, int *);


void DbCompLedaRelTab(int *buffer, int tab, int *keys, int rno, int cno)
{
  /*************************************************************************
   *
   * Author: A. Semenov (JINR, Dubna)
   * Created 03.06.99: 
   *
   * Description: Compose LEDA relation table from row numbers.
   *
   * Called from: DbStoreLedaRel.
   *
   * External references: none.
   *
   * Argument in/out description
   *  buffer  in/out pointer to the relation table
   *  tab     in     ARTE table number
   *  keys    in     array of keys positions in ARTE table
   *  rno     in     number of rows in relation table
   *  cno     in     number of columns in relation table
   *
   * FUNCTION DESCRIPTION
   * In the input, relation table pointered by 'buffer' contains
   * ARTE table row numbers in the first column. This function
   * fills this table with values of key fields of ARTE table.
   *
   ************************************************************************/
	
	int *Link, *Mlink, atc, i, j;
	
	Mlink = (int *)((artelink_.larln1 + tab) * 4);
	Link  = gcbank_.iq + *Mlink;
	atc   = MEMO(tab).col;
	
	for(i=0;i<rno;i++)
	{
		int arno;
		arno=buffer[i*cno]-1;
		if(arno>N_U(Link))
			printf("dc: tab %4.4s: row %d of %d\n",MEMO(tab).name,arno,N_U(Link));
		for(j=0;keys[j];j++)
		{
			buffer[i*cno+j]=Link[2+arno*atc+keys[j]-1];
		}
			
	}
	
}

void DbDecompLedaRelTab(int *buffer, int tab, int *keys, int rno, int cno, 
		int *ierr)
{
  /*************************************************************************
   *
   * Author: A. Semenov (JINR, Dubna)
   * Created 03.06.99: 
   *
   * Description: Decompose LEDA relation table to row numbers.
   *
   * Called from: DbLoadLedaRel.
   *
   * External references: none.
   *
   * Argument in/out description
   *  buffer  in/out pointer to the relation table
   *  tab     in     ARTE table number
   *  keys    in     array of keys positions in ARTE table
   *  rno     in     number of rows in relation table
   *  cno     in     number of columns in relation table
   *
   * FUNCTION DESCRIPTION
   * In the input, relation table pointered by 'buffer' contains
   * the values of key fields of ARTE table. This function finds
   * the corresponding row of ARTE table and puts its number
   * to the first column of relkation table.
   *
   ************************************************************************/
	
	int *Link, *Mlink, atc, atr, i, j, k;
	
	Mlink = (int *)((artelink_.larln1 + tab) * 4);
	Link  = gcbank_.iq + *Mlink;
	atc   = MEMO(tab).col;
	atr   = N_U(Link);
	
	for(i=0;i<rno;i++)
	{
		int arno=0;

		for(j=0;j<atr;j++)
		{
			int found=1;
			for(k=0;keys[k];k++)
				if(buffer[i*cno+k]!=Link[2+j*atc+keys[k]-1])
					found=0;
			if(found)
			{
				arno=j+1;
				break;
			}
		}
		if(!arno)
		{
			static int errPtr;
			static char mess[80],Name[DB_MAX_NAME_SIZE];
			int var;
			var = DB_MAX_NAME_SIZE/4;
  			chhtot_(MEMO(tab).name, Name, &var );
    		sprintf(mess,
    		   "DbLoadLedaRel: can not find the row for key (table '%s')\n",
					 Name );
    		mecErrorCount( &errPtr, mess );
			*ierr=10;
			return;
		}
		buffer[i*cno]=arno;
	}
	
}
