      SUBROUTINE DGPTI( IPL , iMIMF , iMIML )
***********************************************************************
*                                                                     *
* DDDDDDDDD      GGGGGGGGGG   PPPPPPPPPPP   TTTTTTTTTTTT   IIIIIIIIII *
* DDDDDDDDDD    GGGGGGGGGGGG  PPPPPPPPPPPP  TTTTTTTTTTTT   IIIIIIIIII *
* DD       DD   GG        GG  PP        PP       TT            II     *
* DD        DD  GG            PP        PP       TT            II     *
* DD        DD  GG            PP        PP       TT            II     *
* DD        DD  GG            PPPPPPPPPPPP       TT            II     *
* DD        DD  GG     GGGGG  PPPPPPPPPPP        TT            II     *
* DD        DD  GG     GGGGG  PP                 TT            II     *
* DD        DD  GG        GG  PP                 TT            II     *
* DD       DD   GG        GG  PP                 TT            II     *
* DDDDDDDDDD    GGGGGGGGGGGG  PP                 TT        IIIIIIIIII *
* DDDDDDDDD      GGGGGGGGGG   PP                 TT        IIIIIIIIII *
*                                                                     *
***********************************************************************
*   Original version by I. Belyaev        May, 1997                   *
*   Modified by A. Schwartz to make                                   *
*      it compatible with new DGPTL.F     Aug, 1997                   *
*                                                                     *
*   Description: this routine translates mimps from GEANT output      *
*                into zebra bank DIGB for INNER part of high-pt       *
*                system                                               *
*                                                                     *
*   Called from: DGPTL                                                *
*                                                                     *
*   Arguments:   IPL      detector plane                              *
*                iMIMF    first mimp                                  *
*                iMIML    last mimp                                   *
*                                                                     *
*   External references (from CERNLIB):                               *
*      real function RANLAN   - inverse Landau function               *
*      real function VAVRND   - inverse Vavilov function              *
*      subroutine    VAVSET   - set Vavilov parameters                *
*      subroutine    GRNDM    - uniform random numbers from GEANT     *
***********************************************************************

      IMPLICIT NONE
      SAVE

#include "arte/Arte.fi"
#include "arte/ArteFlag.fi"
#include "auto/GEO.fi"
#include "auto/EV.fi"
#include "auto/TOPP.fi"
#include "gean/ZGESL.fi"
#include "gean/ZGMOTH.fi"

#include "digi/TMPDIG.fi"

      INTEGER    iPL , iMIM , iMIMF , iMIML , iDIGB1 , iDIGB2 
      INTEGER    i , IX ,IY , NX , NY , iCELL , iD1 , ID2 , iP 
      INTEGER    jAUX , IFX , IDIR , ICHA , IMAG , IFLAG 
      INTEGER    NLAU , NVAV, NGAU , NURB , IFIRST , ILAST 
  
      REAL       XPMIN , XPMAX , YPMIN , YPMAX , ZPMIN , ZPMAX 
      REAL       XCMIN , XCMAX , YCMIN , YCMAX , ZCMIN , ZCMAX 
      REAL       XSTART, YSTART, ZSTART,TXSTART,TYSTART 
      REAL       XEND  , YEND  , ZEND  ,TXEND  ,TYEND 
      REAL       TMINX , TMAXX , TMINY , TMAXY , TMINZ , TMAXZ 
      REAL       DXDT  , DYDT  , DZDT  , XPAD , YPAD ,k
      REAL       R     , XC    , ZC    , TIME , PMIM 
      REAL       XINGEA( 3 ) , BMAG ( 3 ) , XXX 
      REAL       SINP , COSP , SINE , COSE , BBOX 
      REAL       XX , YY , QUAN , DLEN , A , B , PROB , X , C  
      REAL       PHI , PHI1 , PHI2  , DZ , CONST , TT ,T1,T2
      REAL       BG  , BB , GG , TCORR , PMAS , ALEN 
      REAL       ALAMBDA , ALAMAVE, ALAMMAX , EMAX , XIL , XI 
      REAL       AKAPPA  , ALKAPP , DEDXE , BET2, GAM2 , TKI 
      REAL       DD , DDEL , DD2 , DD3 , DD4 , GAM , TCME , FF 
      REAL       ALMAX , B2 , RATIO , POTIL , EMIM , BOXM 
      REAL       ADSQUAN ,ZPAD

C* PARAMETERS 

      INTEGER    NPOINTS             ! Integral Estimation
      PARAMETER( NPOINTS  =5   )    
      REAL       ZCHARGE            
      PARAMETER( ZCHARGE  = 18.0 )
      REAL       DEDXGAS             ! dE/dX per cm length 
      PARAMETER( DEDXGAS  = 2.1 )    ! keV/cm 
      REAL       XICONST             ! Landau xi divided by x/beta/beta keV/cm 
      PARAMETER( XICONST  = 153.4 * ZCHARGE / 40.0 * 0.0014 )  
      REAL       GAMPRI              ! (1 - gam ), where gam is Euler constant
      PARAMETER( GAMPRI   = 0.422784 ) 
      REAL       PMASE               ! Electron mass [GeV/c2]
      PARAMETER( PMASE    = 0.00051099906 )
      REAL       PAR1 
      PARAMETER( PAR1     = 0.60715    )
      REAL       PAR2 
      PARAMETER( PAR2     = 0.67794    )
      REAL       PAR3 
      PARAMETER( PAR3     = 0.52382E-1 )
      REAL       PAR4 
      PARAMETER( PAR4     = 0.94753    )
      REAL       PAR5 
      PARAMETER( PAR5     = 0.74442    )
      REAL       PAR6 
      PARAMETER( PAR6     = 1.1934     )
      REAL       POTI                ! Mean Ionisation Potential  

      REAL       QION          
      PARAMETER( QION     = 26.0 )
      
      REAL       DUMMY

      INTEGER    LENOCC 
      REAL       RANLAN , VAVRND  , GVAVIV 
      EXTERNAL   LENOCC 
      EXTERNAL   RANLAN , VAVRND  , GVAVIV 

      REAL       QCHARGE 

#include "arte/Arte.ff"
#include "auto/GEO.ff"
#include "auto/EV.ff"
#include "auto/TOPP.ff"

#include "digi/TMPDIG.ff" 


C     *    Local statement functions
      
C     *    Landau Maximum:
      ALMAX(X) = PAR1 + PAR6*X + (PAR2+PAR3*X)*
     &     EXP(PAR4*X+PAR5)
      
C     *    Local function:
C     *     ITEP'95 measurement of Mean ADC Value per cm of 
C     *     track length in the cell as a function of coordinate 
C     *     along the wire. 
      PROB(X) =
     &     MAX(0.0,(1.0-4.0*(X-(ZCMAX+ZCMIN)/2.0)**2/
     &     (ZCMAX-ZCMIN)**2))**6
     &     *(1.0+1.5*SIN(PI*((X-(ZCMAX+ZCMIN)/2.0)/
     &     ((ZCMAX-ZCMIN)/2.0))**2))
C     *
C     *    End of statement functions
      

****************************************************************************
*                                                                          *
*    ssssssssss   tttttttttttt   aaaaaaaaaa   rrrrrrrrrrr   tttttttttttt   *
*   ssssssssssss  tttttttttttt  aaaaaaaaaaaa  rrrrrrrrrrrr  tttttttttttt   *
*   ss        ss       tt       aa        aa  rr        rr       tt        *
*   ss                 tt       aa        aa  rr        rr       tt        *
*   sss                tt       aa        aa  rr        rr       tt        *
*    sssssssss         tt       aaaaaaaaaaaa  rrrrrrrrrrrr       tt        *
*     sssssssss        tt       aaaaaaaaaaaa  rrrrrrrrrrr        tt        *
*            sss       tt       aa        aa  rr    rr           tt        *
*             ss       tt       aa        aa  rr     rr          tt        *
*   ss        ss       tt       aa        aa  rr      rr         tt        *
*   ssssssssssss       tt       aa        aa  rr       rr        tt        *
*    ssssssssss        tt       aa        aa  rr        rr       tt        *
*                                                                          *
****************************************************************************
      
C     *    Start of code itself
      POTI     = 16.0E-9*(ZCHARGE**0.9)
      
      IFIRST   = IMIMF 
      ILAST    = IMIML 
      
C     * check for a temporary booking table:
      IF( JTEMP .EQ. 0 )THEN
         C78='DGPTO warning!temporary table is not booked yet!'
         CALL ME78WC( 9238 ) 
         CALL MMBWRK( JTEMP , 0 , JTMPD_LEN , 50 ) 
      ENDIF 
      IF( N_C( JTEMP ) .NE. JTMPD_LEN )THEN
         C78='DGPTO warning!temporary table has wrong format!'
         CALL ME78WC( 92382345 ) 
         CALL MMBWRK( JTEMP , 0 , JTMPD_LEN , 50 ) 
      ENDIF 
      IF( N_U( JTEMP ) .NE. 0 )THEN
         C78='DGPTO warning!temporary table has wrong size'
         CALL ME78WC( 923825 ) 
         CALL MMBWRK( JTEMP , 0 , JTMPD_LEN , 50 ) 
      ENDIF 
      
C     *    Some geometry information for this Plane 
      
      XPMIN = FGEDE_SXMN(     iPL )       
      XPMAX = FGEDE_SXMX(     iPL )       
      YPMIN = FGEDE_SYMN(     iPL )       
      YPMAX = FGEDE_SYMX(     iPL )       
      ZPMIN = FGEDE_ZMIN(     iPL )       
      ZPMAX = FGEDE_ZMAX(     iPL )      
      
      NX    = IGEDE_NWI ( 1 , IPL )        
      NY    = IGEDE_NWI ( 2 , IPL )        
      
      XPAD  = ( XPMAX - XPMIN ) / FLOAT( NX )
      YPAD  = ( YPMAX - YPMIN ) / FLOAT( NY )
      ZPAD  = ( ZPMAX - ZPMIN ) 
      
      BBOX  = MAX( XPAD , YPAD , ZPMAX - ZPMIN ) 
      BOXM  = MIN( XPAD , YPAD , ZPMAX - ZPMIN ) 
      
      DO 100 iMIM = iFIRST , iLAST
         ZSTART       =      FMIMP_Z ( iMIM ) 
         XSTART       =      FMIMP_X ( iMIM ) 
         YSTART       =      FMIMP_Y ( iMIM )
         IF( ZSTART .GT. ZPMAX .OR. 
     &        ZSTART .LT. ZPMIN .OR. 
     &        XSTART .GT. XPMAX .OR. 
     &        XSTART .LT. XPMIN .OR. 
     &        YSTART .GT. YPMAX .OR. 
     &        YSTART .LT. YPMIN )THEN
            C78='DGPTL warning! MIMP is outside the sensitive volume'
            CALL MEMESC(922373647,1,ILARGEST,C78(1:LENOCC(C78)))
            GO TO 100  
         ENDIF 
         
C     *     Particle Momentum:
         PMIM       = ABS( FMIMP_P ( iMIM ) ) 
C     *     dxdz : 
         TXSTART    =      FMIMP_TX( iMIM )  
C     *     dydz : 
         TYSTART    =      FMIMP_TX( iMIM )
C     *     Particle Charge: 
         IF(  OMIMP_MTRA(iMIM) .GT. 0  .AND. 
     &        OMIMP_MTRA(iMIM) .LE. NMTRA() )THEN 
            IF( OMTRA_TOPP(OMIMP_MTRA(iMIM)) .GT. 0 .AND. 
     &           OMTRA_TOPP(OMIMP_MTRA(iMIM)) .LE. NTOPP())THEN
               ICHA       = ITOPP_CHRG(
     &              OMTRA_TOPP(OMIMP_MTRA(iMIM)))/3 
               PMAS       = FTOPP_MASS(
     &              OMTRA_TOPP(OMIMP_MTRA(iMIM)))
            ELSE 
               ICHA       = 0 
            ENDIF 
         ELSE 
            ICHA = 0 
         ENDIF 
         IDIR       = 1
C     *     Sign of pz: 
         IF( FMIMP_P( iMIM ) .LT. 0 )IDIR = -1 
C     *     Only charged particles:
         IF( ICHA .EQ. 0 ) GO TO 100 
         
C     *     Ionization correction (if particle is not a mip)
C     *     dE/dX dependence on beta (approximately)
         BG     = PMIM/PMAS 
         EMIM   = SQRT( PMIM * PMIM + PMAS * PMAS ) 
         BB     = PMIM/EMIM
         B2     = BB * BB 
         GG     = EMIM/PMAS 
         IF( BG .LE. 3.5 )THEN
C     *      In light gases the minima is positioned at beta*gamma=3.5 
C     *      Corrections according to (beta**(-5/3)) law 
            TCORR = (BB/3.5*(SQRT(1.0+3.5*3.5)))**(-5.0/3.0)
C     *      Correct for slow electrons:
            IF( TCORR .GT. 50.0 ) TCORR = 50.0
         ELSE 
C     *      No relativistic increase yet. 
C     *      Probably it is not nessesary at all. 
            TCORR = 1.0 
         ENDIF 
C     *     Landau Parameter xi divided by x 
         XI = XICONST/BB/BB 
         
         
C     *     Start Cell: 
         IX = INT( ( XSTART - XPMIN ) / XPAD ) + 1
         IY = INT( ( YSTART - YPMIN ) / YPAD ) + 1 
         
C     *     Estimate the magnetic field once per MIMP at MIMP point:
         XINGEA( 1 ) = XSTART
         XINGEA( 2 ) = YSTART 
         XINGEA( 3 ) = ZSTART   
         CALL UTFELD( XINGEA , BMAG )
C     *     For a moment - take into account only Y-component 
C     *     of the magnetic field.
         IMAG = 1 
         IF( ABS( BMAG( 2 ) ) .GE. 1.0E-10 )THEN
            R    = PMIM * 1000.0 / 0.3 / ABS( BMAG( 2 ) ) 
            IMAG = SIGN( 1.0 , BMAG( 2 ) ) 
         ELSE 
            R = 1.0E+10
         ENDIF       
C     *     Avoid the "straigh" lines
         IF( R .GT. BBOX * 5000.0 )R=BBOX*5000.0          
         
 10      CONTINUE 
         
         IF( IX .LE. 0 .OR. IX .GT. NX ) GO TO 100 
         IF( IY .LE. 0 .OR. IY .GT. NY ) GO TO 100 
         
C     *     Cell Parameters:
         XCMIN = XPMIN + FLOAT( IX - 1 ) * XPAD - XPAD * 0.01
         XCMAX = XPMIN + FLOAT( IX     ) * XPAD + XPAD * 0.01        
         YCMIN = YPMIN + FLOAT( IY - 1 ) * YPAD - YPAD * 0.01 
         YCMAX = YPMIN + FLOAT( IY     ) * YPAD + YPAD * 0.01 
         ZCMIN = ZPMIN - (ZPMAX-ZPMIN)*0.01
         ZCMAX = ZPMAX + (ZPMAX-ZPMIN)*0.01
         
C     *     Inside the cell Particle moves as: 
C     *      x = xc + b*R/a*sin(a*t+phi)
C     *      z = zc - b*R/a*cos(a*t+phi) 
C     *      y = ystart + c*t 
C     *       Conditions:    
C     *       (dx/dt)/(dz/dt)|(t=0) = TXSTART  
C     *       (dy/dt)/(dz/dt)|(t=0) = TYSTART
C     *         x            |(t=0) =  XSTART  
C     *         y            |(t=0) =  YSTART  
C     *         z            |(t=0) =  ZSTART
C     *        sign(dz/dt)   |(t=0) =  IDIR 
C     *        abs(a) = 1,abs(b)=1
         
C     *     First calculate a:
         A = FLOAT( ICHA * IMAG ) 
C     *     Phi and B:
         PHI1 = ATAN2(1.0*FLOAT(   IDIR),TXSTART*FLOAT(   IDIR))
         PHI2 = ATAN2(1.0*FLOAT(-1*IDIR),TXSTART*FLOAT(-1*IDIR))
         IF(          SIGN(1.0,SIN(PHI1)).EQ.FLOAT(IDIR))THEN
            PHI = PHI1
            B   =  1.0
         ELSEIF( -1.0*SIGN(1.0,SIN(PHI2)).EQ.FLOAT(IDIR))THEN
            PHI = PHI2
            B   = -1.0  
         ELSE
            C78='DGPTL warning! no solution was found!'
            CALL MEMESC(98347,1,ILARGEST,C78(1:LENOCC(C78)))
            GO TO 100 
         ENDIF 
         SINP = SIN(PHI)
         COSP = COS(PHI)
         CONST  = A/(B*R) 
C     *     xc and zc: 
         XC = XSTART - SINP / CONST 
         ZC = ZSTART + COSP / CONST 
C     *     c:
         C  = TYSTART * SINP / CONST * A  
         IF(ABS(C).LT.1.0E-10)C=SIGN(1.0E-10,C) 
         
C     *     3 time points  - particle leaves cell in X,in Y in Z 
C     *     TIME to  reach XCMIN  
         IF( ABS(XCMIN-XC) .LE. R )THEN
            TT = SQRT(ABS(R**2-(XCMIN-XC)**2))
            T1 = (ATAN2((XCMIN-XC)*CONST,-1.0*TT*CONST)-PHI)/A
            IF( T1 .LT. 0.0)T1=T1+TWOPI        
            IF( T1 .LT. 0.0)T1=T1+TWOPI        
            T2 = (ATAN2((XCMIN-XC)*CONST,     TT*CONST)-PHI)/A
            IF( T2 .LT. 0.0)T2=T2+TWOPI
            IF( T2 .LT. 0.0)T2=T2+TWOPI
            TMINX = MIN(T1,T2)
         ELSE 
            TMINX = 1.0E+10 
         ENDIF
C     *     TIME to  reach XCMAX  
         IF( ABS(XCMAX-XC) .LE. R )THEN
            TT = SQRT(ABS(R**2-(XCMAX-XC)**2))
            T1 = (ATAN2((XCMAX-XC)*CONST,-1.0*TT*CONST)-PHI)/A
            IF( T1 .LT. 0.0)T1=T1+TWOPI
            IF( T1 .LT. 0.0)T1=T1+TWOPI
            T2 = (ATAN2((XCMAX-XC)*CONST,     TT*CONST)-PHI)/A
            IF( T2 .LT. 0.0)T2=T2+TWOPI
            IF( T2 .LT. 0.0)T2=T2+TWOPI
            TMAXX = MIN(T1,T2)
         ELSE 
            TMAXX = 1.0E+10 
         ENDIF
C     *     TIME to  reach ZCMIN
         IF( ABS(ZCMIN-ZC) .LE. R )THEN
            TT = SQRT(ABS(R**2-(ZCMIN-ZC)**2))
            T1 = (ATAN2(     TT*CONST,
     &           -1.0*(ZCMIN-ZC)*CONST)-PHI)/A
            IF( T1 .LT. 0.0)T1=T1+TWOPI
            IF( T1 .LT. 0.0)T1=T1+TWOPI
            T2 = (ATAN2(-1.0*TT*CONST,
     &           -1.0*(ZCMIN-ZC)*CONST)-PHI)/A
            IF( T2 .LT. 0.0)T2=T2+TWOPI
            IF( T2 .LT. 0.0)T2=T2+TWOPI
            TMINZ = MIN(T1,T2)
         ELSE 
            TMINZ = 1.0E+10 
         ENDIF
C     *     TIME to  reach ZCMAX
         IF( ABS(ZCMAX-ZC) .LE. R )THEN
            TT = SQRT(ABS(R**2-(ZCMAX-ZC)**2))
            T1 = (ATAN2(     TT*CONST,
     &           -1.0*(ZCMAX-ZC)*CONST)-PHI)/A
            IF( T1 .LT. 0.0)T1=T1+TWOPI
            IF( T1 .LT. 0.0)T1=T1+TWOPI
            T2 = (ATAN2(-1.0*TT*CONST,
     &           -1.0*(ZCMAX-ZC)*CONST)-PHI)/A
            IF( T2 .LT. 0.0)T2=T2+TWOPI
            IF( T2 .LT. 0.0)T2=T2+TWOPI
            TMAXZ = MIN(T1,T2)
         ELSE 
            TMAXZ = 1.0E+10 
         ENDIF
C     *     Time to reach YCMIN
         TMINY = (YCMIN-YSTART)/C 
         IF( TMINY .LT. 0.0)TMINY=1.0E+10 
C     *     Time to reach YCMAX
         TMAXY = (YCMAX-YSTART)/C 
         IF( TMAXY .LT. 0.0)TMAXY=1.0E+10 
         
         TIME=MIN(TMINX,TMAXX,TMINY,TMAXY,TMINZ,TMAXZ)
         
         IF( TIME .GT. 10000.0 .OR. TIME .LT. 0.0 )THEN 
C     *      Something wrong ....
            C78='DGPTL warning!Incorrect Time='
            CALL CH78AF( TIME , 10 ) 
            CALL MEMESC(373719,1,ILARGEST,C78(1:LENOCC(C78)))
            GO TO 100 
         ELSEIF( TIME .EQ. 0.0 )THEN 
            C78='DGPTL warning! track length is zero!'
            CALL MEMESC(209348,999,ILARGEST,C78(1:LENOCC(C78)))
            GO TO 100 
         ENDIF 
         
C     *     Parameters at the end of trajectory:
         SINE = SIN( A * TIME + PHI ) / CONST 
         COSE = COS( A * TIME + PHI ) / CONST 
         
         XEND = XC + SINE  
         ZEND = ZC - COSE 
         YEND = YSTART + C * TIME 
         
         DXDT  = A *  COSE 
         DYDT  = C 
         DZDT  = A *  SINE 
         
         IF(ABS(DZDT).LT.1.0E-10)DZDT=SIGN(1.0E-10,DZDT)
         TXEND = DXDT/DZDT 
         TYEND = DYDT/DZDT 
         IDIR  = SIGN( 1.0 , DZDT ) 
         
C     *     Track length in this cell 
         DLEN = TIME * SQRT( R * R + C * C ) 
         
         IF( DLEN .LE. 1.0E-2 * BOXM )THEN 
            C78='DGPTL warning! Track length <1% of CELL!'
            CALL MEMESC(7128723,1,ILARGEST,C78(1:LENOCC(C78)))
            GO TO 100 
         ENDIF 
         
C     *     Integrate the track length with the Z-Probability weights
         DZ    = (ZEND - ZSTART)/FLOAT(NPOINTS)
         QUAN  = 0.0 
         DO I  = 1 , NPOINTS
            QUAN = QUAN + PROB(ZSTART+DZ*(FLOAT(i)-0.5))
         ENDDO 
         QUAN  = QUAN * DLEN/FLOAT(NPOINTS)
C     *     Apply M.I.P. & Landau corrections for heavy particles
         XIL     = XI * DLEN 
C     *     (2 m_e beta beta gamma gamma )/(1+2gamma me/m + (me/m)**2)
         EMAX   = 2.0*PMASE*1000.0*1000.0*BG*BG  
         EMAX   = EMAX/(1.0+2*GG*(PMASE/PMAS) + 
     &        (PMASE/PMAS)**2)
         AKAPPA = XIL/EMAX
         ALKAPP = ALOG( AKAPPA ) 
         IFLAG  = 0 
         POTIL  = ALOG( POTI )
         IF( MIN(XIL,DEDXGAS*DLEN*TCORR)/1000000.0
     &        /POTI .LT. 50.0 )THEN 
            IFLAG = 4 
C     *      Urban Case
            NURB  = 0  
 20         CONTINUE 
            NURB = NURB + 1 
            IF(NURB .GT. 100 )THEN 
               C78='DGPTL warning! Number of Urban Sampling >100!'
               CALL MEMESC(373739819,1,ILARGEST,C78(1:LENOCC(C78)))
               GO TO 100 
            ENDIF 
            CALL DGPTIU(
     &           ZCHARGE * FLOAT( ICHA ) , 
     &           DLEN , 
     &           PMIM , 
     &           EMIM ,
     &           DEDXGAS * TCORR / 1000000.0 , 
     &           DEDXE , 
     &           POTI  , 
     &           POTIL ) 
            ALEN=DEDXE/DEDXGAS*1000000.0
            IF( ALEN .LE.0 .OR.
     &           ALEN/DLEN .GT.100.0)GO TO 20 
            QUAN = QUAN * ALEN / DLEN
         ELSEIF( AKAPPA .LE. 0.01 )THEN 
C     *      Landau Case   
            IFLAG = 1 
C     *      Average Lambda
            ALAMAVE = -1.0*GAMPRI-BB*BB-ALKAPP
            ALAMMAX = ALMAX( ALAMAVE ) 
            NLAU = 0 
 30         CONTINUE 
            NLAU = NLAU + 1 
            IF( NLAU .GT. 100 )THEN 
               C78='DGPTL warning! Landau Sampling>100'
               CALL MEMESC(74337,10,ILARGEST,C78(1:LENOCC(C78)))
               GO TO 100 
            ENDIF 
C     *      Generate Landau
            CALL GRNDM( ALAMBDA , 1 ) 
            ALAMBDA = RANLAN( ALAMBDA )  
            IF(ALAMBDA.GT.30.0.OR.ALAMBDA.LT.-4)GO TO 30 
            ALEN = DLEN*TCORR + 
     &           (ALAMBDA-2.919)*XIL/DEDXGAS
            IF( ALEN .LE. 0.0 .OR. ALEN .GT. 
     &           100.0 * DLEN * TCORR)GO TO 30 
            QUAN = QUAN/DLEN*ALEN 
         ELSEIF( AKAPPA .LE. 12 ) THEN 
C     *      Vavilov Case  
            IFLAG = 2 
            IF(AKAPPA.GT.12.OR.AKAPPA.LT.0.01.OR.BB*BB.GT.1.0)THEN
               C78='DGPTL warning! Incorrect Vavilov k='
               CALL CH78AF(AKAPPA,5)
               CALL CH78CC( 'BB-1=' )
               CALL CH78AF(BB*BB-1,-3)
               CALL MEMESC(8273837,1,ILARGEST,C78(1:LENOCC(C78)))
               GO TO 100 
            ENDIF 
            NVAV = 0 
            CALL VAVSET(AKAPPA,BB*BB,1)
 40         CONTINUE   
            NVAV = NVAV + 1 
            IF( NVAV .GT. 100 )THEN 
               C78='DGPTL warning! Vavilov Sampling>100!'
               CALL MEMESC(283782,1,ILARGEST,C78(1:LENOCC(C78)))
               GO TO 100 
            ENDIF 
            CALL GRNDM(ALAMBDA,1)
            ALAMBDA = VAVRND( ALAMBDA )
            ALEN = DLEN*TCORR + 
     &           (ALAMBDA/AKAPPA+GAMPRI+BB*BB)*XIL/DEDXGAS
            IF( ALEN .LE. 0.0 .OR. ALEN .GT. 
     &           100.0 * DLEN * TCORR ) GO TO 40 
            QUAN = QUAN/DLEN*ALEN 
         ELSE 
C     *      Gaussian Case
            IFLAG = 3 
            NGAU = NGAU + 1 
 50         CONTINUE 
            IF( NGAU .GT. 100 )THEN
               C78='DGPTL warning! Gaussian Sampling>100!'
               CALL MEMESC(4385727,1,ILARGEST,C78(1:LENOCC(C78)))
               GO TO 100 
            ENDIF 
            CALL GRANOR(X,ALAMBDA)
            ALEN = DLEN * TCORR + 
     &           ALAMBDA*SQRT(XIL*EMAX*(1.0-BB*BB/2.0))/DEDXGAS         
            IF( ALEN .LE. 0.0 .OR. ALEN .GT.  
     &           DLEN * 100.0 *TCORR )GO TO 50 
            QUAN = QUAN/DLEN*ALEN 
         ENDIF 
         
C     *     at this point QUAN is the "effective" length.
         QUAN = QUAN * DEDXGAS 
C     *     here QUAN is De/Dx in keV 
         QUAN = QUAN * 1000.0 
C     *     here QUAN is De/Dx in eV 
         QUAN = QUAN / QION
C     *     here QUAN is a number of Pairs 
         QUAN = QUAN *(2.0*(10.0**4))*(1.6*(10.0**(-4)))
C     *     here QUAN is a charge in fmC 
         
         
C     *  Book the temporary Digit bank 
         CALL MMBOOK( JTEMP , 1 ) 
C     *  Fill this bank: 
          Q( QTMPD_Q_L   ()) = QUAN 
         IQ(IQTMPD_USED_L()) = 0
C     *     Cell ID 
         iCELL = 1000000 * IX + 10000 * IY + IPL
         IQ(IQTMPD_ID_L ()) = iCELL 
C     * MIMP: 
         IQ(IQTMPD_MIMP_L()) = iMIM 
         
         
C     *     Where is the particle? 
         IF(     TIME .EQ. TMINX )THEN 
            IX = IX - 1 
         ELSEIF( TIME .EQ. TMAXX )THEN
            IX = IX + 1 
         ELSEIF( TIME .EQ. TMINY )THEN
            IY = IY - 1
         ELSEIF( TIME .EQ. TMAXY )THEN
            IY = IY + 1
         ELSE 
C     *      Particle has left the cell via Z-planes
            GO TO 100   
         ENDIF  
         
         XSTART  =  XEND 
         YSTART  =  YEND 
         ZSTART  =  ZEND 
         TXSTART  = TXEND 
         TYSTART  = TYEND
         GO TO 10               ! check the next cell 
         
 100  CONTINUE                  ! Go to next MIMP 
      
            
      RETURN 
      END
****************************************************************************
*                    end of routine(function) dgpti                        * 
****************************************************************************
*                                                                          *
*            EEEEEEEEEEEE  NN        NN  DDDDDDDDD                         *
*            EEEEEEEEEEEE  NNN       NN  DDDDDDDDDD                        *
*            EE            NNNN      NN  DD       DD                       *
*            EE            NN NN     NN  DD        DD                      *
*            EE            NN  NN    NN  DD        DD                      *
*            EEEEEEEE      NN   NN   NN  DD        DD                      *
*            EEEEEEEE      NN    NN  NN  DD        DD                      *
*            EE            NN     NN NN  DD        DD                      *
*            EE            NN      NNNN  DD        DD                      *
*            EE            NN       NNN  DD       DD                       *  
*            EEEEEEEEEEEE  NN        NN  DDDDDDDDDD                        *   
*            EEEEEEEEEEEE  NN         N  DDDDDDDDD                         *
*                                                                          *
****************************************************************************
*                    end of routine(function) dgpti                        * 
****************************************************************************
