// ********************************************************************
// source:    DQ_ITR_hit_evnt.C
// author:    Sonja Wunderlich
// called by: DQ_evnt()
//
// changes:
// 12/21 (SW   rhp_hbook2: 2-dim Occupancy Histograms    ID:5
// 12/21 (SW   rhp_hbook: Dead Helixes                   ID:15
// 12/21 (SW   rhp_hbook: Dead Readout                   ID:16 
// 12/21 (SW   loops for debugging with hbook
// 12/21 (SW)  rhp_hbook: Hits per event per SL          ID:500
// 01/12/01(SW) lookuptable for the chambers
// 01/25 (SW)  rhp_hbook: Hits per event ITR             ID:55             
// 01/27 (SW)  rhp_hbook: not working/dead helix/readout ID:17
// 01/30 (SW) rhp_hbook2: real occupancy per chamber     ID:6
//  2/5  (SW) uses the code DQ_ITR_my_hbook (D_ITR.h) to book rhp and hbook  
//  2/19 (SW) remove MS14 and MS15 out of occupancy-plot
//  4/13 (SW) include 2-dim Bx-spectrum
//  5/17 (SW) fill hitsSL in a 2 dim plot 
//  5/22 (SW) remove the 20 hitsperevent -plots per station 
//  6/25 (SW) coment out not realy necesary staff :)
// ******************************************************************

#include <iostream.h>
#include <map>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "math.h"
#include "vector.h"  

#include "rhp.h"
#include "hbook.h"

#include "arte/arte.hh"
#include "farm/farm.hh"
#include "dq/DQ.h"
#include "dq/DQ_ITR.h"
#include "ITR_marvin.h"

int DQ_ITR_buildIndex();
int DQ_ITR_isKnown(int);
int DQ_ITR_chamberIndex(int);
int DQ_ITR_indexChamber(int);
static map<int, int> chambersInOrder;

int DQ_ITR_hit_init(){
#if defined (DQ_ITR_COMPILE)
  cout<<"IIIIIIIIIIIIIII  DQ_ITR_hit_init.C IIIIIIIIIIIIIIIIII"<<endl;
#endif
  ARTE_mark( "DQ_ITR_hit_init", 0 );

  DQ_ITR_buildIndex();

#if defined (DQ_ITR_CONTROL) 
  DQ_ITR_my_hbook1( h_ITR_targetrate,LVL_HIT, "target rate", 80, -0.5, 39.5);
#endif

 DQ_ITR_my_hbook2(h_HitsPerEventSL ,LVL_HIT,"ITR: hits/EventTR (station)",
		  k_numHalfStation, -0.5, k_numHalfStation -0.5, 100, 0.2, 20.2);
 
 // DQ_ITR_my_hbook1(h_hitsoverSNcut,LVL_HIT,"ITR_ hits over sn cut",
 //	  2, 0, 2);
 // DQ_ITR_my_hbook1(h_Signalovernoise,LVL_HIT, "ITR: Singal/Noise",
 //	  60, 0,12);
 
 // DQ_ITR_my_hbook1( h_HitsPerEvent, LVL_HIT, "ITR:Hits /TR ", 
 //	   100, 0.2 , 20.2);

 DQ_ITR_my_hbook1(h_event,LVL_HIT, "Events",1 ,0.,1.);
 
 DQ_ITR_my_hbook2(h_Occupancy_bin, LVL_HIT,"ITR: Occ Helixbin (chamber)",
		  k_numChamber, -0.5, k_numChamber - 0.5, 
		  k_HelixPerChamber, -0.5, k_HelixPerChamber - 0.5);
 DQ_ITR_my_hbook2(h_Occupancy_raw, LVL_HIT,"ITR: Occ raw (chamber)", 
		  k_numChamber, -0.5, k_numChamber - 0.5, 
		   k_numStrips, -0.5, k_numStrips - 0.5);
 DQ_ITR_my_hbook2(h_BXSpectrum, LVL_HIT,"ITR: BX spectrum (chamber)", 
		  k_numChamber, -0.5, k_numChamber - 0.5, 
		  (float)NBUCKET, -0.5, (float)NBUCKET - 0.5);
 DQ_ITR_my_hbook2(h_NoiseMeanTenEvent,LVL_HIT,"ITR: Noise (history, chamber)",
		  k_numChamber, -0.5, k_numChamber - 0.5,
		   1000, 0, 1000);
 DQ_ITR_my_hbook2(h_numWorkingCham, LVL_HIT,"ITR: working Chamber (history, station)", 
		  k_numHalfStation,-.5, k_numHalfStation- 0.5, 1000, 0,1000); 
 DQ_ITR_my_hbook2(h_NoisePerChamber,LVL_HIT,"ITR: Noise (chamber)",
		  k_numChamber, -0.5, k_numChamber - 0.5,
		  300, 0, 15);
 
 ARTE_mark( "DQ_ITR_hit_init 1", 0 );  
 
 return 0; 
}

int DQ_ITR_buildIndex(){
  
#if defined (DQ_ITR_COMPILE)
    cout<< "DQ_ITR_hit_init.C/fill_chamber_index_map.C"<<endl;
#endif
  
  int chamber[184] = {112, 114, 115, 117, 122, 124, 125, 127,                 //01
			132, 134, 135, 137, 142, 144, 145, 147,               //01
			212, 214, 215, 217, 222, 224, 225, 227,               //03
			312, 314, 315, 317, 322, 324, 325, 327,               //05
			412, 414, 415, 417, 422, 424, 425, 427,               //06
			512, 514, 515, 517, 522, 524, 525, 527,               //10
			532, 534, 535, 537, 542, 544, 545, 547,               //10
			552, 554, 555, 557, 562, 564, 565, 567,               //10
			572, 574, 575, 577, 582, 584, 585, 587,               //10
			612, 614, 615, 617, 622, 624, 625, 627,               //11
			632, 634, 635, 637, 642, 644, 645, 647,               //11
			712, 714, 715, 717, 722, 724, 725, 727,               //12
			732, 734, 735, 737, 742, 744, 745, 747,               //12
			812, 814, 815, 817, 822, 824, 825, 827,               //13
			832, 834, 835, 837, 842, 844, 845, 847,               //13
			852, 854, 855, 857, 862, 864, 865, 867,               //13
			872, 874, 875, 877, 882, 884, 885, 887,               //13
			912, 914, 915, 917, 922, 924, 925, 927,               //14
			932, 934, 935, 937, 942, 944, 945, 947,               //14
			952, 954, 955, 957, 962, 964, 965, 967,               //14
		       1012, 1014, 1015, 1017, 1022, 1024, 1025, 1027,        //15
		       1032, 1034, 1035, 1037, 1042, 1044, 1045, 1047,        //15
		       1052, 1054, 1055, 1057, 1062, 1064, 1065, 1067};       //15
  
  int cnt;
  for ( cnt = 0; cnt < 184; cnt ++) {
    chambersInOrder[chamber[cnt]] = 1;
  }

  int idx = 1;
  for (map<int,int>::iterator mapIt =  chambersInOrder.begin();
       mapIt != chambersInOrder.end(); mapIt ++, idx ++) {
    
    mapIt->second = idx;
  }

  return (GRR_OK);
}

int DQ_ITR_isKnown(int id){
  
#if defined (DQ_ITR_COMPILE)
      cout<<"DQ_ITR_hit_init.C/isaknownchamber"<<endl;
#endif
    
    if (chambersInOrder.find(id) == chambersInOrder.end())     //not a known chamber
      return(0);
    return(1);
}

int DQ_ITR_chamberIndex(int chamber){
#if defined (DQ_ITR_COMPILE)
    cout<<"DQ_ITR_hit_init.C/get_cnt_from_chamber"<<endl;
#endif

  int cnt = -1;   

  if (chambersInOrder.find(chamber) != chambersInOrder.end())     //not a known chamber
    cnt = chambersInOrder[chamber]; 
  
  return(cnt);
}

int DQ_ITR_indexChamber (int idx) {
  int rc = -1;
  
  for (map<int,int>::iterator mapIt =  chambersInOrder.begin();
       mapIt != chambersInOrder.end(); mapIt ++) {
    
    if (mapIt->second == idx) { rc = mapIt->first; break; }
  }

  return (rc);
}
char *chamberName_from_hid (int hid) {
  char slname[10][4] = { "01", "03", "05", "06", 
                                "10", "11", "12", "13", 
                                "14", "15" } ;

  char secname[8][3] = { "", "++", "", "+-", "--", "", "-+", "" };
  static char result[16];

  int sl  = hid / 100;
  int lay = (hid % 100) / 10;
  int sec = hid % 10;
  
  sprintf (result, "MS%2s%2s%1d", slname[sl-1], secname[sec-1], lay);

  return(result);
}
