// ------------------------------------------------------
// ---
// ---  created: Aug 99 
// ---  author:  Denis Dujmic
// ---
// ---
// ---  RICH FED Occupancy Quality Check
// ---
// ---
// changes:
// 28 Oct 1999 (AG) use utility function for ia rate
//  1 Nov 1999 (AG) get rid of dq_hid.h
// 16 Dec 1999 (RP) histograms for fed channels and ph.det.
// 09 Jun 2000 (RP) change id of the channel
// 16 Jun 2000 (AG) remove SEGV bug
// 29 Jun 2000 (AG) bug removal by P. Conde
// 02 Nov 2000 (AG) added ARTE_mark()
// 24 Nov 2000 (AG) check pointer RICH_DAQ
// ------------------------------------------------------
#include "dq/DQ.h"
#include "rhp.h"
#include "rich/rich.h"
#include "farm/farm.hh"
#define MAX_RICH_CHANNELS 28672  
//AG float get_interaction_rate(EVENT_HDR *event);


extern float *fed_rich_buf;
int DQ_RICH_raw_allfed();

//
// function
//
int
DQ_RICH_raw_evnt(EVENT_HDR *event)
{
  ARTE_mark( "DQ_RICH_raw_evnt 0", 0 );

  //
  // check pointer RICH_DAQ
  //
  if( !RICH_DAQ ){
    ARTE_mark( "DQ_RICH_raw_evnt 1", 0 );
    return 0;
  }

  //DQ_RICH_raw_allfed();return 0;
  // --  used histograms
  unsigned int rich_fed_histo_id=DQ_ID(COMP_RICH, LVL_RAW,100);
  unsigned int rich_fed_refer_id=DQ_ID(COMP_RICH, LVL_RAW,101);
  unsigned int rich_fed_ratio_id=DQ_ID(COMP_RICH, LVL_RAW,102);
  unsigned int rich_allfed_histo_id=DQ_ID(COMP_RICH, LVL_RAW,103);
  unsigned int phdet[2]={ DQ_ID(COMP_RICH, LVL_RAW,104),
			  DQ_ID(COMP_RICH, LVL_RAW,105) };
  // -- my event count
  static unsigned int event_count=0;
  event_count++;
  //cout << "DQ_RICH_raw_evnt: RICH_DAQ=" << RICH_DAQ << endl;
  static int nfed=RICH_DAQ->NumFED();
  static float *fed_count=0;
  static float *fed_ratio=0;
  static float *fed_refer=0;  
  if (!fed_count) {
    fed_count=new float[nfed];
    fed_ratio=new float[nfed];
    fed_refer=new float[nfed];
  }

  // -- interaction rate
  //AG float ia=1/get_interaction_rate(event);

  //float ia=1; // for stand-alone test run
  float ia = 1 / faDTARia(); // farm utility

  if (!rhp_hnoent(rich_allfed_histo_id))
    for (int i=0;i< MAX_RICH_CHANNELS + 1;i++) fed_rich_buf[i]=0; 

  // --- update fed occupancy per event
  if (!RICH_DAQ->NextEvent(event)){ 
    ARTE_mark( "DQ_RICH_raw_evnt 1", 0 );
    return 1;
  }
  fed_rich_buf[ MAX_RICH_CHANNELS ]++;
  for (int i=0; RICH_DAQ->HitChannelExist(i); i++)  {
    // fed channels 
    unsigned int id = RICH_DAQ->ChannelIndex();
    if ( id <  MAX_RICH_CHANNELS ) fed_rich_buf[id]++;
    // photon detector
    int sm   =RICH_DAQ->sm();
    int row  =RICH_DAQ->row();
    int col  =RICH_DAQ->col();
    int ch   =RICH_DAQ->pm_channel();
    int type =RICH_DAQ->pm_numchannel();
    int hid = 1;
    if (sm >6) {
      hid = 0;
      sm  = 13-sm;
      col = 11-col;
      row = 31-row;        
    }
    sm--;
    if (type==16) type=0; else type=1;
    int x0=col*4+sm*48;
    int y0=row*4;
    if (!type) {
      x0+=3-(ch%4);
      y0+=3-(ch/4);
    } else {
      x0+=(1-ch%2)*2;
      y0+=(1-ch/2)*2;    
    }
    for (int k=0;k<(type+1);k++) {
      for (int l=0;l<(type+1);l++) {
	rhp_hfill(phdet[hid], float(x0+k) , float(y0+l) ,1);
      }
    }

    rhp_hf1(rich_fed_histo_id,RICH_DAQ->fedid(),ia);
  }
  rhp_hpak(rich_allfed_histo_id,fed_rich_buf);
  //  printf("Number of hit channels=%d\n",RICH_DAQ->NumHitChannel()); 

  if (event_count%100){
    ARTE_mark( "DQ_RICH_raw_evnt 1", 0 );
    return 0;
  }

  // -- fill ratio histogram (rhp cannot divide histograms?!)
  rhp_hunpak(rich_fed_histo_id,fed_count);
  rhp_hunpak(rich_fed_refer_id,fed_refer);
  for (int i=0; i<nfed; i++) 
    fed_ratio[i]=fed_count[i]/(event_count*fed_refer[i]);
  rhp_hpak(rich_fed_ratio_id,fed_ratio);

  ARTE_mark( "DQ_RICH_raw_evnt 1", 0 );
  
  return 0;
}

//
//
//
int
DQ_RICH_raw_allfed()
{
  ARTE_mark( "DQ_RICH_raw_allfed 0", 0 );

  float *fed = fed_rich_buf;
  static int nfed=28;
  unsigned int rich_allfed_histo_id=DQ_ID(COMP_RICH, LVL_RAW,103);
  static int numfedevt=0;
  static ETB_fed *etb_fed;
  if (!etb_fed) etb_fed=new ETB_fed[nfed];
  ARTE_get_feds_DRIC(nfed , etb_fed, &numfedevt);
  int id=0;
  for (int k=0;k<numfedevt; k++) {
    cout << "\n" << id << " [" << k << "]  "<< endl;
    for (int ii=0;ii<128 ;ii++) {
      cout  << ii << ":";
      char *data = (char*) etb_fed[k].data;
      if (data[ii]) {
	int b=0;
        for (int jj=0, b=1;jj<8;jj++,b<<=1) {
	  if (data[ii]&b) {
	    (*fed)++;
	  }
	  id++;
	  fed++;
	}
      } else {
        id+=8;
	fed+=8; 
      }
    }
  }
  cout  << "DQ_RICH_raw_allfed() end\n";
  rhp_hpak(rich_allfed_histo_id,fed_rich_buf);

  ARTE_mark( "DQ_RICH_raw_allfed 1", 0 );

  return 0;
}












