 
// *****************************************************************************
//
// source:    DQ_SLT_rec.C
//
// type:      source code
//
// created:   13 Dec 2000
//
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// symbol:    DQ_SLT_rec_init
//            DQ_SLT_rec_evnt
//
// called by: DQ_init()
//            DQ_evnt()
//
// purpose:   Data Quality Monitoring using rhp
//
// hints:     use ARTE_error() to publish error messages
//
// changes:
// 13 Dec 2000 (AG) creation
// 14 Dec 2000 (BP) Added three histograms
//
// ******************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>

// ONLINE includes
#include "rhp.h"

// Arte includes
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "dq/DQ.h"
#include "dq/DQ_SLT.h"
#include "sltinfo.h"
#include "sltarte/matchslt.hh"

// externals
extern int FADEB;

// from include file "slt_codes.h"
#define SLT_BX_VETO  0x2

// histogram ids
#define TriggerType_ID DQ_ID( COMP_SLT, LVL_REC, 1 )
#define MatchSLT_ID DQ_ID( COMP_SLT, LVL_REC, 2 )
#define BXSuppression_ID DQ_ID( COMP_SLT, LVL_REC, 3 )

// bin ids
enum {
  total=0,
  random,
  bypass_e,
  bypass_mu,
  triggered_e,
  triggered_mu
};

//
// function code
//
int
DQ_SLT_rec_init()
{
  ARTE_mark( "DQ_SLT_rec_init", 0 );
  cout << "DQ_SLT_rec_init" << endl;

  // print (online to grr)
  //ARTE_error( GRR_SFATAL, "I am dead!", 99 );
   
  // book 1-dim histograms
  rhp_hbook1( TriggerType_ID,"SLT: Trigger Type flags", 20, -0.5, 19.5 );
  rhp_hbook1( MatchSLT_ID,"SLT: Matching flags", 10, -0.5, 9.5 );
  rhp_hbook1( BXSuppression_ID,"SLT: Suppressed BX", 256, -0.5, 255.5 );

  // book 2-dim histograms
  //rhp_hbook2( DQ_ID( COMP_SLT, LVL_REC, histogram_id2 ),
  //            "This is a test", nbinx, xmin, xmax, nbiny, ymin, ymax );

  ARTE_mark( "DQ_SLT_rec_init", 0 );
  return 0; // return error code (0 = okay)
}

//
// function code
//
int
DQ_SLT_rec_evnt()
{
  ARTE_mark( "DQ_SLT_rec_evnt 0", 0 );
  
  // Fill info about trigger types
  rhp_hf1( TriggerType_ID, total, 1 );

  int daqcode=faDDAQFCS_tcode();
  if (daqcode>=8)  rhp_hf1( TriggerType_ID, random, 1 );

  if (SLTReadDSLT()==SLT_OK) {
    SLT_STATUS stat;
    if (SLTStatus(&stat)==SLT_OK) {
      SLT_SEEDTRACK seed;
      SLTSeedTrackInfo(0,&seed);
      if (seed.trackID==EPLUS||seed.trackID==EMINUS) {
	if (stat.status&SLTEVENT_BYPASS) 
	  rhp_hf1( TriggerType_ID, bypass_e, 1 );
	if (stat.status&SLTEVENT_ACCEPT) 
	  rhp_hf1( TriggerType_ID, triggered_e, 1 );
      } else { //muons
	if (stat.status&SLTEVENT_BYPASS) 
	  rhp_hf1( TriggerType_ID, bypass_mu, 1 );
	if (stat.status&SLTEVENT_ACCEPT) 
	  rhp_hf1( TriggerType_ID, triggered_mu, 1 );
      }
    }
  }


  // fill info on matching (for ghost rates)  
  TrigType type;
  ArtePointer<RTRA> rtra1,rtra2;
  MatchSLT(rtra1,rtra2,type);
  rhp_hf1( MatchSLT_ID, type, 1 );

  // fill info on bx's suppressed by the SLT
  // only available in randoms (the others are suppressed)

  if (daqcode>=8) { 

    int bx=faDDAQphysBX();
    int sltstatus=faDDAQslt_status();

    rhp_hf1( BXSuppression_ID, 255, 1 ); //normalisation bin
    if (sltstatus&SLT_BX_VETO) rhp_hf1( BXSuppression_ID, bx, 1 );
  }

  ARTE_mark( "DQ_SLT_rec_evnt 1", 0 );
  return 0; // return error code (0 = okay)
}
