
// *****************************************************************************
//
// source:    DQ.C
//
// type:      source code
//
// created:   16 Nov 1999
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// called by: DQ routines
//
// changes:
// 17 Nov 1999 (AG) remove print-outs
// 23 Nov 1999 (AG) repair storing of inital settings
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// ONLINE includes
#include "table.h"

// Arte includes
#include "dq/DQ.h"

// externals
extern int DQ_SVD;
extern int DQ_ITR;
extern int DQ_HIPT;
extern int DQ_OTR;
extern int DQ_RICH;
extern int DQ_TRD;
extern int DQ_ECAL;
extern int DQ_MUON;
extern int DQ_FLT;
extern int DQ_SLT;
extern int DQ_TLT;
extern int DQ_FARM;
extern int DQ_CORR;

extern int FADEB;

//
// function
//
int
DQ_db()
{
  // show up
  //cout << "DQ_db_()" << endl;

  // read DQ_DB
  char *dbName = "DQ_DB";
  char *dqdb = getenv( dbName );
  if( !dqdb ){
    if( FADEB ){
      cout << "DQ_db(): No DB path given in environment variable: "
	   << dbName << endl;
    }
    return -1;
  }

  // print
  //cout << "DQ_db(): DQ_DB=" << dqdb << endl;
  
  // open tables
  Tables x = OpenTables( dqdb, TABLEreadonly );
  if( !x ){
    cout << "DQ_db(): ERROR: OpenTables() DQ_DB=" << dbName << endl;
    return -2;
  }

  // book tables
  char *tableName = "dq_config.dat";
  int tx = BookTable( x, tableName, 0, 0 );
  if( !tx ){
    cout << "DQ_db(): ERROR: BookTables() table=" << tableName << endl;
    return -3;
  }
  
  // stamp
  TableStamp( x, TABLEmaxrev, TABLEmaxrev );
  
  // fetch
  int rc = FetchTable( x, tx, TABLEle );
  if( !rc ){
    cout << "DQ_db(): ERROR: FetchTables() table=" << tableName << endl;
    return -3;
  }
  SkipTableField( x, 1 );
  
  // get 
  unsigned short int data[16];
  GetTableField( x, TABLEushort, 16, data );

  // interpret
  if( FADEB ){
    cout << "DQ_db(): Settings in DB:"   << endl;
    cout << "         SVD= " << data[0]  << endl;
    cout << "         ITR= " << data[1]  << endl;
    cout << "         HIPT=" << data[2]  << endl;
    cout << "         OTR= " << data[3]  << endl;
    cout << "         RICH=" << data[4]  << endl;
    cout << "         TRD= " << data[5]  << endl;
    cout << "         ECAL=" << data[6]  << endl;
    cout << "         MUON=" << data[7]  << endl;
    cout << "         FLT= " << data[8]  << endl;
    cout << "         SLT= " << data[9]  << endl;
    cout << "         TLT= " << data[10] << endl;
    cout << "         FARM=" << data[11] << endl;
    cout << "         CORR=" << data[12] << endl;
    cout << "         USR1=" << data[13] << endl;
    cout << "         USR2=" << data[14] << endl;
    cout << "         USR3=" << data[15] << endl;
  }

  // before
  //DQ_print( "DQ_db(): BEFORE" );

  // store inital settings
  static int first = 1;
  static int DQ_SVD_init  = 0;
  static int DQ_ITR_init  = 0;
  static int DQ_HIPT_init = 0;
  static int DQ_OTR_init  = 0;
  static int DQ_RICH_init = 0;
  static int DQ_TRD_init  = 0;
  static int DQ_ECAL_init = 0;
  static int DQ_MUON_init = 0;
  static int DQ_FLT_init  = 0;
  static int DQ_SLT_init  = 0;
  static int DQ_TLT_init  = 0;
  static int DQ_FARM_init = 0;
  static int DQ_CORR_init = 0;
  if( first ){
    first = 0;
    DQ_SVD_init  = DQ_SVD;
    DQ_ITR_init  = DQ_ITR;
    DQ_HIPT_init = DQ_HIPT;
    DQ_OTR_init  = DQ_OTR;
    DQ_RICH_init = DQ_RICH;
    DQ_TRD_init  = DQ_TRD;
    DQ_ECAL_init = DQ_ECAL;
    DQ_MUON_init = DQ_MUON;
    DQ_FLT_init  = DQ_FLT;
    DQ_SLT_init  = DQ_SLT;
    DQ_TLT_init  = DQ_TLT;
    DQ_FARM_init = DQ_FARM;
    DQ_CORR_init = DQ_CORR;
  }

  // logical AND with tables
  DQ_SVD  = DQ_SVD_init  & data[0];
  DQ_ITR  = DQ_ITR_init  & data[1];
  DQ_HIPT = DQ_HIPT_init & data[2];
  DQ_OTR  = DQ_OTR_init  & data[3];
  DQ_RICH = DQ_RICH_init & data[4];
  DQ_TRD  = DQ_TRD_init  & data[5];
  DQ_ECAL = DQ_ECAL_init & data[6];
  DQ_MUON = DQ_MUON_init & data[7];
  DQ_FLT  = DQ_FLT_init  & data[8];
  DQ_SLT  = DQ_SLT_init  & data[9];
  DQ_TLT  = DQ_TLT_init  & data[10];
  DQ_FARM = DQ_FARM_init & data[11];
  DQ_CORR = DQ_CORR_init & data[12];

  // after
  //DQ_print( "DQ_db(): AFTER" );

  // close
  CloseTables( x );

  return 0;
}
