
// *****************************************************************************
//
// source:    faInit.C
//
// type:      source code
//
// created:    7 Jul 1997
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// symbol:    fainit_
//
// input:     void
// return:    int  ARTE_error_code
//
// called by: ATINIT
//
// purporse:  initialization of farm stuff called for FARM mode only
//
// changes:         
//                  initialize and register rpm (AG: 25-Nov-98)
//                  multiple file input (AG: 27-Nov-98)
//                  daq format logging (AG: 1-Dec-98)
//                  includes (AG: 11 Jan 99)
//                  removed emg status flag (AG: 12 Jan 99)
//                  obey ANSI rules (AG: 15 Jan 99)
//                  call logger initialization at the end (AG: 21 Jan 99)
//                  rpm register (AG: 2 Feb 99)
//                  some printing removed (AG: 10 Feb 99)
//                  removed faUtil.hh (AG: 23 Feb 99)
//                  use grr for error logging (AG: 25 Feb 99)
//                  polishing (AG: 11 Mar 99)
//                  put includes (AG: 1 Jun 99)
// 18 Aug 1999 (AG) polishing
// 09 Sep 1999 (AG) always use cout
// 08 Oct 1999 (AG) write start-up to log-file
// 18 Oct 1999 (AG) add command line arguments to log-file output
// 01 Nov 1999 (AG) re-try rpm_open() if it fails
// 03 Nov 1999 (AG) add process id to logfile
// 11 Nov 1999 (AG) improve log file
// 24 Nov 1999 (AG) use ARTE_error()
// 24 Nov 1999 (AG) write settings into logfile
// 25 Nov 1999 (AG) introduce SMC initialization
// 29 Nov 1999 (AG) print the configuration into the logfile
// 01 Dec 1999 (AG) finish logfile output
// 06 Dec 1999 (AG) initialize pinging
// 07 Dec 1999 (AG) publish event count
// 10 Dec 1999 (AG) publish more quantities
// 23 Feb 2000 (AG) write Arte version number to log file
// 24 Feb 2000 (AG) fixed gethostname problem
// 29 Feb 2000 (AG) linux -> __linux
// 14 Mar 2000 (AG) cooperate with SMC signal handler
// 27 Apr 2000 (AG) use ARTE_mark()
// 28 Apr 2000 (AG) use ARTE_log()
// 05 May 2000 (AG) improve output
// 15 May 2000 (AG) add rhitmask
// 25 May 2000 (AG) add more ARTE_mark()
//  2 Jun 2000 (AG) add more ARTE_mark()
// 14 Jun 2000 (AG) solve potential SEGV
// 16 Jun 2000 (AG) add more ARTE_mark()
// 23 Jun 2000 (AG) add multi timer start
// 27 Jun 2000 (AG) corrected time
// 28 Jun 2000 (AG) commented faMultiTimer() for the time being 
// 11 Jul 2000 (AG) use RHPPUB
// 12 Jul 2000 (AG) improve printout
// 03 Aug 2000 (AG) commissioned trigger settings
// 11 Aug 2000 (AG) move global L4TSET to l4trig/l4t.C
// 11 Dec 2000 (AG) exchanged some ARTE_error() by ARTE_exit()
// 18 Jan 2001 (AG) removed obsolete faDaqLog()
// 27 Jun 2001 (Jose Hernandez) removed bug
// 01 Oct 2001 (JH) some printouts removed 
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>

// ONLINE includes
#include "rpm.h"
#include "rpi.h"
#include "smc.h"
#include "rhp.h"
#include "grr.h"
#include "fld.h"
#include "par.h"
#include "log.h"
#include "runcfg.h"

#define PRCMHITMASK "HITmask"

// ARTE includes
#include "arte/ArteFlag.h"
#include "farm/fa.hh"
#include "farm/farm.hh"

// globals
char *evtBuf = 0;  // event buffer
int  nodeID  = -1; // default 4LT node ID

// externals
extern int  nArg;   // line command arguments
extern char **vArg; // line command arguments

extern int  L4TSET;   // L4T (0: 4LT disabled, 1: 4LT enabled)
extern int  FADEB;    // debug flag
extern int  FARM;     // FARM mode
extern int  FAON;     // FARM online mode
extern char *DBNAME;  // database name
extern char *LOGFILE; // log file
extern int  NOSPAR;   // sparsification
extern int  RPMINIT;  // init rpm/rhp
extern int  RHPPUB;   // rhp_publish()
extern int  FAGRR;    // error logger
extern char *PARNAME; // emg partition
extern int  SEG_NUM;  // emg partitions
extern int  SEG_LEN;  // emg partition size

// C++ prototypes
void faCatchSIG( int sig );
SMC_CODES faSMCinit2conf ( void *dummy1, void *dummy2 );
SMC_CODES faSMCready2exit( void *dummy1, void *dummy2 );

//
// program code
//
extern "C" int
fainit_()
{
  // show up
  ARTE_mark( "fainit_ 0", 0 );

  // 
  // init signal catchers
  //
  if( FARM ){
    signal( SIGINT,  faCatchSIG ); //  2
    signal( SIGQUIT, faCatchSIG ); //  3
    signal( SIGTERM, faCatchSIG ); // 15
    signal( SIGILL,  faCatchSIG ); //  4
    signal( SIGBUS,  faCatchSIG ); // 10
    signal( SIGSEGV, faCatchSIG ); // 11
  }

  // component flags
  char *rcompmask = getenv( RC_Comp_Mask_ENV );
  int compMask = 0;
  if( rcompmask ){
    compMask = atoi( rcompmask );

    cout << "INIT: Components:     ";
    //cout << "(" << compMask << ") ";

    if( compMask & COMP_M_SVD    ) cout << "SVD ";
    if( compMask & COMP_M_ITR    ) cout << "ITR ";
    if( compMask & COMP_M_HIPT   ) cout << "HIPT ";
    if( compMask & COMP_M_OTR    ) cout << "OTR ";
    if( compMask & COMP_M_RICH   ) cout << "RICH ";
    if( compMask & COMP_M_TRD    ) cout << "TRD ";
    if( compMask & COMP_M_ECAL   ) cout << "ECAL ";
    if( compMask & COMP_M_MUON   ) cout << "MUON ";
    if( compMask & COMP_M_MPIX   ) cout << "MPIX ";
    if( compMask & COMP_M_MPAD   ) cout << "MPAD ";
    if( compMask & COMP_M_FLT    ) cout << "FLT ";
    if( compMask & COMP_M_SLT    ) cout << "SLT ";
    if( compMask & COMP_M_FARM   ) cout << "FARM ";
    if( compMask & COMP_M_TARG   ) cout << "TARG ";
    if( compMask & COMP_M_FCS    ) cout << "FCS ";
    if( compMask & COMP_M_COMMON ) cout << "COMMON ";

    cout << endl;    
  }

  // reco flags
  char *rrecomask = getenv( PRCMRECOMASK );
  int recoMask = 0;
  if( rrecomask ){
    recoMask = atoi( rrecomask );
    
    cout << "INIT: Reconstruction: ";
    //cout << "(" << recoMask << ") ";

    if( recoMask & COMP_M_SVD    ) cout << "SVD ";
    if( recoMask & COMP_M_ITR    ) cout << "ITR ";
    if( recoMask & COMP_M_HIPT   ) cout << "HIPT ";
    if( recoMask & COMP_M_OTR    ) cout << "OTR ";
    if( recoMask & COMP_M_RICH   ) cout << "RICH ";
    if( recoMask & COMP_M_TRD    ) cout << "TRD ";
    if( recoMask & COMP_M_ECAL   ) cout << "ECAL ";
    if( recoMask & COMP_M_MUON   ) cout << "MUON ";
    if( recoMask & COMP_M_MPIX   ) cout << "MPIX ";
    if( recoMask & COMP_M_MPAD   ) cout << "MPAD ";
    if( recoMask & COMP_M_FLT    ) cout << "FLT ";
    if( recoMask & COMP_M_SLT    ) cout << "SLT ";
    if( recoMask & COMP_M_FARM   ) cout << "FARM ";
    if( recoMask & COMP_M_TARG   ) cout << "TARG ";
    if( recoMask & COMP_M_FCS    ) cout << "FCS ";
    if( recoMask & COMP_M_COMMON ) cout << "COMMON ";

    cout << endl;
  }

  // hit flags
  char *rhitmask = getenv( PRCMHITMASK );
  int hitMask = 0;
  if( rhitmask ){
    hitMask = atoi( rhitmask );
    
    cout << "INIT: Hits:           ";
    //cout << "(" << hitMask << ") ";

    if( hitMask & COMP_M_SVD    ) cout << "SVD ";
    if( hitMask & COMP_M_ITR    ) cout << "ITR ";
    if( hitMask & COMP_M_HIPT   ) cout << "HIPT ";
    if( hitMask & COMP_M_OTR    ) cout << "OTR ";
    if( hitMask & COMP_M_RICH   ) cout << "RICH ";
    if( hitMask & COMP_M_TRD    ) cout << "TRD ";
    if( hitMask & COMP_M_ECAL   ) cout << "ECAL ";
    if( hitMask & COMP_M_MUON   ) cout << "MUON ";
    if( hitMask & COMP_M_MPIX   ) cout << "MPIX ";
    if( hitMask & COMP_M_MPAD   ) cout << "MPAD ";
    if( hitMask & COMP_M_FLT    ) cout << "FLT ";
    if( hitMask & COMP_M_SLT    ) cout << "SLT ";
    if( hitMask & COMP_M_FARM   ) cout << "FARM ";
    if( hitMask & COMP_M_TARG   ) cout << "TARG ";
    if( hitMask & COMP_M_FCS    ) cout << "FCS ";
    if( hitMask & COMP_M_COMMON ) cout << "COMMON ";

    cout << endl;
  }
 
  // configuration
  int runName = 0;
  int runType = 0;
  int logType = 0;
  int trigger = 0;
  char *configuration = getenv( PRCMRUNCFG );
  unsigned int confMask = 0;
  if( configuration ){
    confMask = atoi( configuration );

    cout << "INIT: Configuration:  ";
    //cout << "(" << configuration << ") ";
    
    // settings
    runName = RCFG_GET_NAME    ( confMask );
    runType = RCFG_GET_TYPE    ( confMask );
    logType = RCFG_GET_LOGGING ( confMask );
    trigger = RCFG_GET_TRIGGERS( confMask );

    // name  
    if( runName == RCFG_NM_PRODUCTION ) cout << "Production ";
    if( runName == RCFG_NM_LOCAL      ) cout << "Local ";
    
    // type
    if( runType == RCFG_TP_PHYSICS     ) cout << "Physics ";
    if( runType == RCFG_TP_CALIBRATION ) cout << "Calibration ";
    if( runType == RCFG_TP_INTERACTION ) cout << "Interaction ";
    if( runType == RCFG_TP_BX          ) cout << "BX ";
    if( runType == RCFG_TP_TEST        ) cout << "Test ";
    
    // logging
    if( logType == RCFG_LG_ARCHIVE ) cout << "Archive ";
    if( logType == RCFG_LG_NONE    ) cout << "None";
    if( logType == RCFG_LG_DISK    ) cout << "Disk ";
    if( logType == RCFG_LG_TAPE    ) cout << "Tape ";
    
    // trigger
    if( trigger & RCFG_TL_FIRST  ) cout << "FLT ";
    if( trigger & RCFG_TL_SECOND ) cout << "SLT ";
    if( trigger & RCFG_TL_THIRD  ) cout << "TLT ";
    if( trigger & RCFG_TL_FOURTH ) cout << "4LT ";

    // 4LT
    L4TSET = 0;
    if( trigger & RCFG_TL_FOURTH ) L4TSET = 1;

    cout << endl;
  }

  //
  // write to log-file
  //
  static char *txt = 0;
  if( !txt ) txt = new char[300];
  memset( txt, '\0', 300 );

  sprintf( txt, "Arte:%.4f comp:%d reco:%d hit:%d name:%d type:%d logging:%d trigger:%d",
	   arteflag_.artevers,
	   compMask,
	   recoMask,
	   hitMask,
	   runName,
	   runType,
	   logType,
	   trigger );
  int len = 0;
  for( int i=0; i<nArg; i++ ){
    len += strlen( vArg[i] ) + 1;
    if( len < 300 ){
      strcat( txt, " " );
      strcat( txt, vArg[i] );
    }
  }
  ARTE_log( LOGFILE, txt );

  //
  // namserver. No message to error logger till rpm is initialized!
  //
  char *namsrv = getenv( "HBNAMEROOT" );
  if( namsrv ) printf("INIT: nameserver: %s\n", namsrv);
  
  //
  // rpm
  //
  int rc = GRR_OK;
  if( RPMINIT ){

    ARTE_mark( "rpm/smc/rhp/par/multi 0" , 0 );
  
    //
    // rpm/smc/rhp/par *********************************************************
    //
    int count = 0;
    while( 1 ){
      count++;
      
      // exit when many trails fail
      if( count > 5 ) faCatchSIG( -1 );

      //
      // rpm *******************************************************************
      //
      ARTE_mark( "rpm_init", 0 );
      
      // rpm init
      if( ( rc = rpm_init( nArg, (const char **) vArg ) ) == GRR_OK ){
	cout << "INIT: rpm_init rc=" << rc << endl; 
      }
      // error
      else {
	cout << "INIT: rpm_init FAILED. rc=" << rc << endl;
	// grr
	ARTE_exit( GRR_SFATAL, "INIT: rpm_init FAILED.", rc );
	
	// rpm_halt()
	rc = rpm_halt();

	// try again after some time
	sleep( 5 );
	continue;
      }
      
      // rpm open
      ARTE_mark( "rpm_open", 0 );
      if( ( rc = rpm_open( RPM_REGISTER ) ) == GRR_OK ){
	cout << "INIT: rpm_open( RPM_REGISTER ) rc=" << rc << endl;
      }
      // error
      else {
	cout << "INIT: rpm_open FAILED. rc=" << rc << endl;

	// grr
	ARTE_exit( GRR_SFATAL, "INIT: rpm_open FAILED.", rc );
	
	// rpm_halt()
	rc = rpm_halt();

	// try again after some time
	sleep( 5 );
	continue;
      }

      // initialize pinging
      ARTE_mark( "rpm_bind", 0 );
      if( ( rc = rpm_bind( RPM_STATREQ, rpi_statmsgx, 0 ) ) == GRR_OK ){
	cout << "INIT: rpm_bind( RPM_STATREQ ) rc=" << rc << endl;
      }
      // error
      else {
	cout << "INIT: rpm_bind( RPM_STATREQ ) FAILED. rc=" << rc << endl;

	// grr
	ARTE_exit( GRR_SFATAL, "INIT: rpm_bind( RPM_STATREQ ) FAILED.", rc );
	
	// rpm_halt()
	rc = rpm_halt();
	
	// try again after some time
	sleep( 5 );
	continue;
      }
      
      // grr
      if (FADEB) ARTE_error( GRR_SINFO, "INIT: RPM DONE.", 0 );

      //
      // smc *******************************************************************
      //

      // smc init
      ARTE_mark( "smc_init", 0 );
      if( ( rc = smc_init( nArg, (const char **) vArg ) ) == GRR_OK ){
	cout << "INIT: smc_init rc=" << rc << endl; 
      }
      // error
      else {
	cout << "INIT: smc_init FAILED. rc=" << rc << endl;

	// grr
	ARTE_exit( GRR_SFATAL, "INIT: smc_init FAILED.", rc );
	
	// rpm_halt()
	rc = rpm_halt();

	// try again after some time
	sleep( 5 );
	continue;
      }

      // let SMC know about our signal handler
      ARTE_mark( "smc_add_user_exit_handler", 0 );
      smc_add_user_exit_handler( faCatchSIG );

      // smc bind
      ARTE_mark( "smc_bind", 0 );
      if( ( rc = smc_bind() ) == GRR_OK ){
	cout << "INIT: smc_bind rc=" << rc << endl; 
      }
      // error
      else {
	cout << "INIT: smc_bind FAILED. rc=" << rc << endl;

	// grr
	ARTE_exit( GRR_SFATAL, "INIT: smc_bind FAILED.", rc );
		
	// rpm_halt()
	rc = rpm_halt();
	
	// try again after some time
	sleep( 5 );
	continue;
      }
    
      // smc define state transitions
      ARTE_mark( "smc_add_st_fn", 0 );
      smc_add_st_fn( smc_state_atoi( "INITIALIZED" ),
		     smc_state_atoi( "CONFIGURED" ),
		     faSMCinit2conf );
      
      smc_add_st_fn( smc_state_atoi( "READY" ),
		     smc_state_atoi( "EXIT" ),
		     faSMCready2exit );
      
      //
      // smc start up
      //
      ARTE_mark( "smc_startup", 0 );
      if( ( rc = smc_startup( NULL, nArg, (const char **) vArg ) ) == GRR_OK ){
	cout << "INIT: smc_startup rc=" << rc << endl; 
      }
      // error
      else {
	cout << "INIT: smc_startup FAILED. rc=" << rc << endl;

	// grr
	ARTE_exit( GRR_SFATAL, "INIT: smc_startup FAILED.", rc );
	
	// rpm_halt()
	rc = rpm_halt();
	
	// try again after some time
	sleep( 5 );
	continue;
      }

      // grr
      if (FADEB) ARTE_error( GRR_SINFO, "INIT: SMC DONE.", 0 );

      // quit loop
      break;
    }

    //
    // rhp *********************************************************************
    //
    if( RHPPUB ){

      // msinit
      ARTE_mark( "rhp_msinit", 0 );
      if( ( rc = rhp_msinit() ) == GRR_OK ){
	cout << "INIT: rhp_msinit rc=" << rc << endl;
      }
      else {
	cout << "INIT: rhp_msinit FAILED. rc=" << rc << endl;
	ARTE_error( GRR_SERROR, "INIT: rhp_msinit FAILED.", rc );
      }

      // msbind
      ARTE_mark( "rhp_msbind", 0 );
      if( ( rc = rhp_msbind() ) == GRR_OK ){
	cout << "INIT: rhp_msbind rc=" << rc << endl;
      }
      else {
	cout << "INIT: rhp_msbind FAILED. rc=" << rc << endl;
	ARTE_error( GRR_SERROR, "INIT: rhp_msbind FAILED.", rc );
      }
    }
    // old mode
    else {

      // gsinit
      ARTE_mark( "rhp_gsinit", 0 );
      if( ( rc = rhp_gsinit() ) == GRR_OK ){
	cout << "INIT: rhp_gsinit rc=" << rc << endl;
      }
      else {
	cout << "INIT: rhp_gsinit FAILED. rc=" << rc << endl;
	ARTE_error( GRR_SINFO, "INIT: rhp_gsinit.", rc );
      }
      
      // gsbind
      ARTE_mark( "rhp_gsbind", 0 );
      if( ( rc = rhp_gsbind() ) == GRR_OK ){
	cout << "INIT: rhp_gsbind rc=" << rc << endl;
      }
      else {
	cout << "INIT: rhp_gsbind FAILED. rc=" << rc << endl;
	ARTE_error( GRR_SINFO, "INIT: rhp_gsbind.", rc );
      }
    }

    // grr
    if (FADEB) ARTE_error( GRR_SINFO, "INIT: RHP DONE.", 0 );

    //
    // par *********************************************************************
    //
    ARTE_mark( "par_sinit", 0 );
    if( ( rc = par_sinit() ) != GRR_OK ){
      ARTE_error( GRR_SERROR, "INIT: par_sinit FAILED.", rc );
    }

    ARTE_mark( "par_sbind", 0 );
    if( ( rc = par_sbind()) != GRR_OK ){
      ARTE_error( GRR_SERROR, "INIT: par_sbind FAILED.", rc );
    }

    // book
    ARTE_mark( "par_book", 0 );
    rc = par_book( "Run Name",     PAR_T_INT |PAR_T_RDONLY|PAR_T_PUBLISH, 1,  &runName );
    rc = par_book( "Run Type",     PAR_T_INT |PAR_T_RDONLY|PAR_T_PUBLISH, 1,  &runType );
    rc = par_book( "Logging Type", PAR_T_INT |PAR_T_RDONLY|PAR_T_PUBLISH, 1,  &logType );
    rc = par_book( "Trigger",      PAR_T_INT |PAR_T_RDONLY|PAR_T_PUBLISH, 1,  &trigger );
    rc = par_book( "Event Count",  PAR_T_INT |PAR_T_RDONLY|PAR_T_PUBLISH, 1,  &arteflag_.numevt );
    rc = par_book( "Run Number",   PAR_T_INT |PAR_T_RDONLY|PAR_T_PUBLISH, 1,  &arteflag_.numrun );
    rc = par_book( "Exp Number",   PAR_T_INT |PAR_T_RDONLY|PAR_T_PUBLISH, 1,  &arteflag_.numexp );

    // grr
    if (FADEB) ARTE_error( GRR_SINFO, "INIT: PAR DONE.", 0 );
    
    ARTE_mark( "rpm/smc/rhp/par 1" , 0 );
  }
  
  //
  // farm mode *****************************************************************
  //
  if( FARM ){
    
    //
    // init buffer
    //
    ARTE_mark( "evtBuf" , 0 );
    if( ( evtBuf = new char[SEG_LEN] ) != 0 ){
      cout << "INIT: event buffer size=" << SEG_LEN << endl;
    }
    else {
      cout << "INIT: ERROR: event buffer" << endl;
      return ARTE_ERR;
    }
    
    //
    // online running
    //
    if( FAON ){
      
      // init emg
      ARTE_mark( "faInitEmg", 0 );
      if( ( rc = faInitEmg() ) != GRR_OK ){
	ARTE_exit( GRR_SFATAL, "INIT: emg init FAILED.", rc );
	return ARTE_ERR;
      }
    }
  }

  // grr error message
  ARTE_error( GRR_SINFO, "INIT: faInit DONE.", 0 );

  // mark
  ARTE_mark( "fainit_ 1", 0 );
  
  // return
  return ARTE_GOOD;
}

//
// SMC functions
//
SMC_CODES faSMCinit2conf ( void *dummy1, void *dummy2 )
{
  cout << "INIT: SMC INITIALIZED -> CONFIGURED" << endl;
  static int first = 1;
  SMC_CODES rc = SMC_OK;
  if( first ){
    first = 0;
    rc = SMC_WAIT;
  }
  else {
    first = 1;
    rc = SMC_OK;
  }

  // grr
  ARTE_error( GRR_SINFO, "INIT: SMC INITIALIZED -> CONFIGURED ", rc );

  return rc;
}

//
// SMC function
//
SMC_CODES faSMCready2exit( void *dummy1, void *dummy2 )
{
  cout << "INIT: SMC READY -> EXIT" << endl;

  static int first = 1;
  SMC_CODES rc = SMC_OK;
  if( first ){
    first = 0;
    rc = SMC_WAIT;
  }
  else {
    first = 1;
    rc = SMC_OK;
  }

  // grr
  ARTE_error( GRR_SINFO, "INIT: READY -> EXIT ", rc );

  // exit gracefully
  faCatchSIG( 0 );

  return rc;
}
