// ********************************************************************
//
//  Routine:      gageFillRhpSegments
//  Author:       T. Oest
//
//  Description:  fill segments pairs in rhp ring buffer
//
//  Modification:
//
// ********************************************************************

// includes
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <algo.h>

#include "arte/arte.hh"
#include "gage/gage.hh"
#include "arte/RSEG_cmp.hh"
#include "rhp.h"
#include "dq/DQ.h"

extern FILE * gageFile;

void gageFitOtr(ArtePointer<RSEG>, float * , float * , float *);

//*****************************************************************
// function
//*****************************************************************

void gageFillRhpSegments(int cmpRSEG, int cmpAlign, float * Cuts){

if (cmpRSEG & cmpAlign) return;

  if (cmpRSEG  == Rsegc::vxd &&
      cmpAlign == Rsegc::patt) {


//-----------------------------------------------------------------
//  loop over segments, select only requested segments
//-----------------------------------------------------------------

  for( ArteTable<RSEG>::iterator rseg = ArteTable<RSEG>::begin();
       rseg != ArteTable<RSEG>::end(); ++rseg ) {

  if ((*rseg)->fit == 0 && ((*rseg)->cmp & cmpRSEG)==cmpRSEG ){

//-----------------------------------------------------------------
//  loop over subdetector segments
//-----------------------------------------------------------------

  for( ArteTable<RSEG>::iterator rsegAlign = ArteTable<RSEG>::begin();
       rsegAlign != ArteTable<RSEG>::end(); ++rsegAlign ) {

  if (((*rsegAlign)->fit == 0 || (*rsegAlign)->fit == 8) &&
       ((*rsegAlign)->cmp & cmpAlign)) {

//-----------------------------------------------------------------
//  trace silicon track to SI06
//-----------------------------------------------------------------

  float z0;
  int ErrorFlag;

  z0 = 160.0;
  //  z0 = 98.588;

  float pAlign[6];
  float pAlignTraced[6];

  pAlign[0] = (*rsegAlign)->zf;
  pAlign[1] = (*rsegAlign)->xf;
  pAlign[2] = (*rsegAlign)->yf;
  pAlign[3] = (*rsegAlign)->txf;
  pAlign[4] = (*rsegAlign)->tyf;
  pAlign[5] = (*rsegAlign)->pf;

  ErrorFlag = gageTraceTRACK(z0 , pAlign, pAlignTraced);
  //  if (ErrorFlag != 0) cout << " gageTrace error " << ErrorFlag << endl;

  if ( ErrorFlag >= 0) {

//-----------------------------------------------------------------
//  VDS OTR fit
//-----------------------------------------------------------------
  float chi2;
  float p0[3];
  float pTrack[6];

  p0[0] = pAlignTraced[1];
  p0[1] = pAlignTraced[2];
  p0[2] = pAlignTraced[0];

  pAlign[0] = (*rseg)->ze;
  pAlign[1] = (*rseg)->xe;
  pAlign[2] = (*rseg)->ye;
  pAlign[3] = (*rseg)->txe;
  pAlign[4] = (*rseg)->tye;
  pAlign[5] = (*rsegAlign)->pf;

  ErrorFlag = gageTraceTRACK(z0 , pAlign, pTrack);
  //  if (ErrorFlag != 0) cout << " gageTrace error " << ErrorFlag << endl;
  chi2 = 0.0;
  //  gageFitOtr(*rseg, p0, pTrack, &chi2);

//-----------------------------------------------------------------
//  cuts on Alignment segment segments
//-----------------------------------------------------------------

  if (chi2 < 20 &&  ErrorFlag >= 0){

    /*
  cout << " vds segment " << endl;
  cout << " z   " << pAlignTraced[0] << endl;
  cout << " x   " << pAlignTraced[1] << endl;
  cout << " y   " << pAlignTraced[2] << endl;
  cout << " tx  " << pAlignTraced[3] << endl;
  cout << " ty  " << pAlignTraced[4] << endl;
  cout << " rho " << pAlignTraced[5] << endl;

  cout << " otr track traced to segment " << chi2 << endl;
  cout << " z   " << pTrack[0] << endl;
  cout << " x   " << pTrack[1] << endl;
  cout << " y   " << pTrack[2] << endl;
  cout << " tx  " << pTrack[3] << endl;
  cout << " ty  " << pTrack[4] << endl;
  cout << " rho " << pTrack[5] << endl;
    */

  float dx, dy, dtx, dty;

  dx    = pTrack[1] - pAlignTraced[1];
  dy    = pTrack[2] - pAlignTraced[2];
  dtx   = pTrack[3] - pAlignTraced[3];
  dty   = pTrack[4] - pAlignTraced[4];

  /*
  cout << " cuts " << endl;
  cout << " dx  " << dx << endl;
  cout << " dy  " << dy << endl;
  cout << " dtx " << dtx << endl;
  cout << " dty " << dty << endl;
   */


 //  cout << " goodAlignmentSegment  " << goodAlignmentSegment << endl;

//-----------------------------------------------------------------
//  fill structure and pass to buffer
//-----------------------------------------------------------------

  if (fabs(dx) < Cuts[0] && fabs(dy) < Cuts[1] &&
      fabs(dtx)< Cuts[2] && fabs(dty)< Cuts[3] &&
      (Cuts[4] == 0.0 ||  1./fabs(pTrack[5]) > Cuts[4])) {

     GageSegmentPair SegmentPair;

     SegmentPair.key    = ArtePointer<EVHD>(1)->dbmajor;
     SegmentPair.z      = pAlignTraced[0];
     SegmentPair.p1[0]  = pAlignTraced[1];
     SegmentPair.p1[1]  = pAlignTraced[2];
     SegmentPair.p1[2]  = pAlignTraced[3];
     SegmentPair.p1[3]  = pAlignTraced[4];

     for (int i=0; i<10; i++) SegmentPair.V1[i] = (*rsegAlign)->cvf[i];

     SegmentPair.p2[0]  = pTrack[1];
     SegmentPair.p2[1]  = pTrack[2];
     SegmentPair.p2[2]  = pTrack[3];
     SegmentPair.p2[3]  = pTrack[4];

     for (int i=0; i<10; i++) SegmentPair.V2[i] = 0.0;


//-----------------------------------------------------------------
//  write to file or copy in rhp buffer
//-----------------------------------------------------------------

     if (gageFile != 0 ){

      fwrite(&SegmentPair, sizeof(GageSegmentPair), 1, gageFile);
      cout << " track written to file " << endl;

     } else {

      int size, currentsize;
      char *buf = rbf_obtain(AC_ID( COMP_CORR, LVL_REC, 1),
                             RBF_READ, &size, &currentsize); 
 
      int newsize = sizeof(GageSegmentPair) / sizeof(char);

      memcpy(buf, &SegmentPair, newsize);

      rbf_release(AC_ID( COMP_CORR, LVL_REC, 1), 
                  buf, newsize);

 //      cout << " track written to rhp buffer " << newsize << endl;

     };

    };
    };
    };


  };
  };
  };
  };
 };
 };

