/*************************************************************************
*
* CR_ARTE
* written by Siegmund Nowak
*
* This routines fills the arte NEDE, NWAL and NCAL banks from GDL and GEANT
*
*       called from detgeo
*
*************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteUtil.h"
#include "arte/ArteMessage.hh"
#include "gean/gclink.h"
#include "gean/gcvolu.h"
#include "gean/zgeom.h"
#include "gean/ZGESL.h"
#include "gean/Geant2Arte.hh"

extern "C" {
    void crlvbk_(char*,int*,int*,int*,char*,int[]);
    void crvtrm_(int&,char*,int[],float[],float[]);    

}
void cr_arte_user(const int,const int,const char*,const int[],const int);


void cr_arte(int nr_sl) {
/***********************/
    zgvol ZGVOL;
    int ivos[15],in[15],nin[15],lcopy[15],ltype[15],nlevsl,iuser,indt,nr_vol;
    int nlev =0; int module =0;
    char  lvols[60], volname[4];

//  check whether the superlayer volume already exists
    strncpy(volname,ZGVOL.Get_volname(nr_sl),4);
    int Gvolnum = ZGVOL.Check_Gvol(volname);
    if (Gvolnum >0){
       int found =0; 
       for(unsigned int isl=1;isl<=ArteTable<NESL>::size();isl++) {
           ArtePointer<NESL> ge(isl);
           if(!strncmp(ge->name,volname,4)) { 
              found = isl; break;
           }
       }
       if(!found)   { 
           arteMsg << "cr_arte - nonvalid superlayer volume";
           arteMsg.error();
       } 
       crlvbk_(volname,&found,&indt,&nlevsl,lvols,lcopy);
       nlevsl = nlevsl -1;
       nlev = nlevsl;
       ivos[nlev] = Gvolnum;
       in[nlev] = 0; 
    }
    else {
       arteMsg << "cr_arte -nonvalid superlayer volume";   arteMsg.error();}

L10:       //modules and components
//  define the number of subcomponents nin(nlev) for the new level nlev 
    int jvos = *(gcbank_.lq + gclink_.jvolum -1 - ivos[nlev]);
    int *pji = gcbank_.iq + jvos + 2;
    float *pjx = (float*)pji; 
    nin[nlev] = int(*pjx);     

L15:
//  go to the next (first) component of level nlev          
    in[nlev]++;
//  if all components of this level are handled go to one level above or return 
    if(in[nlev] > nin[nlev]) { 
       nlev--;
       if (nlev >= nlevsl) goto L10;
       else {
         gcvolu_.nlevel = 0;
         return;           
       }
    }
//  get the gean banks of the component ivos(nlev) and the name of next subcomp
//  volname or as integer ii
    jvos = *(gcbank_.lq + gclink_.jvolum -1 - ivos[nlev]);
    int jin = *(gcbank_.lq + jvos -1 -in[nlev]);
    pji = gcbank_.iq + jin -1;
    pjx = (float*)pji;
    int ivot = int(*(pjx+2));
    memcpy(volname,gcbank_.iq+gclink_.jvolum -1 + ivot,4);      
    iuser = int(*(pjx+3));
    int type = iuser>>16;
    if((nlev-nlevsl) == 0) module = iuser&65535; 
    nr_vol = ZGVOL.Get_volnr(volname,type);
    if(nr_vol <= 0) goto L15;    

//  increase the level and put the new name and user number in lvols/lcopy
    nlev++;
    strncpy(&lvols[4*nlev],volname,4);
    ltype[nlev] = type;
    lcopy[nlev] = iuser;
//  define the number of subcomponents nin(nlev)
    int jvot = *(gcbank_.lq + gclink_.jvolum -1 -ivot);
    pji = gcbank_.iq + jvot +2;
    pjx = (float*)pji;
    nin[nlev] = int(*pjx);

    if(nin[nlev] != 0) {                //    modules only 
       in[nlev] = 0;
       ivos[nlev] = ivot;
       module = iuser&65535;
       goto L10;            
    }

//  components only
    Geant2Arte G2A;

//  prepare and transform positions and rotation matrces
    int nl = nlev+1;

    ArtePointer<NESL> cursl(zgmoth_.CURRSL);
//  fill NEDE table
    if(ZGVOL.Get_sens_flag(nr_vol) > 2) {
       if(cursl->cmp != LGESL_HOD3) {
          zgpos ZGPOS;
          zgdet ZGDET;
          int  nnede = G2A.CreateNede();
          ArtePointer<NEDE> nd(nnede);
	  G2A.SetGvolNumber(volname,ivot);
          G2A.FillTree(nl,lvols,lcopy);
          G2A.FillGvolParameter();
          G2A.FillNedeParameter(nnede);
          ZGDET.FillSensParameter(nr_vol,nd);
          nd->gcop = iuser&65535;
          int in = module<<16;
          nd->gcop = nd->gcop + in;
          nd->lay = ZGPOS.Get_layer(nr_sl,module);
          nd->sec = ZGPOS.Get_sector(nr_sl,module);

//        possibility to make special arrangements for a special detector
	  cr_arte_user(nnede,0,lvols,lcopy,nlev); 
       }
       else {
          zgpos ZGPOS;
          int  nncal = G2A.CreateNcal();
          ArtePointer<NCAL> nc(nncal);
	  G2A.SetGvolNumber(volname,ivot);
          G2A.FillTree(nl,lvols,lcopy);
          G2A.FillGvolParameter();
          G2A.FillNcalParameter(nncal);
          nc->sec = ZGPOS.Get_sector(nr_sl,module);          
//        possibility to make special arrangements for a special subdetector
          cr_arte_user(nncal,0,lvols,lcopy,nlev); 
       }
    }
      
//  fill wall table   
    else if(ZGVOL.Get_sens_flag(nr_vol) <=2) { 
        G2A.SetGvolNumber(volname,ivot);
        G2A.FillTree(nl,lvols,lcopy);
        float xms = G2A.FillGvolParameter();
        if(xms <= 1.e-10) {nlev = nlev-1;   goto L15;}          
        int  nnwal = G2A.CreateNwal();
        G2A.FillNwalParameter(nnwal);
//      possibility to change something in the automatically produced tables
        cr_arte_user(0,nnwal,lvols,lcopy,nlev); 
    }
    nlev = nlev-1;
    goto L15; 
}
      
/* cr_arte_ */
