
/***********************************************************************
* CRVTRM
* written by Siegmund Nowak
*
* This routine returns the geometric parameters POS and rotation
* matrix RMAT translated from the different GEANT daughter systems
* into the HERA-B detector system 
*
* Arguments  kind         description
*          (in,update,out)
*-----------------------------------------------------------------------
* var1        in           this variable will only be read
***********************************************************************/
#include <string.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "gean/gclink.h"
#include "gean/gcflag.h"
#include "gean/gcvolu.h"
#include "gean/zgeom.h"

extern "C" {
  void crvtrm_(int &, char[], int[], float[], float[]);
  void glvolu_(int &, char *, int[] ,int &, const int);
  void gmtod_(float [], float[], int &); 
}
  

void crvtrm_(int &nlev, char lvols[], int lcopy[], float pos[], float rmat[]) {

  zgvol ZGVOL;

  if(nlev <= 0) {
     arteMsg << "crvtrm: nlev < 0 "; arteMsg.error();
  }

//get detector parameters shape, ivol and relation gmat-nede
  int ivos = ZGVOL.Check_Gvol(&lvols[(nlev-1)*4]);
  if(ivos == 0) {
     arteMsg << "crvtrm >> nonvalid volume"; arteMsg.error();
  }
  int jvos = *(gcbank_.lq +gclink_.jvolum-1 -ivos);
  int *pji = gcbank_.iq + jvos + 1;
  float *pjx = (float *)pji;
  int ishap = int(*pjx);

//to get the parameters in master  one has to define the volume tree

  gcvolu_.nlevel = 0;
  int ier = 0;
  glvolu_(nlev,lvols,lcopy,ier,60);
  if(ier != 0) {
     arteMsg << "glvolu-ier = " << ier; arteMsg.error();
  }
  int jin = 0;
  if(nlev != 1) {
      int imoth = ZGVOL.Check_Gvol(&lvols[(nlev-2)*4]);
      int jmoth = *(gcbank_.lq +gclink_.jvolum-1 -imoth);
      pji =gcbank_.iq + jmoth + 2;
      pjx = (float*)pji;
      int nin = int(*pjx);
      if(nin <= 0) { 
	 char cname[5];
         strncpy(cname,&lvols[(nlev-2)*4],4);
         cname[4] ='\0';
         arteMsg << "crvtrm: mother has nin <= 0 " << cname;
         arteMsg.error();
      }
      int found = 0;

      for (int i= 0; i<nin; i++) {
         jin = *(gcbank_.lq + jmoth - 2 - i);     
         pji = gcbank_.iq + jin - 1;
         pjx = (float*)pji;
         if(ivos == int(*(pjx+2)) && lcopy[nlev-1] == int(*(pjx+3))) {
            found = 1;
            break;
         }  
      }
      if(found  == 0) {
         arteMsg << "crvtrm: volume not found in jin-bank ";
         arteMsg.error();
      } 
  } 

// get the parameters in master system
  pji = gcbank_.iq + jvos + 4;
  pjx = (float*)pji;
  int js = jvos+6;                   //  volume positioned by gspos
  if(int(*pjx) == 0) {
      pji = gcbank_.iq + jin + 8;
      pjx = (float*)pji;
      if(int(*pjx) == 0) {
         arteMsg << "crvtrm >>  no parameters found"; arteMsg.error();
      }
      js = jin+9;
  }
  int npar = int(*pjx);    
  if(ishap == 4) npar =11;

  float posm[3], qs[11]; 
  for(int i=0; i<11; i++ ) {
      pos[i] = 0.,        qs[i] = 0.;
  }
  pos[2] = zgmoth_.ZZERO;
  int i1=1; 
  gmtod_(pos,posm,i1);

  memcpy(qs,gcbank_.iq + js,4*npar);
  float sinang = 0.,     cosang = 1.,      tanang = 0.;



  /* for trapezoids a given side is always the bigger one of the two at the 
     endsides in z, the additional parameters contain the tangens of the 
     slope, its sign indicates where the bigger side/radius is */ 

  if(ishap == 1) {	                                // box 
     pos[0] = -posm[0] - qs[0];
     pos[1] = -posm[0] + qs[0]; 
     pos[2] = -posm[1] - qs[1];
     pos[3] = -posm[1] + qs[1];
     pos[4] = -posm[2] - qs[2]; 
     pos[5] = -posm[2] + qs[2];
  } 
  else if(ishap == 2) {                                 // trapezoid x varying
//   for trapezoids a given side is always the bigger one of the two at the 
//   endsides in z, the additional parameters contain the tangens of the 
//   slope, its sign indicates where the bigger side is.
     float qsm = qs[0]>qs[1] ? qs[0] : qs[1];
     pos[0] = -posm[0] - qsm;
     pos[1] = -posm[0] + qsm;
     pos[2] = -posm[1] - qs[2];
     pos[3] = -posm[1] + qs[2];
     pos[4] = -posm[2] - qs[3]; 
     pos[5] = -posm[2] + qs[3]; 
     pos[6] = 0.5 * (qs[1] - qs[0]) / qs[3];
  }
  else if(ishap == 3) {                                // trapezoid x/y varying
//   for trapezoids the given sides are always the bigger ones of the two at 
//   the endsides in z, the additional parameters contain the tangens of the 
//   slope, its sign indicates where the bigger side is
     float qsm = qs[0]>qs[1] ? qs[0] : qs[1];
     pos[0] = -posm[0] - qsm;
     pos[1] = -posm[0] + qsm;
     qsm = qs[2]>qs[3] ? qs[2] : qs[3];
     pos[2] = -posm[1] - qsm;
     pos[3] = -posm[1] + qsm;
     pos[4] = -posm[2] - qs[4]; 
     pos[5] = -posm[2] + qs[4]; 
     pos[6] = 0.5 * (qs[1] - qs[0]) / qs[4];
     pos[7] = 0.5 * (qs[3] - qs[2]) / qs[4];
  }
  else if(ishap == 4) {                                   // general trapezoid
     float qsm = (qs[4]+qs[5]) > (qs[8]+qs[9]) ? (qs[4]+qs[5]) : (qs[8]+qs[9]);
     pos[0] = -posm[0] - .5*qsm;
     pos[1] = -posm[0] + .5*qsm;
     qsm = qs[2]>qs[7] ? qs[2] : qs[7];
     pos[2] = -posm[1] - qsm;
     pos[3] = -posm[1] + qsm;
     pos[4] = -posm[2] - qs[0]; 
     pos[5] = -posm[2] + qs[0]; 
     pos[6] = 0.5 * (qs[8] - qs[4]) / qs[0];
     pos[7] = 0.5 * (qs[9] - qs[5]) / qs[0];
     pos[8] = 0.5 * (qs[7] - qs[2]) / qs[0];
  }
  else if(ishap == 5) {                                   // tube
//   inner radius, outer radius, dz
     pos[0] = qs[0];
     pos[1] = qs[1];
     pos[2] = -posm[0];
     pos[3] = -posm[1];
     pos[4] = -posm[2] - qs[2]; 
     pos[5] = -posm[2] + qs[2];
  }
  else if(ishap == 6) {                                  //  tubs  
//   inner radius, outer radius, x, y, zmin ,zmax, phi1, phi2
     pos[0] = qs[0];
     pos[1] = qs[1];
     pos[2] = -posm[0];
     pos[3] = -posm[1];
     pos[4] = -posm[2] - qs[2]; 
     pos[5] = -posm[2] + qs[2]; 
     pos[6] = qs[3];
     pos[7] = qs[4];
  }
  else if(ishap == 7) {                                  // cone 
//   inner radius,outer radius,xpos,ypos,zmin,zmax,tg(delta ri),tg(delta ro)
//   both radii are the bigger one of the two at the endsides in z, the 
//   additional parameters contain the tangens of the slope, its sign 
//   indicates where the bigger radius is.
     pos[0] = qs[1]>qs[3] ? qs[1] : qs[3];
     pos[1] = qs[2]>qs[4] ? qs[2] : qs[4];  
     pos[2] = -posm[0];
     pos[3] = -posm[1];
     pos[4] = -posm[2] - qs[0]; 
     pos[5] = -posm[2] + qs[0]; 
     pos[6] = 0.5 * (qs[3] - qs[1])/ qs[0];
     pos[7] = 0.5 * (qs[4] - qs[2])/ qs[0];

  }
  else if(ishap == 9) {                                 // sphere 
//   inner radius, outer radius, x, y, z,theta1, theta2, phi1, phi2
     pos[0] = qs[0];
     pos[1] = qs[1];
     pos[2] = -posm[0];
     pos[3] = -posm[1];
     pos[4] = -posm[2];
     pos[5] = qs[2];
     pos[6] = qs[3];
     pos[7] = qs[4];
     pos[8] = qs[5];
  }
  else if(ishap == 10) {         		  // parallelepiped
     pos[0] = -posm[0] - qs[0];
     pos[1] = -posm[0] + qs[0]; 
     pos[2] = -posm[1] - qs[1];
     pos[3] = -posm[1] + qs[1];
     pos[4] = -posm[2] - qs[2]; 
     pos[5] = -posm[2] + qs[2]; 
     if(fabs(qs[3]) > 1.e-6) {
        tanang = qs[3];       
        cosang = 1./sqrt(1.+tanang*tanang);
        sinang = tanang*cosang;
        float jx=0;
        memcpy(&jx,gcbank_.iq + jin +5,4);
        pos[0] = pos[0] - tanang * jx;
        pos[1] = pos[1] - tanang * jx;
     }
     else if(fabs(qs[4]) > 1.e-6 && fabs(qs[5]) < 1.e-6) {
        pos[0] = pos[0] - fabs(qs[4]) * qs[2];
        pos[1] = pos[1] + fabs(qs[4]) * qs[2];
        pos[6] = qs[4];
        pos[7] = 0.;
     }
     else if(fabs(qs[3]) < 1.e-6 && fabs(qs[4]) < 1.e-5  
                                      && fabs(qs[5]) < 1.e-6) {
//          take it as box - do nothing
     }
     else {
        arteMsg << "crvtrm: check parameters for shape 10"; 
        arteMsg.error();
     }
  }
  else {
     arteMsg << "crvtrm: ishap = " << ishap; arteMsg.error();
  }

  pos[9]  = cosang;
  pos[10] = sinang;

//     get the rotation matrix
  if(gcvolu_.grmat[9 + (nlev-1)*10] == 0.) {
     rmat[0]  = 1.;
     rmat[1]  = 0.;
     rmat[2]  = 0.;
     rmat[3]  = 0.;
     rmat[4]  = 1.;
     rmat[5]  = 0.;
     rmat[6]  = 0.;
     rmat[7]  = 0.;
     rmat[8]  = 1.;
  }
  else {
     rmat[0] = gcvolu_.grmat[0+(nlev-1)*10];
     rmat[1] = gcvolu_.grmat[3+(nlev-1)*10];
     rmat[2] = gcvolu_.grmat[6+(nlev-1)*10];
     rmat[3] = gcvolu_.grmat[1+(nlev-1)*10];
     rmat[4] = gcvolu_.grmat[4+(nlev-1)*10];
     rmat[5] = gcvolu_.grmat[7+(nlev-1)*10];
     rmat[6] = gcvolu_.grmat[2+(nlev-1)*10];
     rmat[7] = gcvolu_.grmat[5+(nlev-1)*10];
     rmat[8] = gcvolu_.grmat[8+(nlev-1)*10];
  }

  gcvolu_.nlevel = 0;
  }




