*CMZ :  1.00/00 26/11/94  15.09.41  by  Dominik Ressing
*CMZ :  0.01/02 22/08/94  14.17.13  by  Dominik Ressing
*-- Author :    Dominik Ressing   27/07/94
cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
c
c     calculate momentum for track, which is given by two points x1,x2
c     behind the magnet. The magnetig field is approximated to be a circle.
c
c     INPUTS:
c
c     x1,y1,z1   first point
c     x2,y2,z2   second point
c
c     iV defines the mode of calculation: ( parameters from kinema.inc.f )
c           L1APPROXfull  : perform correct calculation
c           L1APPROXfirst : use only first approximation ( as in hardware )
c           L1APPROXseco  : use one more aproximation
c
c     OUTPUT:
c
c     E,Px,Py,Pz   4-momentum of the track
c     q            charge of the track
c
cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
c
      SUBROUTINE l1kin4m(x1,y1,z1,x2,y2,z2,iV,E,Px,Py,Pz,q)
c
      IMPLICIT NONE
c
#include "l1si/l1param.fi"
#include "l1si/l1geodf.fi"
#include "l1si/l1detdf.fi"
#include "l1si/l1kindf.fi"
c
      REAL    x1,y1,z1,x2,y2,z2,E,Px,Py,Pz,q,vM,Pzz,Pxx,
     &        dx,dy,dz,z_ext,zM,xM,yM,tanAl,Pxz,vvM,PPxz,
     &        PxPz,PyPz,rM,xxM,zzM,rrM,rLeMag,kkM,aaM
      INTEGER iV
c
c
c     --- check input:
      IF ( z1 .eq. z2 ) THEN
         E = -1.
         PZ = -1.
         Px = 0.
         Py = 0.
         q  = 0.
         RETURN
      END IF
c
c     --- distance of points in 3 dimensions
      dx = (x2-x1)
      dy = (y2-y1)
      dz = (z2-z1)
c     --- calculate distance to center of Magnet
      z_ext = T1DETmagz - z1
c     --- extrapolation to center of Magnet
      zM = T1DETmagz
      xM = z_ext/dz*dx + x1
      yM = z_ext/dz*dy + y1
c     --- Momentum ratios are like the track slopes
      PxPz = dx/dz
      PyPz = dy/dz
c
c     --- now get the length of the magnetic field
      rM = T1DETmagd/2.
      xxM = xM**2.
      zzM = zM**2.
      rrM = rM**2.
      kkM = rrM/zzM
      aaM = xxM/rrM
      vM  = xM/zM
      vvM = aaM*kkM
      IF ( xM .gt. rM ) THEN
         rLeMag = 0.
      ELSE
c        --- get rLeMag in meter !!!
         IF ( iV.eq.L1APPROXfirst ) THEN
            rLeMag = rM*(2.- aaM)/100.
         ELSE IF ( iV.eq.L1APPROXseco ) THEN
            rLeMag = rM*(2.- aaM*(1. - aaM*(kkM - 0.25)))/100.
         ELSE
            rLeMag = rM*sqrt(1.-aaM/(1.+vvM))/50.
         END IF
      END IF
c
c     --- alpha is the bending angle in the Magnet ( vertex constrained to (0,0,0))
c         tanAl = tan(alpha)
      IF ( iV.eq.L1APPROXfirst ) THEN
         tanAl = PxPz - vM
      ELSE IF ( iV.eq.L1APPROXseco ) THEN
         tanAl = PxPz*( 1. + vvM - vM*PxPz ) - vM
      ELSE
         tanAl = tan(atan(PxPz) - atan(vM))
      END IF
c
      IF ( tanAl.gt.0 ) THEN
         q = +1.
      ELSE
         q = -1.
      END IF
      tanAl = abs(tanAl)
c
c     --- finally one can calculate the Momentum in the x-z plane
      IF ( abs(tanAl).eq.0. ) THEN
         IF ( rLeMag.eq.0. ) THEN
            Pxz = 0.
         ELSE
            Pxz  = 9999999.
         END IF
      ELSE
         IF ( iV.eq.L1APPROXfirst ) THEN
            Pxz = 0.3*T1DETmagB*rLeMag/tanAl
         ELSE
            Pxz = 0.3*T1DETmagB*rLeMag*(1./tanAl - tanAl/4.)
         END IF
      END IF
c
c     --- now its easy to get the 4-momentum
      PPxz = Pxz*Pxz
      Pzz  = PPxz/(1+vvM)
      Pxx  = Pzz*vvM
      Pz   = sqrt(Pzz)
      Px   =  Pz*vM
      Py   = Pz*yM/zM
      E    = sqrt(PPxz+Py**2)
c
      RETURN
      END
