
// Boris Fominykh 25.01.2000
// fill NESL, NEDE from muon DB

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <iostream.h>
#include <iomanip.h>

#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

#include "murec/MuDbNede.h"
#include "murec/MuDbNesl.h"

extern "C" {
  void MuCheckNesl(MuNesl*);
  void MuFillNesl(ArteTableIterator<NESL>,MuNesl*);
  void MuAddNesl(MuNesl*);
  void MuCheckNede(int,MuNesl*,MuNede*);
  void MuFillNede(ArteTableIterator<NEDE>,MuNede*);
  void MuAddNede(MuNede*);
  void MuGeoPoint();
}

extern "C" {
int MuGeometry(int major,char* DBname)
{
  int ret;
  MuNesl   lnesl;  
  MuDbNesl Nesl;  
  Nesl.setDBname(DBname);
  Nesl.setKTstatus(-2);
  Nesl.setRun(major);
  Nesl.setMinor(0);
  ret=Nesl.Read();
  if(ret != 0) {
    arteMsg << "MuGeometry: NESL table is NOT read from DB, ret= " << ret;
    arteMsg << " DBname= " << DBname << " major= " << major << endl;
    arteMsg.warn();
    return -1;
  }

  MuNede   lnede;  
  MuDbNede Nede;  
  Nede.setDBname(DBname);
  Nede.setKTstatus(-2);
  Nede.setRun(major);
  Nede.setMinor(0);
  ret=Nede.Read();
  if(ret != 0) {
    arteMsg << "MuGeometry: NEDE table is NOT read from DB, ret= " << ret;
    arteMsg << " DBname= " << DBname << " major= " << major << endl;
    arteMsg.warn();
    return -2;
  }

// loops over muon superlayers
  for(int i=0; i<8; i++) {
    int j=i/2;
// superlayer key
    int neslkey = 18+10*j+i-2*j;
    ret=Nesl.Get(&neslkey,&lnesl);
    if(ret != 0) {
      arteMsg << "MuGeometry: no NESL for key " << neslkey << endl;
      arteMsg.warn();
      return -3;
    }
    else {
// parameters of chamber structure
      int nxmem[2][3];
      int nx99m[2][3];
      for(int i=0; i<2; i++) {
        for(int j=0; j<3; j++) {
          nxmem[i][j]=0;
          nx99m[i][j]=0;
        }
      }
      int msl = lnesl.nsl;
      int cmp = lnesl.cmp;
      int laym = lnesl.lay;
      if(cmp == 8 && laym > 0) {
        for(int l=0; l<laym; l++) {
          for(int i=0; i<2; i++) {
            nxmem[i][l]=lnesl.chamb[2*l+i];
            nx99m[i][l]=lnesl.nx99[2*l+i];
          }
        }
      }

// cross-check
      int nvol=lnesl.npl;
      int nvl=0;
      if(cmp == 8 && laym > 0) {
        for(int l=0; l<laym; l++) {
          for(int i=0; i<2; i++) {
            nvl+=2*nxmem[i][l];
            nvl+=2*nx99m[i][l];
          }
        }
      }
      if(cmp == 9) nvl=laym;
      if(nvl != nvol) {
        arteMsg << "MuGeometry: cross-check: nvol= " << nvol;
        arteMsg << " is not equal nvl= " << nvl << endl;
        arteMsg.warn();
        return -4;
      }
// check NESL in ARTE
      MuCheckNesl(&lnesl);

// loops over cmp=8 nede elements
      if(cmp == 8 && laym > 0) {
        for(int lay=0; lay<laym; lay++) {
          for(int ud=0; ud<2; ud++) {
            if(nxmem[ud][lay] > 0) {
              for(int nx=0; nx<nxmem[ud][lay]+1; nx++) {
                int nxx=0;
                if(nx < nxmem[ud][lay]) nxx=nx+1;
                if(nx == nxmem[ud][lay] && nx99m[ud][lay] == 1) nxx=99;
                if(nxx > 0) {
                  for(int sd=0; sd<2; sd++) {
// key word
  int nedekey = 100000*msl+10000*(lay+1)+1000*ud+10*nxx+sd;
                    ret=Nede.Get(&nedekey,&lnede);
                    if(ret != 0) {
		      arteMsg << "MuGeometry: no NEDE for key " << nedekey << endl;
                      arteMsg.warn();
                      return -5;
                    }
                    else {
// check NEDE in ARTE
                      MuCheckNede(sd,&lnesl,&lnede);
                    }
                  }
                }
              }
            }
          }
        }
      }

// loops over cmp=9 nede elements
      if(cmp == 9 && laym > 0) {
        for(int qu=0; qu<laym; qu++) {
// key word
          int nedekey = 1000*msl+100*(qu+1)+cmp;
          ret=Nede.Get(&nedekey,&lnede);
          if(ret != 0) {
	    arteMsg << "MuGeometry: no NEDE for key " << nedekey << endl;
            arteMsg.warn();
            return -5;
          }
          else {
// check NEDE in ARTE
            MuCheckNede(0,&lnesl,&lnede);
          }
        }
      }
    }
  }
  MuGeoPoint();
  return 2;
}

void MuCheckNesl(MuNesl* pnesl)
{
// superlayer name
  char namsl[5];
  strncpy(namsl,&(pnesl->name[0]),5);
  namsl[4]='\0';
  int check=0;
// loops over ARTE superlayers
  for ( ArteTableIterator<NESL> isl = ArteTable<NESL>::begin(); 
	isl != ArteTable<NESL>::end();
	++isl ) {
    int cmp  = (*isl)->cmp;
    if(cmp == 8 || cmp == 9) {
// ARTE superlayer name
      char name[5];
      strncpy(name,(*isl)->name,5);
      name[4]='\0';
      if(!strcmp(namsl,name)) {
        check=1;
        MuFillNesl(isl,pnesl);
      }
    }
  }
  if(check == 0) MuAddNesl(pnesl);
  return;
}

void MuFillNesl(ArteTableIterator<NESL> isl,MuNesl* pnesl)
{
  (*isl)->cmp = pnesl->cmp;
  (*isl)->shp = pnesl->shp;
  (*isl)->nsl = pnesl->nsl;
  (*isl)->mfl = pnesl->mfl;
  (*isl)->lay = pnesl->lay;
  (*isl)->npl = pnesl->npl;
  (*isl)->fpl = pnesl->fpl;
  (*isl)->nwl = pnesl->nwl;
  (*isl)->fwl = pnesl->fwl;
  (*isl)->ltyp= pnesl->ltyp;
  (*isl)->x1  = pnesl->x1;
  (*isl)->x2  = pnesl->x2;
  (*isl)->y1  = pnesl->y1;
  (*isl)->y2  = pnesl->y2;
  (*isl)->z1  = pnesl->z1;
  (*isl)->z2  = pnesl->z2;
  (*isl)->dx  = pnesl->dx;
  (*isl)->dy  = pnesl->dy;
  return;
}

void MuAddNesl(MuNesl* pnesl)
{
  struct NESL nesl;
  char nam[4];
  strncpy(&nam[0],pnesl->name,4);
  strncpy(nesl.name,nam,4);

  nesl.cmp = pnesl->cmp;
  nesl.shp = pnesl->shp;
  nesl.nsl = pnesl->nsl;
  nesl.mfl = pnesl->mfl;
  nesl.lay = pnesl->lay;
  nesl.npl = pnesl->npl;
  nesl.fpl = pnesl->fpl;
  nesl.nwl = pnesl->nwl;
  nesl.fwl = pnesl->fwl;
  nesl.ltyp= pnesl->ltyp;
  nesl.x1  = pnesl->x1;
  nesl.x2  = pnesl->x2;
  nesl.y1  = pnesl->y1;
  nesl.y2  = pnesl->y2;
  nesl.z1  = pnesl->z1;
  nesl.z2  = pnesl->z2;
  nesl.dx  = pnesl->dx;
  nesl.dy  = pnesl->dy;
  ArtePointer<NESL>::append(nesl);
  return;
}

void MuCheckNede(int sd,MuNesl* pnesl,MuNede* pnede)
{
// chamber name
  char name[9];
  char namde[9];
  strncpy(namde,&(pnede->hwc[0]),9);
  namde[8]='\0';
  int sdch = 0;
  for ( ArteTableIterator<NEDE> iv = ArteTable<NEDE>::begin(); 
	iv != ArteTable<NEDE>::end();
	++iv) 
  {
    int cmp  = (*iv)->cmp;
    if(cmp == 8 || cmp == 9) {
// ARTE chamber name
      strncpy(name,(*iv)->hwc,9);
      if(cmp == 9) {
        for(int i=5; i< 8; i++) name[i]=' ';
      }
      name[8]='\0';

      if(!strcmp(namde,name)) {
        if(sd == sdch) {
          MuFillNede(iv,pnede);
          return;
        }
        if(sd == 1 && sdch == 0) sdch=1;
      }
    }
  }
  MuAddNede(pnede);
  return;
}

void MuFillNede(ArteTableIterator<NEDE> iv,MuNede* pnede)
{
  (*iv)->cmp = pnede->cmp;
  (*iv)->lay = pnede->lay;
  (*iv)->sec = pnede->sec;
  (*iv)->shp = pnede->shp;
  (*iv)->xmin = pnede->xmin;
  (*iv)->xmax = pnede->xmax;
  (*iv)->ymin = pnede->ymin;
  (*iv)->ymax = pnede->ymax;
  (*iv)->zmin = pnede->zmin;
  (*iv)->zmax = pnede->zmax;
  (*iv)->sxmn = pnede->sxmn;
  (*iv)->sxmx = pnede->sxmx;
  (*iv)->symn = pnede->symn;
  (*iv)->symx = pnede->symx;
  (*iv)->cos  = pnede->cos;
  (*iv)->sin  = pnede->sin;
  for(int j=0; j<9; j++) (*iv)->rot[j]=pnede->rot[j];
  (*iv)->acc  = pnede->acc;
  (*iv)->eff  = pnede->eff;
  (*iv)->dod  = pnede->dod;
  (*iv)->pit  = pnede->pit;
  (*iv)->ms   = pnede->ms;
  for(int j=0; j<4; j++) (*iv)->xfw[j]=pnede->xfw[j];
  for(int j=0; j<4; j++) (*iv)->zfw[j]=pnede->zfw[j];
  for(int j=0; j<4; j++) (*iv)->nwi[j]=pnede->nwi[j];
  (*iv)->nsl = pnede->nsl;
  (*iv)->nwl = pnede->nwl;
  (*iv)->fwl = pnede->fwl;
  (*iv)->xmad= pnede->xmad;
  (*iv)->ymad= pnede->ymad;
  (*iv)->zmad= pnede->zmad;
  for(int j=0; j<4; j++) (*iv)->y1w[j]=pnede->y1w[j];
  for(int j=0; j<4; j++) (*iv)->y2w[j]=pnede->y2w[j];
  (*iv)->dedx = pnede->dedx;
  (*iv)->gcop = pnede->gcop;
  (*iv)->noise= pnede->noise;
  (*iv)->ares = pnede->ares;

  char buf[4];
  strncpy(buf,&(pnede->gvol[0]),4);
  strncpy((*iv)->gvol,buf,4);

  return;
}

void MuAddNede(MuNede* pnede)
{
  struct NEDE nede;

  nede.cmp = pnede->cmp;
  nede.lay = pnede->lay;
  nede.sec = pnede->sec;
  nede.shp = pnede->shp;
  nede.xmin = pnede->xmin;
  nede.xmax = pnede->xmax;
  nede.ymin = pnede->ymin;
  nede.ymax = pnede->ymax;
  nede.zmin = pnede->zmin;
  nede.zmax = pnede->zmax;
  nede.sxmn = pnede->sxmn;
  nede.sxmx = pnede->sxmx;
  nede.symn = pnede->symn;
  nede.symx = pnede->symx;
  nede.cos  = pnede->cos;
  nede.sin  = pnede->sin;
  for(int j=0; j<9; j++) nede.rot[j]=pnede->rot[j];
  nede.acc  = pnede->acc;
  nede.eff  = pnede->eff;
  nede.dod  = pnede->dod;
  nede.pit  = pnede->pit;
  nede.ms   = pnede->ms;
  for(int j=0; j<4; j++) nede.xfw[j]=pnede->xfw[j];
  for(int j=0; j<4; j++) nede.zfw[j]=pnede->zfw[j];
  for(int j=0; j<4; j++) nede.nwi[j]=pnede->nwi[j];
  nede.nsl = pnede->nsl;
  nede.nwl = pnede->nwl;
  nede.fwl = pnede->fwl;
  nede.xmad= pnede->xmad;
  nede.ymad= pnede->ymad;
  nede.zmad= pnede->zmad;
  for(int j=0; j<4; j++) nede.y1w[j]=pnede->y1w[j];
  for(int j=0; j<4; j++) nede.y2w[j]=pnede->y2w[j];
  nede.dedx = pnede->dedx;
  nede.gcop = pnede->gcop;
  nede.noise= pnede->noise;
  nede.ares = pnede->ares;

  char name[9];
  strncpy(name,pnede->hwc,9);
  if(pnede->cmp == 8) {
    name[8]='\0';
    strncpy(nede.hwc,name,9);
  }
  if(pnede->cmp == 9) {
    name[5]='\0';
    strncpy(nede.hwc,name,6);
  }

  char buf[4];
  strncpy(buf,pnede->gvol,4);
  strncpy(nede.gvol,buf,4);

  ArtePointer<NEDE>::append(nede);

  return;
}

// update NESL -> NEDE pointers
void MuGeoPoint()
{
  for ( ArteTableIterator<NESL> isl = ArteTable<NESL>::begin(); 
        isl != ArteTable<NESL>::end();
        ++isl ) {
    int   cmp  = (*isl)->cmp;
    if(cmp == 8 || cmp == 9) {
      (*isl)->fpl=0;
      (*isl)->npl=0;
    }
  }
// loops over NEDE
  int nede=0;
  for ( ArteTableIterator<NEDE> iv = ArteTable<NEDE>::begin(); 
        iv != ArteTable<NEDE>::end();
        ++iv ) {
    nede++;
    int cmp = (*iv)->cmp;
    if(cmp == 8 || cmp == 9) {
      ArtePointer<NESL> nesl = (*iv)->nesl;
      if(nesl->fpl == 0) nesl->fpl=nede;
      nesl->npl += 1;
    }
  }
  return;
}

}


