/************************************************************************
*
* jsMuPixTool.C
* 
* first author : Jing-Ge Shiu 
* purpose      : C++/SLT code for checking muon pixel digitization
* called from  : 
* includ file  : MuHitInfo.h, jsMuPixTool.h
* start date   : May. 09, 2000
*
*
*-----------------------------------------------------------------------
*   some routines here should also work for pixel occ. of real data
*
*   NOTE : pixel and virtual pad(group of pixel) count from 0
*          readout connector and channel count from 1
*
*   2000, 1017  physjg  
*               first release 
*           
******************************************************************************/
#include "murec/jsMuPixTool.h"

MXPIX::MXPIX() {

  for (int sl=0;sl<Sl;sl++) 
    for(int xi=0;xi<MaxVpCol;xi++)
      for (int yi=0;yi<MaxVpRow;yi++) {
        vp_pix[sl][xi][yi].npix = 0;
        vp_readout[sl][xi][yi].i1 = 0;
        vp_readout[sl][xi][yi].i2 = 0;
      } 
  nonereadpix();
  buildpix2vp();
  buildvp2rd();

  return;
};


MXPIX::~MXPIX() {
};


MXPIX::MXPIX(const MXPIX& dodo) {
  cout << "Sorry, actually I don't like you to copy this." << endl;
};

/****************************************************
  build the array information for pixel number without readout 
*/
void MXPIX::nonereadpix() {
  
  //Mx1 and Mx2, Mx3
  for (int sl=0;sl<3;sl++) {
    for (int yi= 0;yi< 6;yi++) nordpix[sl][yi] = 18;
    for (int yi= 6;yi< 9;yi++) nordpix[sl][yi] = 17;
    for (int yi= 9;yi<11;yi++) nordpix[sl][yi] = 16;
    nordpix[sl][11] = 15;
    nordpix[sl][12] = 14;
    nordpix[sl][13] = 13;
    nordpix[sl][14] = 12;
    nordpix[sl][15] = 10;
    nordpix[sl][16] =  8;
    nordpix[sl][17] =  5;
    for (int yi=18;yi<=NyM[sl];yi++) nordpix[sl][yi] = 0;
  }
  //Mx3  ---> only one difference from Mx1/2
    nordpix[2][9] = 17;
  //Mx4
    for (int yi= 0;yi< 5;yi++) nordpix[3][yi] = 18;
    for (int yi= 5;yi< 8;yi++) nordpix[3][yi] = 17;
    for (int yi= 8;yi<10;yi++) nordpix[3][yi] = 16;
    for (int yi=10;yi<12;yi++) nordpix[3][yi] = 15;
    nordpix[3][12] = 14;
    nordpix[3][13] = 13;
    nordpix[3][14] = 12;
    nordpix[3][15] = 10;
    nordpix[3][16] =  8;
    nordpix[3][17] =  5;
    for (int yi=18;yi<=NyM[3];yi++) nordpix[3][yi] = 0;

  return;
};



/****************************************************
  build the array for the sense pixel to 
  readout virtual pad in 1st-qudrant-like system

  count from 0

*/
void MXPIX::buildpix2vp() {

  for (int sl=0;sl<Sl;sl++) {
     int aux_x = (sl<2?1:0), aux_y = (sl==2?2:0);
     for (int ix=0;ix<NxM[sl];ix++) {
        int vpx = (ix+aux_x)/NxC[sl];     
        for (int iy=0;iy<NyM[sl];iy++) {
           int vpy = (pix_vp[sl][ix][iy].i2 = (iy - aux_y)/NyC[sl]);
           pix_vp[sl][ix][iy].i1 = vpx;
           if (ix>=nordpix[sl][iy]) {
              pix_rdvp[sl][ix][iy].i1 = vpx;
              pix_rdvp[sl][ix][iy].i2 = vpy;
              vp_pix[sl][vpx][vpy].pixid[vp_pix[sl][vpx][vpy].npix].i1=ix;
              vp_pix[sl][vpx][vpy].pixid[vp_pix[sl][vpx][vpy].npix].i2=iy;
              if ((vp_pix[sl][vpx][vpy].npix++)>6) 
		cout <<"[Warning!!!] Oh, my god! " << sl << "," 
                     << ix<<","<<iy<<","<<vp_pix[sl][vpx][vpy].npix<< endl;
	   } else {
              pix_rdvp[sl][ix][iy].i1 = -20;
              pix_rdvp[sl][ix][iy].i2 = -20;
           }
        }
     }
  }

  return ;
};


/****************************************************
  build the array for the mappig from 
  virtual pad to readout channels in 1st-qudrant-like system
*/
void MXPIX::buildvp2rd() {

  Nxrd[0]=Nxrd[1]=23; Nyrd[0]=Nyrd[1]=17;        // low density Mx1, Mx2
  Nxrd[2]=45; Nyrd[2]=8;                         // Mx3
  Nxrd[3]=48; Nyrd[3]=9;                         // Mx4


  for (Word sl=0;sl<Sl;sl++) 
    for (Word nc=0;nc<Ncon;nc++ )       // there should be all have Ncon(24) connector so far 
       for (Word bit=0;bit<Nbit;bit++)
	  if ( ( Pnx1[sl][nc][bit]>0 )&&(Pny1[sl][nc][bit]>0) ) 
	    if ( (Pnx1[sl][nc][bit]<=Nxrd[sl])&&(Pny1[sl][nc][bit]<=Nyrd[sl]) ) {
              vp_readout[sl][ Pnx1[sl][nc][bit]-1 ][ Pny1[sl][nc][bit]-1 ].i1 = nc+1;
              vp_readout[sl][ Pnx1[sl][nc][bit]-1 ][ Pny1[sl][nc][bit]-1 ].i2 = bit+1;
            } else {
              cout << "Data error sl/nc/bit (" << sl << "/" << nc << "/" << bit 
                   << ") at vp position " << Pnx1[sl][nc][bit] << ", " << Pny1[sl][nc][bit] << endl; 
          
            }
  return ;
};


/****************************************************
  get the pixel coordinates from x,y position
  
  cout from 0
  I probably will have problem from round out.

*/
TWOINT MXPIX::getxy2pix(int slid,int subid,  float x, float y) {

  TWOINT pixid={-1,-1};
  float xmin = (slid<3)?-.90:-.94, ymin = xmin; 
  float pitch = (slid<3)?.90:.94;           // could I get this from other place? 

  if ((slid>-1)&&(slid<4)) {  
     if ( (subid==1)||(subid==2) ) x = -x;
     if ( (subid==1)||(subid==3) ) y = -y;
     pixid.i1 = int ((x-xmin)/pitch);
     pixid.i2 = int ((y-ymin)/pitch);
     if ( (pixid.i2>=NxM[slid])||(pixid.i2>=NyM[slid]) )
        pixid.i1 = pixid.i2 = -1; 
  }

  return pixid;
   
};



/****************************************************
  print out the sense pixels to readout virtual pads
*/
void MXPIX::printpix2vp(int sl) {

  for (int iy=NyM[sl]-1;iy>=0;iy--) {
     for (int ix=0; ix<NxM[sl]; ix++) {
       cout << "(" << setw(2) << pix_rdvp[sl][ix][iy].i1 
            << "," << setw(2) << pix_rdvp[sl][ix][iy].i2 <<")" ;
     }
     cout << endl;
  }

  return;
   
};


/* *************************************************
   print out the readout pixel pattern in text format 
*/
void MXPIX::printropix(int sl) {
    
  int x1 = 0, x2=NxM[sl],
      y1 = 0, y2=NyM[sl];

  for (int yi=y2-1;yi>=y1;yi--) {
    cout << setw(2) << nordpix[sl][yi]<< ":"<<"|";
    for (int xi=x1;xi<x2;xi++) {
      if (pix_rdvp[sl][xi][yi].i1<0) cout << ".|";
      else cout << "*|" ;
    }
    cout << endl;
  }

  return;

}


/* *****************************************************
   print out the virtual pad to readout channel mapping 
*/
void MXPIX::printvp2rd(int sl) {
    
  cout << "Superlayer " << sl << " with " << Nxrd[sl] << " columns and "
       << Nyrd[sl] << " rows." << endl;
       
  for (Word ix=0; ix<Nxrd[sl] ;ix++) {    
  Word prindex = 0;
    cout << "column " << ix+1 << endl;
    for (Word iy=0; iy<Nyrd[sl]; iy++) {
      cout << "row " << iy+1 << " c-bit " << vp_readout[sl][ix][iy].i1 
           <<  " - " << vp_readout[sl][ix][iy].i2 << ";   ";       
      if (++prindex>3) { prindex=0; cout << endl;}
    }
    cout << endl;
  }
  cout << endl;

  return;

}



