      integer function otr2hit(x0,y0,tx,ty,z1,z2,usemask,countcell)
************************************************************************
* This routine returns number of passes of OTR single layers
* for a straight track.
* Track is specified by its starting point (x0,y0) at Z=0
* and tangents (tx,ty) in laboratory frame.
* Counting is done only for GEDEs inside Z interval [Z1,Z2].
*
* Usemask = if non-zero, channel mask is used
*            (=2 means count bad cells, not good ones)
* Countcell = if non-zero, we count cells (not single layers)
*
* Author: Alexander Lanyov      April 2001
*
************************************************************************
      implicit none

#include "arte/Arte.fi"
#include "arte/ArteFlag.fi"
#include "auto/GEO.fi"
#include "gean/ZGESL.fi"

      integer n,i,l,usemask,countcell,status,nw,slay
      real x0,y0,tx,ty,z1,z2,x,y,z,xrot,yrot,zrot,pit,w

      integer otr_gede_status

      integer ci,otr_hitprep_type
      integer otrhitpreparation,otriisok,otrifromnedelayerwire

#include "arte/Arte.ff"
#include "auto/GEO.ff"

      otr_hitprep_type = otrhitpreparation()
      if ( otr_hitprep_type.ne.0.and.otr_hitprep_type.ne.1 ) then
         write (*,*) "otr2hit: unknown OtrHitPreparation = ",
     @        otr_hitprep_type, ", using default value == 0 (OtrI)"
         otr_hitprep_type = 0
      end if
      
      n = 0
      do i=1,ngede()
         if (igede_cmp(i) .ne. lgesl_wch) goto 30
         if (igede_sec(i) .eq. 0) goto 30

         if (fgede_zmin(i) .gt. z2) goto 30
         if (fgede_zmax(i) .lt. z1) goto 30

c Coordinates in Lab frame
         z = 0.5 * (fgede_zmin(i)+fgede_zmax(i))
         x = x0 + tx*z
         y = y0 + ty*z

c Coordinates in GEDE frame
         xrot = fgede_rot(1,i)*x + fgede_rot(4,i)*y + fgede_rot(7,i)*z
         yrot = fgede_rot(2,i)*x + fgede_rot(5,i)*y + fgede_rot(8,i)*z
         zrot = fgede_rot(3,i)*x + fgede_rot(6,i)*y + fgede_rot(9,i)*z

         if (xrot .le. fgede_sxmn(i)) goto 30
         if (xrot .ge. fgede_sxmx(i)) goto 30

         if (yrot .le. fgede_symn(i)) goto 30
         if (yrot .ge. fgede_symx(i)) goto 30

         pit = fgede_pit(i)

         do slay=1,(igede_nsl(i)+1)/2
            do l=2*slay-1,min(2*slay,igede_nsl(i))

c Coordinates in Lab frame
               z = fgede_zfw(l,i)
               x = x0 + tx*z
               y = y0 + ty*z
               
c Coordinates in GEDE frame
               xrot = fgede_rot(1,i)*x+fgede_rot(4,i)*y+fgede_rot(7,i)*z
               yrot = fgede_rot(2,i)*x+fgede_rot(5,i)*y+fgede_rot(8,i)*z
               zrot = fgede_rot(3,i)*x+fgede_rot(6,i)*y+fgede_rot(9,i)*z

               if (yrot .le. fgede_y1w(l,i)+0.5*2.5) goto 10
               if (yrot .ge. fgede_y2w(l,i)-0.5*2.5) goto 10

               w = (xrot-fgede_xfw(l,i))/pit + 1./3.
               if (w.le.0.) goto 10
               nw = w+1
               if (nw.gt.igede_nwi(l,i)) goto 10
               if (w-nw+1 .gt. 2./3.) goto 10

c Take e-beam into account: use approximate values (only for PC,TC)
               if (z.gt.680.) then
                  if ((x.gt.40. .and. x.lt.55.)
     @                 .and. (y.gt. -95. .and. y.lt. -67.)) goto 10
                  
                  if ((x.gt.40. .and. x.lt.70.) .and. igede_nsl(i).eq.2
     @                 .and. (y.gt. -95. .and. y.lt. -67.)) goto 10
               end if

               if (usemask.ne.0) then

                  if ( otr_hitprep_type.eq.0 ) then    ! OtrI
                     ci = otrifromnedelayerwire(i, l-1, nw-1)
                     if (otriisok(ci).eq.1) then    ! good
                        status = 0
                     else
                        status = 1
                     end if

                  else
                     status = otr_gede_status(i,l,nw)  ! Wax
                  end if

                  if (usemask.eq.1 .and. status.ne.0) goto 10
                  if (usemask.eq.2 .and. status.eq.0) goto 10
               end if

c This cell is hit by track
               n = n + 1
               if (countcell.eq.0) goto 20
 10            continue ! end of monolayer l
            end do
 20         continue ! end of single layer slay
         end do
 30      continue ! end of GEDE
      end do

      otr2hit = n
      return
      end

c=======================================================================
      integer function otr2hit_pc(x0,y0,tx,ty)
************************************************************************
* This routine returns number of passes of OTR single layers
* for a straight track.
* Track is specified by its starting point (x0,y0) at Z=0
* and tangents (tx,ty) in laboratory frame.
* Counting is done only for PC superlayers.
*
* Author: Alexander Lanyov      April 2001
*
************************************************************************
      implicit none
      real x0,y0,tx,ty,z1,z2
      integer otr2hit

      z1 = 680.
      z2 = 900.
      otr2hit_pc = otr2hit(x0,y0,tx,ty,z1,z2,0,0)
      return
      end
