// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
//
//  Modified:  
//  new flag HITWIRE_FUDGE introduced                28/07/00  A.Spiridonov
//  - inverse filter before updinternal              17/07/01  Wouter Hulsbergen

#include <iostream>
#include <math.h>
#include "arte/arte.hh"
#include "rang/Node.hh"
#include "rang/Hit.hh"
#include "rang/HitWire.hh"
#include "rang/Geom.hh"
#include "rang/StateP.hh"
#include "rang/StatePI.hh"
#include "rang/StateL.hh"
#include "rang/StateLI.hh"

#include "wax/HITB.hh"
#include "wax/RTRelation.hh"
#include <TH1.h>

//#include "rang/zfconst.hh"

//-- Constructor
rangerHitWire::rangerHitWire() {
  LRSign = 1;
}

rangerHitWire::rangerHitWire(ArtePointer<RHIT> rhit,rangerFitMode fm) 
: rangerHit(rhit,fm) {
  LRSign = (rhit->q)>0 ? 1 : -1;
  m = new HepVector(1);
  (*m)(1) = LRSign * fabs(rhit->hitb->q);
  //(*m)(1) = LRSign * ((rhit->hitb->q)>0 ? (rhit->hitb->q) : 0) ;
  //(*m)(1) = LRSign * (fabs(rhit->hitb->q) - zfconst_.dOff);
  V = new HepSymMatrix(1);
  (*V)(1,1) = (rhit->hitb->r) * (rhit->hitb->r);
//  cout << "This is the rangerHitWire constructor" << endl;
}

rangerHitWire::rangerHitWire(ArtePointer<HITB> hb,rangerFitMode fm) 
: rangerHit(hb,fm) {
  LRSign = 1;
  m = new HepVector(1);
  (*m)(1) = LRSign*fabs(hb->q);
  V = new HepSymMatrix(1);
  (*V)(1,1) = (hb->r) * (hb->r);
}

rangerHitWire::~rangerHitWire() {
  delete m;
  delete V;
//  cout << "The HitWire destructor deletes m and V" << endl;
}

void rangerHitWire::projectP(HepVector& resid,
			     HepMatrix& HH,rangerStateP* s) {

#ifdef FULL
  cout << "This is rangerHitWire::projectP" << endl;
#endif
  ArtePointer<GEDE> gede = getHitb()->gede;
#ifdef FULL
  cout << "                       hitb=" << getHitb() << endl;
  cout << "                      xhitb=" << getHitb()->x << endl;
  cout << "                       gede=" << gede << endl;
#endif
  const float& cosTot = rangerGeom::vol( gede )->Cos();
  const float& sinTot = rangerGeom::vol( gede )->Sin();
  double tanThetaU = s->tx() * cosTot - s->ty() * sinTot;
  double cosThetaU = 1 / sqrt( 1 + tanThetaU * tanThetaU);
  double deltaU = s->x() * cosTot - s->y() * sinTot - getHitb()->x;
  double Hx = deltaU * cosThetaU;
  
  resid(1) = (*m)(1) - Hx;

#ifdef FULL
  cout << "rangerHitWire::projectP: the stereo angle is " << gede->sin << endl;
  cout << "rangerHitWire::projectP: the residual is " << deltaU <<
       " error " << sqrt((*V)(1,1)) << endl;
#endif

  HH(1,1) =   cosTot * cosThetaU;
  HH(1,2) = - sinTot * cosThetaU;
  double ct3= cosThetaU * cosThetaU * cosThetaU;
  HH(1,3) = - deltaU * tanThetaU * ct3 * cosTot;
  HH(1,4) =   deltaU * tanThetaU * ct3 * sinTot;
  HH(1,5) = 0;  
  
}

void rangerHitWire::projectPI(HepVector& resid,
			      HepMatrix& HH,rangerStatePI* s) {

#ifdef FULL
  cout << "This is rangerHitWire::projectPI" << endl;
#endif
  ArtePointer<GEDE> gede = getHitb()->gede;
#ifdef FULL
  cout << "                       hitb=" << getHitb() << endl;
  cout << "                      xhitb=" << getHitb()->x << endl;
  cout << "                       gede=" << gede << endl;
#endif
  const float& cosTot = rangerGeom::vol( gede )->Cos();
  const float& sinTot = rangerGeom::vol( gede )->Sin();
  double tanThetaU = s->tx() * cosTot - s->ty() * sinTot;
  double cosThetaU = 1 / sqrt( 1 + tanThetaU * tanThetaU);
  double deltaU = s->x() * cosTot - s->y() * sinTot - getHitb()->x;
  // assume that the drift velocity is 0.01 cm/ns
  double Hx = deltaU * cosThetaU - LRSign * 0.01 * s->I();

#ifdef NEW_MODEL
  // Wouter's new measurement model
  double newmeasurement = (*m)(1) * fabs(tanh( (*m)(1) * Hx / (*V)(1,1) )) ;
  resid(1) = newmeasurement - Hx;
#else
  resid(1) = (*m)(1) - Hx;
#endif

#ifdef FULL
  cout << "rangerHitWire::projectPI: the stereo angle is " 
       << gede->sin << endl;
  cout << "rangerHitWire::projectPI: the residual is " << deltaU <<
       " error " << sqrt((*V)(1,1)) << endl;
#endif

  HH(1,1) =   cosTot * cosThetaU;
  HH(1,2) = - sinTot * cosThetaU;
  double ct3= cosThetaU * cosThetaU * cosThetaU;
  HH(1,3) = - deltaU * tanThetaU * ct3 * cosTot;
  HH(1,4) =   deltaU * tanThetaU * ct3 * sinTot;
  HH(1,5) = 0;
  // assume that the drift velocity is 0.01 cm/ns
  HH(1,6) = -LRSign * 0.01;

}


void rangerHitWire::projectL(HepVector& resid,
			     HepMatrix& HH,rangerStateL* s) {

#ifdef FULL
  cout << "This is rangerHitWire::projectL" << endl;
#endif
  ArtePointer<GEDE> gede = getHitb()->gede;
#ifdef FULL
  cout << "                       hitb=" << getHitb() << endl;
  cout << "                      xhitb=" << getHitb()->x << endl;
  cout << "                       gede=" << gede << endl;
#endif
  const float& cosTot = rangerGeom::vol( gede )->Cos();
  const float& sinTot = rangerGeom::vol( gede )->Sin();
  double tanThetaU = s->tx() * cosTot - s->ty() * sinTot;
  double cosThetaU = 1 / sqrt( 1 + tanThetaU * tanThetaU);
  double deltaU = s->x() * cosTot - s->y() * sinTot - getHitb()->x;
  double Hx = deltaU * cosThetaU;
#ifdef NEW_MODEL
  // Wouter's new measurement model
  double newmeasurement = (*m)(1) * fabs(tanh( (*m)(1) * Hx / (*V)(1,1) )) ;
  resid(1) = newmeasurement - Hx;
#else
  resid(1) = (*m)(1) - Hx;
#endif

#ifdef FULL
  cout << "rangerHitWire::projectL: the stereo angle is " << gede->sin << endl;
  cout << "rangerHitWire::projectL: the residual is " << deltaU <<
       " error " << sqrt((*V)(1,1)) << endl;
#endif

  HH(1,1) =   cosTot * cosThetaU;
  HH(1,2) = - sinTot * cosThetaU;
  double ct3= cosThetaU * cosThetaU * cosThetaU;
  HH(1,3) = - deltaU * tanThetaU * ct3 * cosTot;
  HH(1,4) =   deltaU * tanThetaU * ct3 * sinTot;

}


void rangerHitWire::projectLI(HepVector& resid,
			      HepMatrix& HH,rangerStateLI* s) {

#ifdef FULL
  cout << "This is rangerHitWire::projectLI" << endl;
#endif
  ArtePointer<GEDE> gede = getHitb()->gede;
#ifdef FULL
  cout << "                       hitb=" << getHitb() << endl;
  cout << "                      xhitb=" << getHitb()->x << endl;
  cout << "                       gede=" << gede << endl;
#endif
  const float& cosTot = rangerGeom::vol( gede )->Cos();
  const float& sinTot = rangerGeom::vol( gede )->Sin();
  double tanThetaU = s->tx() * cosTot - s->ty() * sinTot;
  double cosThetaU = 1 / sqrt( 1 + tanThetaU * tanThetaU);
  double deltaU = s->x() * cosTot - s->y() * sinTot - getHitb()->x;
  // assume that the drift velocity is 0.01 cm/ns
  double Hx = deltaU * cosThetaU - LRSign * 0.01 * s->I();
#ifdef NEW_MODEL
  // Wouter's new measurement model
  double newmeasurement = (*m)(1) * fabs(tanh( (*m)(1) * Hx / (*V)(1,1) )) ;
  resid(1) = newmeasurement - Hx;
#else
  resid(1) = (*m)(1) - Hx;
#endif

#ifdef FULL
  cout << "rangerHitWire::projectLI: the stereo angle is " 
       << gede->sin << endl;
  cout << "rangerHitWire::projectLI: the residual is " << deltaU <<
       " error " << sqrt((*V)(1,1)) << endl;
#endif

  HH(1,1) =   cosTot * cosThetaU;
  HH(1,2) = - sinTot * cosThetaU;
  double ct3= cosThetaU * cosThetaU * cosThetaU;
  HH(1,3) = - deltaU * tanThetaU * ct3 * cosTot;
  HH(1,4) =   deltaU * tanThetaU * ct3 * sinTot;
  // assume that the drift velocity is 0.01 cm/ns
  HH(1,5) = -LRSign * 0.01;
}

void rangerHitWire::updInternal(){
  
  if (!r) return;
  
  double scale = (*getCovMeasurement())(1,1)/(*getCovResidual())(1,1) ;
  // this is the residual `as if' hit were not used in fit
  double bareResidual = (*r)(1) ;
  if(scale>1) bareResidual *= scale ;
  // this is where the track would be if hit was not used in fit:
  float bareD = (*m)(1) - bareResidual ;
  if(bareD*LRSign<0) {
    LRSign  = - LRSign;
    (*m)(1) = - (*m)(1);
    // this is what happens to bare residual if we flip sign:
    bareResidual += 2*(*m)(1) ;    
    // so this is our estimate of what the residual would be if we include hit again:
    if(scale>1) bareResidual /= scale ;
    (*r)(1) = bareResidual ;
  }
}

