// 
//  Author:  Wouter Hulsbergen                 Created:  14-Aug-00
//  Modifications:
//

#include <iostream.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "rang/StateP.hh"
#include "rang/StatePI.hh"
#include "rang/StateL.hh"
#include "rang/StateLI.hh"
#include "rang/MatchedRsegL.hh"
#include "rang/ScatThick.hh"
#include "rang/Geom.hh"
#include "rang/zfpar.hh"
#include "rang/utfeld.hh"
#include "rang/RadEl.hh"

//-- Constructor
rangerMatchedRsegL::rangerMatchedRsegL() : fRseg(0), fSide(0), fPlane(0)
{
  cout << "This is the rangerMatchedRsegL default constructor" << endl;
}

rangerMatchedRsegL::rangerMatchedRsegL(ArtePointer<RSEG> rseg,rangerFitMode fm, int side)
  : rangerHitBase(rseg,fm,side), fRseg(rseg), fSide(side)
{
#ifdef FULL
  cout << "This is the rangerMatchedRsegL(rseg) constructor, rseg=" << rseg << endl ;
#endif
  fDim=4 ;
  if(rseg->zf>680 || rseg->ze<250) fDim=4; 
  
  m = new HepVector(dim());
  V = new HepSymMatrix(dim());
  float *x, *c ;
  
  if(side==0) {
    x = &rseg->xf ;
    c = rseg->cvf ;
  } else {
    x = &rseg->xe ;
    c = rseg->cve ;
  } 

  int j=0;
  for(int row=1; row<=dim(); ++row) {
    (*m)(row) = x[row-1] ;
    for(int col=1; col<=row; ++col, ++j)
      (*V).fast(row,col) = c[j] ;
  }

  // find the plane
  fPlane = 0;
}

rangerMatchedRsegL::~rangerMatchedRsegL()
{
  delete m;
  delete V;
}

void rangerMatchedRsegL::project(HepVector& resid,
			       HepMatrix& HH,rangerState* s) 
{
#ifdef FULL
  cout << "This is rangerMatchedRsegL::projectL" << endl;
#endif
  resid = (*m) - s->vec().sub(1,4) ;
  for(int row=1; row<=dim(); row++)
    for(int col=1; col<=s->dim(); col++)
      HH(row,col) = (row==col && col<=4) ? 1 : 0 ;
}

void rangerMatchedRsegL::projectL(HepVector& resid,
				HepMatrix& HH,rangerStateL* s) 
{
#ifdef FULL
  cout << "This is rangerMatchedRsegL::projectP" << endl;
#endif
  project(resid,HH,s) ;
}

void rangerMatchedRsegL::projectP(HepVector& resid,
				HepMatrix& HH,rangerStateP* s) 
{
#ifdef FULL
  cout << "This is rangerMatchedRsegL::projectP" << endl;
#endif
  project(resid,HH,s) ;
}


void rangerMatchedRsegL::projectPI(HepVector& resid,
			     HepMatrix& HH,rangerStatePI* s) 
{
  project(resid,HH,s) ;
}


void rangerMatchedRsegL::projectLI(HepVector& resid,
			     HepMatrix& HH,rangerStateLI* s)
{
#ifdef FULL
  cout << "This is rangerMatchedRsegL::projectL" << endl;
#endif
  project(resid,HH,s) ;
}
