// 
//  Author:  Rainer Mankel                     Created:  23-Apr-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

// rgstabl
// Fortran interface to rangerDomObj::stab().
// The seed tableau pointers in the global ranger domain object, and
// the associated propagation directions are stored into a fixed-length 
// array which must be defined by the calling program.

#include "rang/SeedTableau.hh"
#include "rang/DomObj.hh"
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

extern rangerDomObj* rangerDO;   // global DO pointer

extern "C" {
  void rgstabl_(const int& maxTab,           // avail. space in array (input)
		rangerSeedTableau* tabVec[], // array for output
		int dirVec[],                // array for directions
		int& nTab) {                 // number of items on output

    if (!rangerDO) {
      cout << "rgstabl: rangerDO=" << rangerDO << endl;
      return;
    }

    nTab = 0;

    // loop over tableaus
    for (vector<rangerSeedTableau*>::iterator sti=rangerDO->stab().begin();
	 sti != rangerDO->stab().end(); ++sti) {
      if (nTab < maxTab) {
	tabVec[nTab] = (*sti);
	dirVec[nTab] = (*sti)->direction();
	++nTab;
      }
      else {
	arteMsg << "rgstabl: overflowing nTab=" << nTab << endl;
	arteMsg.warn();
	break;
      }
    }
  }
}




    



