// ********************************************************************
//
// Routine:      rc99Bremsstrahlung
// Author:       T. Oest
//
// Description:  this routine returns the Bremsstrahlung
//               for a given track
//
// Modification:
// 13/11/00 (DS) radius for Bremsstrahlung search fixed to 4 cm
//               (before old momentum dependent approximation)
// 28/11/00 (BG) radius for brem search is given by the cluster cell width
// 30/11/00 (AG) some polishing                 
//
// ********************************************************************

// includes
#include <iostream.h>
#include <stdlib.h>
#include <math.h>
#include <algo.h>

// ARTE includes
#include "arte/arte.hh"
#include "farm/farm.hh"

//
// function
//
float
rc99Bremsstrahlung(ArtePointer<RTRA> rtra)
{
  ARTE_mark( "rc99Bremsstrahlung 0", 0 );

  //-----------------------------------------------------------------
  //   select two body decays
  //-----------------------------------------------------------------
  
  float energy    = 0;
  float minImpact = 100000;
  float yimpact   = 0; 
    float ximpact   = 0; 
  float rimpact   = 0;
  float ycut      = 0;
  float dimcell   = 11.15;
  
  ArteRelationIterator<RCCL> rcclTrack  = rtra->rccl.begin();
  
  for( ArteTable<RCCL>::iterator rccl = ArteTable<RCCL>::begin();
       rccl != ArteTable<RCCL>::end(); ++rccl ) {
    
    if ((*rccl) != rcclTrack){
      
      yimpact  = rtra->yf + rtra->tyf * ((*rccl)->z - rtra->zf) - (*rccl)->y;
      ximpact  = rtra->xf + rtra->txf * ((*rccl)->z - rtra->zf) - (*rccl)->x;
      rimpact  = sqrt (ximpact*ximpact + yimpact*yimpact);
      if (rimpact < minImpact) {
	minImpact = rimpact;
	energy    = (*rccl)->esum;
	ycut = dimcell/((float)((int)((*rccl)->flag) & 0x7));
      }
    }
  }
  
  // float ycut = (1.0 + 3. * (1. - energy / 100.))/sqrt(2);
  
  if (minImpact > ycut) energy = 0.0;  

  ARTE_mark( "rc99Bremsstrahlung 1", 0 );
  return energy;
}



