
// *****************************************************************************
//
// source:    rcParticleID.C
//
// type:      source code
//
// created:   01 Dec 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Department of Physics
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    rcParticleID
//
// called by: rcevnt
//
// purpose:   invoke pid functions
//
// changes:
// 14 Sep 1999 (TO) creation
// 01 Dec 2000 (AG) introduced MuonID()
//  6 Dec 2000 (AG) polished printout
// 02 Jan 2001 (AG) removed 99A
// 19 Feb 2001 (AG) cured print-out bug
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>

// ARTE includes
#include "arte/Chrono.hh"
#include "arte/ChronoManager.hh"
#include "reco/rcflags.hh"
#include "farm/farm.hh"

// externals
extern int FADEB;
extern int FATIME;

// prototypes
int rcPidRICH( int module );
extern "C" void careelid_();
extern "C" void MuonID();

//********************************************************************
// rcParticleID
//********************************************************************
int 
rcParticleID()
{
  ARTE_mark( "rcParticleID 1", 0 );

  // inits
  int RICHidFlag = 0;
  int ECALidFlag = 0;

  // timers
  static Chrono *cron_rcPidRICH = 0;
  static Chrono *cron_rcPidECAL = 0;
  static Chrono *cron_rcPidMUON = 0;

  static bool first = 1;
  if( first ){
    first = 0;
    ChronoManager CM;
    CM.instance();
    
    cron_rcPidRICH = CM.createChrono( "rcPidRICH" );
    cron_rcPidECAL = CM.createChrono( "rcPidECAL" );
    cron_rcPidMUON = CM.createChrono( "rcPidMUON" );
  }

  // inits
  float time_rcPidRICH = 0;
  float time_rcPidECAL = 0;
  float time_rcPidMUON = 0;

  //===================================================================
  //  RICH particle ID for electrons
  //===================================================================
  if( FADEB ) cout << "rcParticleID: recoPidRICH=" << recoPidRICH << endl;
  if( recoPidRICH ){
    cron_rcPidRICH->start();
    ARTE_mark( "rcPidRICH", 0 );
    RICHidFlag = rcPidRICH( recoPidRICH ) ;
    cron_rcPidRICH->stop();
    time_rcPidRICH = cron_rcPidRICH->lastTime();
  }
  
  //===================================================================
  //  ECAL particle ID for electrons & gammas
  //===================================================================
  if( FADEB ) cout << "rcParticleID: recoPidECAL=" << recoPidECAL << endl;
  if( recoPidECAL ){
    cron_rcPidECAL->start();
    ARTE_mark( "careelid", 0 );
    careelid_();
    cron_rcPidECAL->stop();
    time_rcPidECAL = cron_rcPidECAL->lastTime();
  }
  
  //===================================================================
  //  MUON particle ID
  //===================================================================
  if( FADEB ) cout << "rcParticleID: recoPidMUON=" << recoPidMUON << endl;
  if( recoPidMUON ){
    cron_rcPidMUON->start();
    ARTE_mark( "MuonID", 0 );
    MuonID();
    cron_rcPidMUON->stop();
    time_rcPidMUON = cron_rcPidMUON->lastTime(); 
  }
  
  // print timing
  if( FATIME ){
    printf( "\033[1m TIME: rich=%4.2f ecal=%4.2f muon=%4.2f \033[0m \n",
	    time_rcPidRICH, time_rcPidECAL, time_rcPidMUON );
  }
  
  // return
  ARTE_mark( "rcParticleID 1", 0 );
  return 0;
}
