
// *****************************************************************************
//
// source:    rckamodsel.C
//
// type:      source code
//
// created:   25 Jul 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    kamodsel_
//
// called by: 
//
// purpose:   handle kuip commands /RECON/MODSEL
//
// changes:
// 25 Jul 2000 (AG) creation
// 27 Jul 2000 (AG) improved printout
// 04 Aug 2000 (AG) cured bug
// 11 Aug 2000 (AG) added argument to MODSEL to chose mode
// 03 Jan 2000 (AG) removed 99A
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

// ARTE includes
#include "arte/arte.hh"
#include "farm/farm.hh"

// globals
int MODSEL = 0;

// externals
extern int FADEB;

// prototypes
extern "C" void kugetc_( char*, int *, int );

//
// function code
//
extern "C" void
kamodsel_()
{
  ARTE_mark( "kamodsel_ 0", 0 );

  //
  // read in name of selection
  //
  int len  = 0;
  int len1 = 8;
  char input[9];
  memset( input, '\0', 9 );
  kugetc_( input, &len, len1 );
  input[len] = '\0';
  char *name  = (char *) malloc( len+1 );
  memset( name, '\0', len+1 );
  sprintf( name  ,"%s", input );
  if( len>0 ) for( int i=0; i<len; i++ ) name[i] = toupper( name[i] );

  // check name
  if( !strncmp( name, "SLTBASED", 8 )
      ){
    if( FADEB ) cout << "name=" << name << endl;
  }
  else {
    cout << "name=" << name << " UNKOWN!" << endl;
    return;
  }
    
  //
  // read in status
  //
  len  = 0;
  len1 = 3;
  memset( input, '\0', 9 );
  kugetc_( input, &len, len1 );
  input[len] = '\0';
  char *status  = (char *) malloc( len+1 );
  memset( status, '\0', len+1 );
  sprintf( status  ,"%s", input );
  if( len>0 ) for( int i=0; i<len; i++ ) status[i] = toupper( status[i] );

  // check status
  if( !strncmp( status, "ON",  2 ) ||
      !strncmp( status, "OFF", 3 )
      ){
    if( FADEB ) cout << "status=" << status << endl;
  }
  else {
    cout << "status=" << status << " UNKOWN!" << endl;
    return;
  }
  
  //
  // print
  //
  if( FADEB ) cout << "RECON/MODSEL " << name << " " << status << endl;
  
  //
  // set flag
  //
  if( !strncmp( status, "ON", 2 ) ){
    if( !strncmp( name, "SLTBASED", 8 ) ) MODSEL = 1;
  }
  else {
    if( !strncmp( name, "SLTBASED", 8 ) ) MODSEL = 0;
  }
  
  //
  // print
  //
  cout << "RECON/MODSEL: MODSEL=" << MODSEL << endl;
  
  ARTE_mark( "kamodsel_ 1", 0 );
}
