#ifndef CROBUSTMEAN_HH
#define CROBUSTMEAN_HH

// Implementation file for the median class
// =======================================================================
// Author: Martin Braeuer 
//
//    From: 'Robust Estimation and Hypothesis testing'
//    (R. R. Wilcox, Academic Press, 1997)
//
// Modification history:
//
// MBr  12.01.2001: 1. Implementaion according to 
// MBr  23.01.2001: Compile problems removed          
// MBr  00.00.2001:                   
// =======================================================================

#include "CMedian.h"

#define EPSILON 1E-10

class CRobustMeans : public CMedian {
 public: // ---------------------------------------- PUBLIC :

  /* ========================================================
   *  Def. Constructor, reserves space for 1000 elements
   * ========================================================
   */
  CRobustMeans();

  /* ========================================================
   *  Constructor, reserves space for 'iNumber' elements
   * ========================================================
   */
  CRobustMeans(int iNumber) : CMedian(iNumber) {Setall();};

  /* ========================================================
   *  Def. Destructor, frees the space
   * ========================================================
   */  
   ~CRobustMeans();


  /* ========================================================
   *  GetHuberMean  : Returns the robust mean, using hubers PSI
   *   fCutOffConst : +- this value normality is assumed, 
   *                  exponential outside
   * ========================================================
   */
  float GetHuberMean(float fCutOffConst = 1.28);
  
  /* ========================================================
   *  GetHampelMean  : Returns the robust mean, using hubers PSI
   *   fValA : +- this value normality is assumed, 
   *   fValB : from A to B exponential tails are assumed
   *   fValC : outside +- C a flat noise is assumed
   * ========================================================
   */
  float GetHampelMean(const float fValA = 2.5,
		      const float fValB = 5.0,
		      const float fValC = 7.5 );

  float BiWeightMidVariance(float fScale=9.0);
  float MedianBendMidvariance(float fScale=1.0);

  // Not really ready:....
  float GetHuberMeanErr(float fCutOffConst = 1.28);

  
  /* ========================================================
   *  Reset  : Reset the entries for further use
   * ========================================================
   */
  void  Reset();

 private: // ------------- PRIVATE:
  // Preload the constants:
  void Setall();

  // steering parameter:
  int    iMaxHuberIts; // max no of iterations
  double dHuberBreak;  // break-off value
  int    iMaxHampelIts; // max no of iterations
  double dHampelBreak;  // break-off value
 
  // done flags:
  int    iHuberMean;    // huber already done
  int    iHampelMean;    // huber already done
  int    iHuberMeanErr; // huber already done

  // values:
  float fHubMean,fHamMean,fHubMeanErr;
  float fBiWeiMidVar,fMedBendMidVar;

}; // of class def.

#endif // end of class definiton
// =========== EOF ===== EOF ===== EOF ===== EOF ===== EOF ===== EOF =============



