// =================================================================
//
// silarte_masking.C
//
// G. Sciacca - MPI-K Heidelberg
//
// Called by: fillHITBfromSPARSEVTmask.C
// 
// Retrieve VDSchipmasking maps from Database /MC_MASEF
//
// Return values:
//		0 => OK
//		1 => error
//
// DBS: default is /MC_MASEF - Can be changed by setenv DB_VDS_MASK
//
// 16.03.01 - (GS) Conceived
// 17.03.01 - (GS) First version filling struct VDSchipmask_struct
// 26.06.01 - (GS) Final version filling struct VDSchipmask_struct
// =================================================================

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <math.h>
#include <ctype.h>
#include <sys/time.h>
#include "database/table.h"
#include "database/keytable.h"
#include "silarte.hh"
#include "silarte/silarte.hh"
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "grr.h"   
   
int silarte_masking(char *cs)
{
   int rc = 0;	//  OK
   int error = 0;
   char *routine = "silarte_masking";

   Tables x;
   int t;
   int major = 0;
   int minor = 0;

   VDSchipmask_struct VDSchipmask[MAXPLANES];

   // init structure -> all chips OK by default
   for (int p=0; p<MAXPLANES; p++) {
     VDSchipmask[p].usid = p;
     VDSchipmask[p].sideid = p;
     VDSchipmask[p].chipstat0 = (char *) 1;
     VDSchipmask[p].chipstat1 = (char *) 1;
     VDSchipmask[p].chipstat2 = (char *) 1;
     VDSchipmask[p].chipstat3 = (char *) 1;
     VDSchipmask[p].chipstat4 = (char *) 1;
     VDSchipmask[p].chipstat5 = (char *) 1;
     VDSchipmask[p].chipstat6 = (char *) 1;
     VDSchipmask[p].chipstat7 = (char *) 1;
     VDSchipmask[p].chipstat8 = (char *) 1;
     VDSchipmask[p].chipstat9 = (char *) 1;
   }
   ArtePointer < EVHD > evhd (1);
   int run = evhd->run;

//printf("address received = %p\n", cs);
   major = run;
   char *message = 0;
   if (!message) message = new char[512];

   cout<<""<<endl;
   sprintf(message,
	"%s: VDSchipmasking table relative to Run %i will be retrived",
	routine,run);
   cout<<message<<endl;

// ------------------------------------------------------------------
// database server (default: /MC_MASEF)
//                  can be changed by setting DB_VDS_MASK)
// ------------------------------------------------------------------
  char defaultserver[11];
  sprintf(defaultserver,"/MC_MASEF");
  char * dbserver;
  if (getenv("DB_VDS_MASK") == NULL){
    dbserver = defaultserver;
  } else {
    dbserver = getenv("DB_VDS_MASK");
  };

// ------------------------------------------------------------------
// Looking up and retrieving VDSchipmasking
// ------------------------------------------------------------------
   
   // Opening and booking tables
   sprintf(message,
	"%s: Opening Masking & Efficiency DB server %s", routine,dbserver);
   cout<<message<<endl;

   if(!(x=OpenTables(dbserver,TABLEreadonly))) {
     sprintf(message,
	"%s: Error opening DB server %s\n", routine,dbserver);
     ARTE_exit(GRR_SFATAL, message, 1);	//return 1
   }
   if((t=BookTable(x,"VDSchipmasking",0,0))==0) {
     sprintf(message,
	"%s: Error booking table VDSchipmasking\n", routine);
     ARTE_exit(GRR_SFATAL, message, 1);	//return 1
   }

   // Setting major and minor
   if (major == 0) {				// getting last revision if major=0
     TableStamp(x,TABLEmaxrev,TABLEmaxrev);
     if (!(MatchTable(x,t,TABLElt))) {          // goes to the last revision of table
       sprintf(message,
	  "%s: Error matching table VDSchipmasking\n", routine);
       ARTE_exit(GRR_SFATAL, message, 1);	//return 1
     }
     major = TableMajor(x);
     minor = TableMinor(x);
   } else {					// getting table .le. specified revision
     TableStamp(x,major,TABLEmaxrev);
     if (!(MatchTable(x,t,TABLEle))) {
       sprintf(message,
	  "%s: Error matching table VDSchipmasking\n", routine);
       ARTE_exit(GRR_SFATAL, message, 1);	//return 1
     }
     major = TableMajor(x);
     minor = TableMinor(x);
   }
   TableStamp(x,major,minor);
   if (major == 1) {  
     sprintf(message,
	"%s: **WARNING**: No entries found in database for table VDSchipmasking: Loading default map",
	routine);
     cout<<message<<endl;
     sprintf(message,
	"%s: **WARNING**: Assuming ALL VDS Helix-chips are OK!!", routine);
     cout<<message<<endl;
   } else {
     sprintf(message,
	"%s: Retrieving VDSchipmasking table for (major, minor) (%i, %i)",
                        routine,major,minor);
     cout<<message<<endl;
   }
   // Retrieving tables
   if(FetchTable(x,t,TABLEeq)==0) {
     sprintf(message,
	"%s: Error fetching table VDSchipmasking\n", routine);
     ARTE_exit(GRR_SFATAL, message, 1);	//return 1
   }

   int structsize = sizeof(VDSchipmask_struct);

   SkipTableField(x,1);
   rc = GetTableStruct(x,TABLEint,structsize,MAXPLANES,&VDSchipmask[0].usid);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i usid elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   rc = GetTableStruct(x,TABLEint,structsize,MAXPLANES,&VDSchipmask[0].sideid);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i sideid elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   rc = GetTableStruct(x,TABLEbyte,structsize,MAXPLANES,&VDSchipmask[0].chipstat0);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i chipstat0 elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   rc = GetTableStruct(x,TABLEbyte,structsize,MAXPLANES,&VDSchipmask[0].chipstat1);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i chipstat1 elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   rc = GetTableStruct(x,TABLEbyte,structsize,MAXPLANES,&VDSchipmask[0].chipstat2);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i chipstat2 elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   rc = GetTableStruct(x,TABLEbyte,structsize,MAXPLANES,&VDSchipmask[0].chipstat3);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i chipstat3 elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   rc = GetTableStruct(x,TABLEbyte,structsize,MAXPLANES,&VDSchipmask[0].chipstat4);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i chipstat4 elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   rc = GetTableStruct(x,TABLEbyte,structsize,MAXPLANES,&VDSchipmask[0].chipstat5);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i chipstat5 elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   rc = GetTableStruct(x,TABLEbyte,structsize,MAXPLANES,&VDSchipmask[0].chipstat6);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i chipstat6 elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   rc = GetTableStruct(x,TABLEbyte,structsize,MAXPLANES,&VDSchipmask[0].chipstat7);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i chipstat7 elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   rc = GetTableStruct(x,TABLEbyte,structsize,MAXPLANES,&VDSchipmask[0].chipstat8);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i chipstat8 elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   rc = GetTableStruct(x,TABLEbyte,structsize,MAXPLANES,&VDSchipmask[0].chipstat9);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i chipstat9 elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   
// --------------------------------------------------------
// Unbooking and closing tables
// -------------------------------------------------------- 
   sprintf(message,
	"%s: Closing Masking & Efficiency DB server %s", routine,dbserver);
   cout<<message<<endl;

   if(!UnbookTable(x,t)) {
     sprintf(message,
	"%s: Error unbooking table VDSchipmasking\n", routine);
     ARTE_error(GRR_SERROR, message, 0);
   }
   if(!CloseTables(x)) {
     sprintf(message,
	"%s: Error closing table VDSchipmasking\n", routine);
     ARTE_error(GRR_SERROR, message, 0);
   }

/*   printf("# Contents of VDSchipmask table:\n");
   printf("# --------------------------------------------------------------------------------------");
   printf("#\n");
//   printf("# Contents for major, minor = %i, %i\n", major, minor);
//   printf("#          => major is the RunN\n");
//   printf("#\n");
   printf("# UsId SideId Deadchips\n");
   printf("#\n");

   for (int j=0; j<MAXPLANES; j++) {
     printf("++usid[%i] = %i sideid = %i - %d %d %d %d %d %d %d %d %d %d \n",
            j,VDSchipmask[j].usid,VDSchipmask[j].sideid,VDSchipmask[j].chipstat0,
            VDSchipmask[j].chipstat1,VDSchipmask[j].chipstat2,VDSchipmask[j].chipstat3,
            VDSchipmask[j].chipstat4,VDSchipmask[j].chipstat5,VDSchipmask[j].chipstat6,
            VDSchipmask[j].chipstat7,VDSchipmask[j].chipstat8,VDSchipmask[j].chipstat9);
   }
*/
// --------------------------------------------------------
// Fill Martin's array
// --------------------------------------------------------
   // loop over chipstat entries retrieved from DB
   for (int i=0; i<MAXPLANES; i++) {	//planes
//printf("address received = %p\n", cs);
    for (int y=0; y<10; y++){		 //chips in plane
         switch (y) {
            case 0:
		*cs = (char) VDSchipmask[i].chipstat0;
		cs++;
		break;
            case 1:
		*cs = (char) VDSchipmask[i].chipstat1;
		cs++;
		break;
	    case 2:
		*cs = (char) VDSchipmask[i].chipstat2;
		cs++;
		break;
            case 3:
		*cs = (char) VDSchipmask[i].chipstat3;
		cs++;
		break;
            case 4:
		*cs = (char) VDSchipmask[i].chipstat4;
		cs++;
		break;
            case 5:
		*cs = (char) VDSchipmask[i].chipstat5;
		cs++;
		break;
            case 6:
		*cs = (char) VDSchipmask[i].chipstat6;
		cs++;
		break;
            case 7:
		*cs = (char) VDSchipmask[i].chipstat7;
		cs++;
		break;
            case 8:
		*cs = (char) VDSchipmask[i].chipstat8;
		cs++;
		break;
            case 9:
		*cs = (char) VDSchipmask[i].chipstat9;
		cs++;
              break;
            default:
		sprintf(message,
		 "%s: Inconsistency - VDS chip %i on plane %i does NOT exist!!\n", routine,y,i);
		ARTE_error(GRR_SERROR, message, 0);
		error = 1;
         } // switch

    }	 // end of chips in plane
   }	// end of planes
   // end of loop over efficiency entries retrieved from DB

   if (error) {
     sprintf(message,
	   "%s: *** WARNING *** VDS masking map NOT succesfully loaded\n", routine);
     cout<<message<<endl;
     return 1;
   }
  sprintf(message,
	"%s: VDS masking map succesfully loaded\n", routine);
  cout<<message<<endl;

  return GRR_OK;
}
