//*****************************************************************************
//
// source:    sltsim.C
//
// type:      source code
//
// created:   29 Dec 2000
//      
// author:    Brian Petersen, E-mail: aagaard@nbi.dk
//            Lars Soezueer <soezueer@mail.desy.de>
//
// symbol:    sltsim.C
//
// called by: usevnt 
//
// purpose:   Run SLT simulation by piping data to and from a standalone SLT
//            process
//
// hints:     
//
// changes:
// 29 Dec 2000 (BP) Creation
// 16 Mar 2001 (LS) Adaption to user-friendly daqif/sltsim scheme
// 5  Jul 2001 (BP) Add sltsimkuip command initialization
// 5  Aug 2001 (BP) Fix include problem under Irix
// 25 Aug 2001 (BP) Let namserver remove itself cleanly
//
//*****************************************************************************
#include <iostream.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/signal.h>
#include "sltinfo.h"
#include "smc.h"
#include "rpm.h"
#include "grr.h"
#include <fcntl.h>
#include "arte/ArteMessage.hh"
#include "care/CmdCareECDB.hh"
#include "daqif/daqif.hh"
#include "sltarte/sltsim.hh"
#include "sltarte/kasltsim.hh"
#include "arte/arte.hh"
#include "farm/farm.hh"
// prototypes
extern "C" void sltsimkuip_();
//
// initialize the Sltsim KUIP commands
//
static class SLTSIMIniter
{
public:
  SLTSIMIniter() { atregkuip( sltsimkuip_, 1 ); }
} _sltsimIniter;


extern "C" {
#include "l2format.h"
// Could not find these two anywhere 
  int kill(pid_t,int);
  int setenv(const char*, const char*, int);
}

/***
   * create an SLT_DOM from the pid of the ARTE job and the hostname
   *
   ***/

char* setget_slt_dom(void) {
  char* hostname=getenv("HOST");
  if (! hostname) hostname="unknown";
  char* slt_dom=new char[80];
  sprintf(slt_dom, "/SLT%d%s/", (int)getpid(), hostname);
  setenv("SLT_DOM", slt_dom, 1);
  //  cout << "Set SLT_DOM to " << slt_dom << endl;
  return slt_dom;
}


/***
   * fork various processes for State Machine Control
   *
   ***/

int fork_SMC(void)
{
  int rc=0;

  char *hbbintype=getenv("HBBINTYPE");
  if (hbbintype==NULL) hbbintype="";
  if (  (strcmp(hbbintype,"Linux_intel")==0)
      ||(strcmp(hbbintype,"Linux_intel_glibc")==0) ) { // if non-Linux, external SMC GUI needed
    //prepare arguments 

    char* slt_dom=setget_slt_dom();

    char* hbroot=getenv("HBROOT");
    if (! hbroot) {
      hbroot="/afs/desy.de/group/hera-b";
      arteMsg << "sltsim.C: $HBROOT not set, assume " << hbroot;
      arteMsg.warn();
    }

    char* path=new char[256];
    path[0]=0;
    strcpy(path,hbroot);
    path=strcat(path,"/ONLINE/dev/");
    path=strcat(path,hbbintype);
    path=strcat(path,"/bin/");

    char *domain=new char[80];
    strcpy(domain,"-Domain:");
    domain=strcat(domain,slt_dom);

    char *process=new char[80];
    strcpy(process,"-Process:");
    process=strcat(process,slt_dom);

    char *dom_nam=new char[80];
    strcpy(dom_nam,slt_dom);
    dom_nam=strcat(dom_nam,"RC/top");

    // fork namserve:
    int proc;
    switch(proc=fork()) {
      case -1:
        arteMsg << "sltsim.C: error forking namserve" ;
        arteMsg.warn();
        return 1;
      case 0:
	std::close(STDOUT_FILENO); // Get rid of output from namserve
	cout << "path 1= " << path << endl; 
        path=strcat(path,"namserve");
	cout << "path 2= " << path << endl; 
        execlp(path,"namserve",process,0);
	cout << "path 3= " << path << endl; 
        exit(1);
    } 
    sltsim_AddProcess(proc);
    sleep(2);

    // fork grrlog
    switch(proc=fork()) {
      case -1:
        arteMsg << "sltsim.C: error forking grrlog" ;
        arteMsg.warn();
        return 1;
      case 0:
        path=strcat(path,"grrlog");
        execlp(path,"grrlog","-Process:RC/logger",domain,0);
        exit(1);
    } 
    sltsim_AddProcess(proc);
    sleep(2);

    // fork erwin
    if (kasltsim::get_debuglevel()) {
      switch(proc=fork()) {
        case -1:
          arteMsg << "sltsim.C: error forking erwin" ;
          arteMsg.warn();
          return 1;
        case 0:
          path=strcat(path,"erwin");
          execlp(path,"erwin",domain,"-q","-s","0",0);
         exit(1);
      }
      sltsim_AddProcess(proc);
      //      cout << "Added erwin proc = " << proc << endl;
      sleep(2);
    } // if (kasltsim::get_debuglevel)

  } //if (  (strcmp(hbbintype ...
  else {
      cout << "State Machine Control has to be run from a Linux_intel machine." << endl;
      cout << "Please call SMC GUI on such a machine if you haven't done yet." << endl;
      cout << "<CR> to proceed" << endl;
      char s[100];
      cin >> s;
  }
  return rc;
}

/***
   * fork an EDC process with its standard input connected to the pipe pfd
   *
   ***/

int fork_EDC(int pfd[2])
{
  char* slt_dom=setget_slt_dom();
  
  int proc;
  switch(proc=fork()) {
  case -1:
    arteMsg << "sltsim.C: error forking EDC" ;
    arteMsg.warn();
    return -1;
  case 0:
    if(std::close(0)==-1){arteMsg << "sltsim.C: error closing fd 0"; arteMsg.warn();}
    if(dup(pfd[0])!=0){arteMsg << "sltsim.C: dup error "; arteMsg.warn();}
    if (std::close(pfd[0])==-1||std::close(pfd[1])==-1) {
      arteMsg << "sltsim.C: error closing pfds";arteMsg.warn();
    }
    //    if (! kasltsim::get_debuglevel()) std::close(STDOUT_FILENO);
    
    char *domain=new char[80];
    strcpy(domain,"-Domain:");
    domain=strcat(domain,slt_dom);
    
    char *SLPdbg="-SLPdbg:0";
    switch (kasltsim::get_debuglevel()) {
    case 0:
    case 1: SLPdbg="-SLPdbg:0"; break;
    case 2: SLPdbg="-SLPdbg:1"; break;
    case 3: SLPdbg="-SLPdbg:-1";
    }
    
    char ecalcalib[200];
    //This does not work, I need the right CmdCareECDB object
    //CmdCareECDB boardfile;
    //sprintf(ecalcalib,"-ECALCALIB:%s",boardfile.calibfile());
    char *edc_cal = getenv("EDC_CAL");
    if (edc_cal==0) edc_cal="ECAL/ecalcalibmc0999.dat";
    sprintf(ecalcalib,"-ECALCALIB:%s",edc_cal);
    
    char *edc_pipeout = "-EDCpipeout:1";
    
    char *plbtrigtype;
    Trig_Sym_major_minor ts=get_major_minor(kasltsim::get_runnumber());
    if ((ts.tt_major==DI_LEPTON_FLT)||(ts.tt_major==SINGLE_LEPTON_FLT))
      plbtrigtype="-PlbTrigType:0";
    else 
      plbtrigtype="-PlbTrigType:1";
    
    cout << "About to fork "
	 << kasltsim::get_edcpath()
	 << kasltsim::get_edcfile() 
	 << " with " << plbtrigtype << endl;
    
    char sltprogname[800]="";
    strcat(sltprogname,kasltsim::get_edcpath());
    strcat(sltprogname,kasltsim::get_edcfile());
    execlp(sltprogname,
	   kasltsim::get_edcfile(),
//  	   "-EDChbook:gut508.his",
//	   "-EDCevts:10000",
	   "-SMCPNM:~/RC/top",
	   "-SMCSTT:10",
	   domain ,
	   "-SLPevc:MON_0",
	   "-Process:SLP/_0",
	   "-EVAfile:",
	   "-SLPerrpost:0",
	   SLPdbg,
	   "-SLPnodist:1",
	   "-SLPseglen:200000",
	   "-EVAdiag:0",
	   "-EVAformat:raw",
	   "-EDCnoRND",
	   "-EDCtarg",
	   "-EDCreadpipe",
	   "-SLPscn:scn",
	   "+Pvds-itr.iol",
	   "-SLPNoSMC",
	   "-SLPNoSCN",
	   ecalcalib,
	   "-EDCmc:2",
	   "-CARE_noHotCl:0",
	   "-SLT_EcalIniMode:1",
	   "-SLT_geoIniMode:3",  // 2 = ASCII, 1 = bin, 0 = CnA, 3 = pipe
	   "-SLT_geoIniFile:l2in_mc.geo",
	   "-SLT_mapIniFile:l2in_mc.map",
	   "-SLT_otrIniFile:l2bin_mc.otr",
	   "-TGStrigMode:3",     // 2 = ASCII, 1 = bin, 0 = CnA, 3 = pipe
	   "-TGStrigFile:l2tt.txt",
	   "-TGSsymFile:l2st.txt",
	   plbtrigtype,
	   edc_pipeout,
	   0);
    cerr<<"SLTsim: internal ERROR"<<endl;
    exit(1);
  } 
  sltsim_AddProcess(proc);
  return 0;
}

/***
   * send the event to the SLT simulation
   *
   ***/

void sltsim(EVENT_HDR* event) {

  static bool first=true;
  int stat;

  if (first) {
    first=false;
    if ( (stat=DAQ_MC_IF::init_pipe())!=0 ) {
      arteMsg << "sltsim: DAQ_MC_IF::init_pipe returned " << stat;;
      arteMsg.warn();
    }
    cout <<" init_pipe done -now write event"<<endl;
    if ( (stat=DAQ_MC_IF::writeEvent(event))!=0 ) { //first event used for setup
      arteMsg << "sltsim: DAQ_MC_IF::writeEvent returned " << stat;
      arteMsg.warn();
    }
    L2FormArte();    // pass geo/map/tt/st after first event 
  }
  if ( (stat=DAQ_MC_IF::writeEvent(event))!=0 ) { //first event sent again 
      arteMsg << "sltsim: DAQ_MC_IF::writeEvent returned " << stat;
      arteMsg.warn();
  }
  SLTPipeToDSLT(); // Get result back
}

/***
   * Keep track of forked processes - could be implemented as a class?
   *
   ***/

static int subprocess[10];
static int proccnt=0;

void sltsim_KillProcess() {
  // kill SLT processes
  for(int i=0;i<proccnt;i++) {
    if (i==0) // namserver needs to unregister, so kill softly!!!
      kill(subprocess[i],SIGINT);
    else
      kill(subprocess[i],SIGKILL);
    //    cout << "killed process " << subprocess[i] << endl;
  }
  //unlink FIFO
  char* slt_dom=setget_slt_dom();
  if (slt_dom[0]=='/')  slt_dom = slt_dom + 1;
  if (slt_dom[strlen(slt_dom)-1]=='/')  slt_dom[strlen(slt_dom)-1]='\0';
  char fifo[80];
  sprintf(fifo,"/tmp/fifo_%s",slt_dom);
  if (unlink(fifo)!=0) {
    arteMsg << "sltsim_KillProcess: Could not unlink " << fifo ;
    arteMsg.warn();
  }
}

void sltsim_AddProcess(int num) {
  if (num<1) return;
  if (proccnt==0) // make sure process are killed when Arte quits
    atexit(sltsim_KillProcess);
  if (proccnt<10) {
    subprocess[proccnt++]=num;
  }
  else {
    arteMsg <<"sltsim: Too many processes forked!";
    arteMsg.warn();
  }
}

/***
   * Look up whether the event has been accepted after sltsim
   *
   ***/

bool sltaccept()
 {
  if (SLTReadDSLT()!=SLT_OK) return false;
  SLT_STATUS stat;
  // Only good events
  if (SLTStatus(&stat)!=SLT_OK) return false;
  if (!(stat.status&SLTEVENT_ACCEPT)) return false;
  return true;
 }
