!-----------------------------------------------------------------------
! Track finding by Expectation Maximisation Algorithm - tema
! M. Staric, sep-2000, feb-2001
!
!-----------------------------------------------------------------------


      SUBROUTINE TEMA_RECO(OK)

      IMPLICIT NONE
      LOGICAL OK                ! return status (T = reco is done)

#include "tema/TEMA_PAR.fi"
#include "tema/TEMA_HITS.fi"
#include "tema/TEMA_RSEG.fi"
#include "tema/TEMA_HSEG.fi"

      integer MAXNB
      PARAMETER (MAXNB=1 000 000)
      INTEGER NB
      INTEGER IX(MAXNB)

      INTEGER NGR
      LOGICAL START /.TRUE./
      SAVE START

      IF(START) THEN
         START=.FALSE.
         CALL TEMA_INIT
      ENDIF

      NN_HS=0
      NSEG=0
      CALL TEMA_GET_PC_HITS(OK)
      IF(.NOT.OK) RETURN
      IF(NHIT(1).LT.3) RETURN

      NGR=0
      IF(USE_OTR.AND.NHIT_OTR.GT.0) THEN
         CALL TEMA_ITER_HOR(0,NGR,MAXNB,IX,NB,OK)
         IF(.NOT.OK) RETURN
      ENDIF
      IF(USE_ITR.AND.NHIT_ITR.GT.0) THEN
         CALL TEMA_ITER_HOR(1,NGR,MAXNB,IX,NB,OK)
         IF(.NOT.OK) RETURN
      ENDIF

      CALL TEMA_FIT_HOR
      IF(NN_HS.EQ.0) RETURN

      CALL TEMA_ITER_VERT(MAXNB,IX,NB,OK)
      IF(.NOT.OK) RETURN

      CALL TEMA_FIT_VERT

      CALL TEMA_FIT_3D

      CALL TEMA_PUT_RSEG

      RETURN
      END



      SUBROUTINE TEMA_INIT

#include "tema/TEMA_PAR.fi"
      CHARACTER*80 TXT

      DATA Z_CENTER /780./
      DATA WMA_MIN /.1/
      DATA NITER_HOR /8/
      DATA NITER_VER /4/
      DATA USE_DRIFT /.TRUE./
      DATA USE_SIGME /.FALSE./
      DATA SIG_CELL  /.1644, .3075/
      DATA SIG_DRIFT /200.E-4, 200.E-4/
      DATA SIG_ITR /100.E-4/
      DATA USE_ITR /.TRUE./
      DATA USE_OTR /.TRUE./

      CALL MEALLP('-- TEMA started')
      WRITE(TXT,*,IOSTAT=IO) '        Z_CENTER = ',Z_CENTER
      CALL MEALLP(TXT)
      WRITE(TXT,*,IOSTAT=IO) '         WMA_MIN = ',WMA_MIN
      CALL MEALLP(TXT)
      WRITE(TXT,*,IOSTAT=IO) '       NITER_HOR = ',NITER_HOR
      CALL MEALLP(TXT)
      WRITE(TXT,*,IOSTAT=IO) '       NITER_VER = ',NITER_VER
      CALL MEALLP(TXT)
      WRITE(TXT,*,IOSTAT=IO) '       USE_DRIFT = ',USE_DRIFT
      CALL MEALLP(TXT)
      WRITE(TXT,*,IOSTAT=IO) '       USE_SIGME = ',USE_SIGME
      CALL MEALLP(TXT)
      IF(USE_SIGME) THEN
         IF(USE_DRIFT) THEN
            WRITE(TXT,*,IOSTAT=IO) '           SIGMA = ',SIG_DRIFT
         ELSE
            WRITE(TXT,*,IOSTAT=IO) '           SIGMA = ',SIG_CELL
         ENDIF
         CALL MEALLP(TXT)
      ENDIF
      WRITE(TXT,*,IOSTAT=IO) '         USE_ITR = ',USE_ITR
      CALL MEALLP(TXT)
      WRITE(TXT,*,IOSTAT=IO) '         USE_OTR = ',USE_OTR
      CALL MEALLP(TXT)
      CALL MEALLP('---------------')

      RETURN
      END


!-----------------------------------------------------------------------
! Modification of parameters in /TEMA_PAR/ with kuip interface
! M. Staric, mar-2000
!----------------------------------------------------------------------- 

      SUBROUTINE TEMA_Z_CENTER

#include "tema/TEMA_PAR.fi"

      CALL KUGETR(Z_CENTER)
      RETURN
      END


      SUBROUTINE TEMA_WMA_MIN

#include "tema/TEMA_PAR.fi"

      CALL KUGETR(WMA_MIN)
      RETURN
      END


      SUBROUTINE TEMA_NITER

#include "tema/TEMA_PAR.fi"

      CALL KUGETI(NITER_HOR)
      CALL KUGETI(NITER_VER)
      RETURN
      END


      SUBROUTINE TEMA_USE_DRIFT

#include "tema/TEMA_PAR.fi"
      CHARACTER*80 CC
      INTEGER L

      CALL KUGETC(CC,L)
      IF(L.GT.0) CALL CUTOL(CC)
      IF(CC(1:L).EQ.'on') THEN
         USE_DRIFT=.TRUE.
      ELSE IF(CC(1:L).EQ.'off') THEN
         USE_DRIFT=.FALSE.
      ELSE
         PRINT*,'**** ERROR in TEMA/DRIFT:  Unavailable option'
      ENDIF

      RETURN
      END


      SUBROUTINE TEMA_USE_ITR

#include "tema/TEMA_PAR.fi"
      CHARACTER*80 CC
      INTEGER L

      CALL KUGETC(CC,L)
      IF(L.GT.0) CALL CUTOL(CC)
      IF(CC(1:L).EQ.'on') THEN
         USE_ITR=.TRUE.
      ELSE IF(CC(1:L).EQ.'off') THEN
         USE_ITR=.FALSE.
      ELSE
         PRINT*,'**** ERROR in TEMA/ITR:  Unavailable option'
      ENDIF

      RETURN
      END


      SUBROUTINE TEMA_USE_OTR

#include "tema/TEMA_PAR.fi"
      CHARACTER*80 CC
      INTEGER L

      CALL KUGETC(CC,L)
      IF(L.GT.0) CALL CUTOL(CC)
      IF(CC(1:L).EQ.'on') THEN
         USE_OTR=.TRUE.
      ELSE IF(CC(1:L).EQ.'off') THEN
         USE_OTR=.FALSE.
      ELSE
         PRINT*,'**** ERROR in TEMA/OTR:  Unavailable option'
      ENDIF

      RETURN
      END



      SUBROUTINE TEMA_SIGMAS

#include "tema/TEMA_PAR.fi"

      CALL KUGETR(SIG_CELL(0))
      CALL KUGETR(SIG_CELL(1))
      CALL KUGETR(SIG_ITR)
      RETURN
      END





