#ifndef __WAX_HIT_HH__
#define __WAX_HIT_HH__

#ifdef USE_ARTE
 #ifndef __ARTE_HH__
 #include "arte/arte.hh"
 #endif
 typedef ArtePointer<HITB> ArtePointerHITB ;
#endif
#ifndef __WAX_DECODE_MANAGER_HH__
#include "wax/DecodeManager.hh"
#endif
#ifndef __WAX_RTMANAGER_HH__
#include "wax/RTManager.hh"
#endif

class std::ostream ;

class waxHit
{
public:
  // constructors, destructors, assignment
  waxHit() : fChannel(0) {}
  waxHit(const waxDecodeChannel* chan) : fChannel(chan) {}
  virtual ~waxHit() {}

  // forwarded from the channel
  TdcCount_t  GetTdcCount()   const { return fChannel->GetTdcCount(); }
  TdcCount_t  GetTZeroCount() const { return fChannel->GetTZeroCount(); }
  Status_t    GetStatus()     const { return fChannel->GetStatus(); }
  GedeIndex_t GetGedeIndex()  const { return fChannel->GetGedeIndex() ; }
  WireIndex_t GetWireIndex()  const { return fChannel->GetWireIndex() ; }
  DriftTime_t GetTZero()      const { return fChannel->GetTZero() ; }
  DriftTime_t GetDriftTime()  const { return fChannel->GetDriftTime() ; }
  AsdIndex_t  GetAsdIndex()   const { return fChannel->GetAsdIndex() ; }
  ChannelIndex_t GetChannelIndex() const { return fChannel->GetChannelIndex() ; }

  // decoding the geometry  
  const ArtePointer<GEDE> GetGede() const {
    return ArtePointer<GEDE>(GetGedeIndex()) ; }
  const waxRTRelation* GetRTRelation() {
    return waxRTManager::GetUncheckedMe()->
      GetRTRelationFromGedeIndex(GetGedeIndex()) ; }

//   float GetRawX() const { return GetGede()->GetRawX(GetWireIndex()) ; }
//   float GetRawY1() const { return GetGede()->GetRawY1(GetWireIndex()) ; }
//   float GetRawY2() const { return GetGede()->GetRawY2(GetWireIndex()) ; }
//   float GetRawZ() const { return GetGede()->GetRawZ(GetWireIndex()) ; }
//   int GetMonoLayer() const { 
//     return GetWireIndex()%(GetGede()->GetNMonoLayers()) ; }

  // arte interface
#ifdef USE_ARTE
  ArtePointerHITB GetHITB() const { return fHITB; } ;
  void SetHITB(ArtePointerHITB hitb) { fHITB = hitb ; } ;
  ArtePointerHITB AppendHITB();// append this hit to the HITB bank
  void AppendHITB(ArtePointerHITB hitb) ;
  ArtePointer<DIGB> GetDIGB() const ;

  static int GetCHID(WireIndex_t wireId, AsdIndex_t asdId,ChannelIndex_t chId)
    { return wireId + 128*(chId + 16*asdId) ; }
  static WireIndex_t GetWireIndex(int CHID) { return CHID%128 ; }
  static ChannelIndex_t GetChannelIndex(int CHID) { return (CHID/128)%16 ; }
  static AsdIndex_t GetAsdIndex(int CHID) { return CHID/(128*16) ; }
  static float GetDriftRadiusOffset() { return fgDriftRadiusOffset ; }
  static void SetDriftRadiusOffset(float r) { fgDriftRadiusOffset = r ; }
  
  //   static float GetHITBY(TdcCount_t t, TdcCount_t tzero, Status_t status) { 
  //     return t + 256*(tzero+256*status) ; }
  //   static TdcCount_t GetTdcCount(float hitby) {
  //     return (int(hitby))%256 ; }
  //   static TdcCount_t GetTZeroCount(float hitby) {
  //     return ((int(hitby))/256)%256 ; }
  //   static Status_t GetStatus(float hitby) {
  //     return (int(hitby))/256 ; }
  //   static DriftTime_t GetDriftTime(float hitby) {
  //     return waxDecodeChannel::GetDriftTime(GetTdcCount(hitby),GetTZeroCount(hitby)) ; }
#endif

private:
  const waxDecodeChannel* fChannel ;
#ifdef USE_ARTE
  ArtePointerHITB fHITB ; // pointer to arte hitb bank
#endif
  static float fgDriftRadiusOffset ;
} ;

#endif
